/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2000  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h> 
#include <strings.h> 
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <time.h>
#include <signal.h>

#include <gossip.h>

static int endian_flag=1;
static char *little_endian=(char *)&endian_flag;
static int must_init_signal=1;

/* size of socket buffers in KiloBytes */
#define SOCK_BUF_SIZE 256

/* write hostname:port_number into channel description file */
/* hostname normally in IPV4 notation xx.yy.zz.tt           */
int set_host_and_port(char *channel_file, char *host_and_port)  /*   %ENTRY%   */
{
     int fserver;
     char buf[1024];
     int nc;

     if(strncmp(channel_file,"Anonym",6)==0) return(0) ; /* anonymous channel created */

     /* $HOME/.broker/channel_name is file name for channel descriptor */
     snprintf(buf,1023,"%s/.broker/%s",getenv("HOME"),channel_file);

     if((fserver=open(buf,O_WRONLY+O_CREAT,0700)) == -1) { 
       fprintf(stderr,"Can't open or create Channel Description file\n");
       return(-1);
     };
     nc=snprintf(buf,1023,"%s\n",host_and_port);
     if(write(fserver,buf,nc)<=0) {
       fprintf(stderr,"Can't write into Channel Description file\n");
       return(-1);
     }
     close(fserver);
     return(0);
}

/* read hostname:port_number from channel description file 
   return pointer to character string upon success
   return NULL pointer in case of failure
*/
char *get_host_and_port(char *channel_file)  /*   %ENTRY%   */
{
     char *chan_buf=malloc(1024);
     int fserver;
     char buf[1024];

     snprintf(buf,1023,"%s/.broker/%s",getenv("HOME"),channel_file);

     if((fserver=open(buf,O_RDONLY)) == -1) {
       fprintf(stderr,"Can't open Channel Description file\n");
       return(NULL) ; /* exit(1); */
     };
     if(read(fserver,chan_buf,1024)<=0) {
       fprintf(stderr,"Can't read Channel Description file\n");
       close(fserver);
       return(NULL) ; /* exit(1); */
     }
     close(fserver);
     if(index(chan_buf,'\n')){
       *index(chan_buf,'\n') = '\0' ;
     } else {
       fprintf(stderr,"Invalid Channel Description file\n");
       return(NULL) ; /* exit(1); */
     }
     return(chan_buf);
}

/* get Authorization token from file $HOME/.broker/.Bauth  
   return pointer to character string upon success
   return NULL pointer in case of error
*/
char *get_broker_Authorization()  /*   %ENTRY%   */
{
     char *auth_buf=malloc(1024);
     int fserver;
     char buf[1024];
     char *homedir=getenv("HOME");

     snprintf(buf,1023,"%s/.broker",homedir);
     if(chmod(buf,0711)){
       fprintf(stderr,"Improper permissions for broker directory %s\n",buf);
       return(NULL) ; /* exit(1); */
     }

     snprintf(buf,1023,"%s/.broker/.Bauth",getenv("HOME"));

     if(chmod(buf,0600)){
       fprintf(stderr,"Improper permissions for Authorization file\n");
       return(NULL) ; /* exit(1); */
     }
     if((fserver=open(buf,O_RDONLY)) == -1) {
       fprintf(stderr,"Can't open Authorization file\n");
       return(NULL) ; /* exit(1); */
     };
     if(read(fserver,auth_buf,1024)<=0) {
       fprintf(stderr,"Can't read Authorization file\n");
       close(fserver);
       return(NULL) ; /* exit(1); */
     }
     close(fserver);
     if(index(auth_buf,'\n')){
       *index(auth_buf,'\n') = '\0' ;
     } else {
       fprintf(stderr,"Invalid Authorization file\n");
       return(NULL) ; /* exit(1); */
     }
     return(auth_buf);
}

/* write Authorization token into file $HOME/.broker/.Bauth */
void set_broker_Authorization(int auth_token)  /*   %ENTRY%   */
{
     int fserver;
     char buf[1024];
     int nc;

     snprintf(buf,1023,"%s/.broker/.Bauth",getenv("HOME"));
     if((fserver=open(buf,O_WRONLY)) == -1) {
       fprintf(stderr,"Can't open Authorization file\n");
       exit(1);
     };
     nc=snprintf(buf,1023,"%d\n",auth_token);
     write(fserver,buf,nc+1);
     close(fserver);
}

static struct  sockaddr_in server;                /* server socket */
static int sizeserver = sizeof server;

/* accept connections on the bound server socket return socket for incoming connection */
/* bind_sock_to_port must have been called before connection can be accpeted           */
int accept_from_sock(int fserver)  /*   %ENTRY%   */
{
    int fclient =  accept(fserver,(struct  sockaddr *)&server, &sizeserver);
    if(fclient < 0) {
      fprintf(stderr,"Accept failed!\n");
      return(-1);
      }
    return fclient;
}

/* bind an existing socket to a free (automatic) port, return port number */
/* existing socket usually created by get_sock_net                        */
int bind_sock_to_port(int s)  /*   %ENTRY%   */
{

     struct sockaddr_in server_eff;
     int sizeserver_eff = sizeof server_eff ;

     server.sin_family= AF_INET;
     server.sin_port= htons(0);
     server.sin_addr.s_addr= INADDR_ANY;

     if(bind(s,(struct  sockaddr *)&server,sizeserver)<0)
          {
          fprintf(stderr,"Bind failed! \n");
          return(-1);
          }
     getsockname(s,(struct  sockaddr *)&server_eff,&sizeserver_eff);
     return ntohs(server_eff.sin_port);
}

/* create a network socket ; return socket descriptor */
int get_sock_net()  /*   %ENTRY%   */
{
/*   ignore SIGPIPE signal (i.e. do no abort but return error) */

     if(must_init_signal) {  /* DO THIS ONLY ONCE */

       signal(SIGPIPE,SIG_IGN);
       must_init_signal=0;

     }
     return socket(AF_INET, SOCK_STREAM, 0);
}

/* set buffer sizes (recv and send) for a newly created socket (always returns 0) */
int set_sock_opt(int s)  /*   %ENTRY%   */
{
     int optval, optsize;
     int b0=0;

     optval=SOCK_BUF_SIZE*1024;
     b0=setsockopt(s,SOL_SOCKET,SO_SNDBUF,(char *)&optval,sizeof(optval));
     if(b0!=0) { fprintf(stderr,"Error setting SO_SNDBUF size \n"); }
     optval=0;optsize=4;
     getsockopt(s,SOL_SOCKET,SO_SNDBUF,(char *)&optval,&optsize);
#ifdef DEBUG
     fprintf(stderr,"SO_SNDBUF=%d, optsize=%d\n",optval,optsize);
#endif

     optval=SOCK_BUF_SIZE*1024;
     b0=setsockopt(s,SOL_SOCKET,SO_RCVBUF,(char *)&optval,sizeof(optval));
     if(b0!=0) { fprintf(stderr,"Error setting SO_RCVBUF size \n");}
     optval=0;optsize=4;
     getsockopt(s,SOL_SOCKET,SO_RCVBUF,(char *)&optval,&optsize);
#ifdef DEBUG
     fprintf(stderr,"SO_RCVBUF=%d, optsize=%d\n",optval,optsize);
#endif

     return(0);
}

/* obtain the IPV4 adress of a host specified by name */
int get_ip_address(char *hostname)  /*   %ENTRY%   */
{
     int **addr_list;
     struct hostent *answer;
     int ipaddr=0;
     int b0,b1,b2,b3;

     if(NULL==(answer=gethostbyname(hostname))) {
       fprintf(stderr,"Cannot get address for host=%s\n",hostname);
       return(-1);
       }

     addr_list=(int **)answer->h_addr_list;
     ipaddr=ntohl(**addr_list);
     b0 = ipaddr >> 24; b1 = ipaddr >> 16 ; b2 = ipaddr >> 8 ; b3 = ipaddr;
     b0 &= 255;
     b1 &= 255;
     b2 &= 255;
     b3 &= 255;
#ifdef DEBUG
     fprintf(stderr,"IP address of %s=%d.%d.%d.%d \n",hostname,b0,b1,b2,b3);
#endif
     return(ipaddr);
}

/* obtain own host's IPV4 address 
*/
int get_own_ip_address()  /*   %ENTRY%   */
{
     char buf[1024];
     if(gethostname(buf,sizeof buf)){
       fprintf(stderr,"Can't find hostname\n");
       return(-1);
       }
     return get_ip_address(buf);

}

/* given a [host:]port specification, connect to it
   if host: is not specified, use localhost
   the return value is the connected socket
*/
int connect_to_hostport(char *target)  /*   %ENTRY%   */
{
     char buf[1024];
     int n;
     int fserver;
     int b0,b1,b2,b3;
     int ipaddr;
     char *portno;
     int optval=0;
     int optsize=sizeof optval;

     struct sockaddr_in server;
     int sizeserver = sizeof server;

     if(NULL==strstr(target,":")) {      /* no host specified, use local host */
       portno=target;
       if(gethostname(buf,sizeof buf)){
         fprintf(stderr,"Can't find hostname\n");
         return(-1);
       } 
       ipaddr=get_ip_address(buf);
     }
     else {         /* use specified host, find address */
       portno=strstr(target,":"); *portno='\0' ; portno++;
       ipaddr=get_ip_address(target);
     }

     b0 = ipaddr >> 24; b1 = ipaddr >> 16 ; b2 = ipaddr >> 8 ; b3 = ipaddr;
     b0 &= 255;
     b1 &= 255;
     b2 &= 255;
     b3 &= 255;
     snprintf(buf,1023,"%d.%d.%d.%d",b0,b1,b2,b3);
#ifdef DEBUG
     fprintf(stderr,"connecting to %d.%d.%d.%d:%s\n",b0,b1,b2,b3,portno);
#endif

     fserver = socket(AF_INET, SOCK_STREAM, 0);

     set_sock_opt(fserver);

     server.sin_family= AF_INET;
/*
     server.sin_port= htons(1077);
     server.sin_addr.s_addr= INADDR_ANY;
*/
     server.sin_port= htons(atoi(portno));
     server.sin_addr.s_addr= inet_addr(buf);

     if(connect(fserver,(struct  sockaddr *)&server,sizeserver)<0) 
          {
          fprintf(stderr,"Connect failed! \n");
          return(-1);
          }

     return(fserver);
}

/* connect to a port on local host return socket descriptor
*/
int connect_to_localport(int port)  /*   %ENTRY%   */
{
     struct sockaddr_in server;
     int sizeserver = sizeof server;
     int fserver;

     fserver = socket(AF_INET, SOCK_STREAM, 0);
     server.sin_family= AF_INET;
     server.sin_port= htons(port);
     server.sin_addr.s_addr= INADDR_ANY;

     if(connect(fserver,(struct  sockaddr *)&server,sizeserver)<0) 
          {
          fprintf(stderr,"Connect to local port %d failed! \n",port);
          return(-1);
          }

     return(fserver);
}

/* bind a server port to a local port, return hostname:port string 
   as well as socket descriptor. bind to any free port
*/
int bind_to_localport(int *port, char *buf, int maxbuf)  /*   %ENTRY%   */
{
     int fserver,ipaddr,b0,b1,b2,b3,server_port,junk;

/*   get a socket */
     fserver = get_sock_net();

/*   set buffer sizes for socket */
     junk=set_sock_opt(fserver);

/*   bind to a free port, get port number */
     server_port = bind_sock_to_port(fserver);
     *port = server_port;

/*   write host:port into buffer and return the socket file descriptor */
     ipaddr=get_own_ip_address();                    /* get own IPV4 address as 32 bit integer */
     b0 = ipaddr >> 24; b1 = ipaddr >> 16 ; b2 = ipaddr >> 8 ; b3 = ipaddr;
     b0 &= 255; b1 &= 255; b2 &= 255; b3 &= 255;     /* split IPV4 address */
     snprintf(buf,maxbuf,"%d.%d.%d.%d:%d",b0,b1,b2,b3,server_port);

     return(fserver);
}

/* send reply to command, ACK if status=0, NACK if status nonzero */
void send_ack_nack(int fclient,int status)  /*   %ENTRY%   */
{
   if(status)
     write(fclient,"NACK\0",5);
   else
     write(fclient,"ACK\0",4);
}

/* get reply to command from server 0=ACK, nonzero=NACK*/
int get_ack_nack(int fserver)  /*   %ENTRY%   */
{
   char reply[1024];
   int n;
   n = read(fserver, reply, sizeof reply);
   reply[n]='\0';
   return(strncmp(reply,"ACK",3));
}

/* send command and return 0 or -1 upon ACK or NACK */
int send_command_to_server(int fserver, char *buf)  /*   %ENTRY%   */
{

/* send command to server */
   write(fserver,buf,strlen(buf));

/* and check reply */
   if(get_ack_nack(fserver)) { fprintf(stderr,"command rejected,\n"); return(-1); }
   else { return(0);}

}

/* get/put integer through a socket in network (BIG-ENDIAN) order */
/* can also be used for a float as long as sizeof(float) is equal */
/* to sizeof(int) and both are equal to 4                         */
int get_int32_from_channel(int channel)  /*   %ENTRY%   */
{
  int to_receive;

  read(channel,&to_receive,sizeof(to_receive));
  if(*little_endian) {
    swap_4(to_receive);
  }
  return(to_receive);
}
void put_int32_to_channel(int channel, int to_send)  /*   %ENTRY%   */
{
  int to_be_sent=to_send;

  if(*little_endian) {
    swap_4(to_be_sent);
  }
  write(channel,&to_be_sent,sizeof(to_be_sent));
}

/* connect to channel_name and send LOGIN sequence */
/*         channel name has 2 forms :              */
/*                               1- name           */
/*                               2- @hostname:port */
/* login sequence has the following syntax         */
/* LOGIN uid pid Auth_token hostname               */
int connect_to_channel_by_name(char *name)  /*   %ENTRY%   */
{
     char *Auth_token=get_broker_Authorization();
     int Bauth_token=0xFFFFFFFF;
     char buf[1024];
     char host_name[1024];
     int fserver;
     int n;

     if(Auth_token == NULL) {
       fprintf(stderr,"Authorizartion token failure \n");
       return(-1);
     }
     if(gethostname(host_name,sizeof host_name)){
       fprintf(stderr,"Can't get local hostname\n");
       return(-1);
     }

    /* connect to data server by channel_name or @host:port */
     if ( *name == '@' ) {
       name++;
       fprintf(stderr,"Connecting to %s\n",name);
       fserver=connect_to_hostport(name);
       if(fserver < 0) return(-1);
     } else {
       char *host_and_port=get_host_and_port(name);
       if(host_and_port == NULL) return(-1);
       fprintf(stderr,"Opening channel %s to %s\n",name,host_and_port);
       fserver=connect_to_hostport(host_and_port);
       free(host_and_port);
       if(fserver < 0) return(-1);
     }

/*   send LOGIN command to server */
     sscanf(Auth_token,"%u",&Bauth_token);
     free(Auth_token);

     snprintf(buf,1023,"%s %d %d %u:%s:","LOGIN", getuid(), getpid(), Bauth_token, host_name);
     if(send_command_to_server(fserver,buf)) {
       fprintf(stderr,"LOGIN rejected\n");
       return(-1);
     } else {
       fprintf(stderr,"LOGIN accepted\n");
       return(fserver);
     }
}

/* dummy client, black hole */
static int black_hole(int client_uid,int client_auth,int from_client,int to_client,char * buf)
{
  int buflen;
  char iobuf[65536]; int recvd; int to_receive;
  int nbuf=0;
  
  fprintf(stderr,"Black Hole called with command:%s\n",buf);
  to_receive=get_int32_from_channel(from_client);
  fprintf(stderr,"Starting receive for %d bytes\n",to_receive);
  recvd=0;
  while((buflen=read(from_client, iobuf, sizeof iobuf)) > 0 ) {
    nbuf++;
    recvd+=buflen;
    reset_timeout_counter();
    if(recvd>=to_receive)break;
  }
  fprintf(stderr,"server received from client %d chars in %d buffers \n",recvd,nbuf);
  return(0);
}

/* basic server code, forking variety ( as opposed to thread variety) */
void gossip_fork_server(char *LOGFILE, char *channel, int (*user_client)(int,int,int,int,char *), int PING_INTERVAL, int from_inetd)   /*   %ENTRY%   */
{
    char *Auth_token;
    unsigned int Bauth_token;
    char buf[1024];
    char buf2[1024];
    int from_client, to_client, fserver;
    int ping_pid=0;
    int client_uid, client_pid, client_auth;
    int buflen;
    int server_port=0;
    int child_pid=0;
    int child_status=0;
    int myuid=getuid();

    if(from_inetd){     /* inetd / xinetd mode */

      Auth_token=NULL;
      Bauth_token=0;
      fserver = -1;
      from_client=0;         /* stdin */
      to_client=1;           /* stdout */

      buflen = read(from_client, buf, sizeof buf);    /* get LOGIN command */
      buf[buflen]='\0';

      close(2);    /* close and reopen STDERR */
      freopen(LOGFILE,"a",stderr);

      client_uid = -1 ;
      client_pid = -1 ;
      client_auth = -1 ;
      sscanf(buf,"LOGIN %d %d %u",&client_uid,&client_pid,&client_auth);

      send_ack_nack(to_client,     /* send ACK/NACK according to client code */
           (*user_client)(client_uid,client_auth,from_client,to_client,buf)  );
      close(from_client);
      close(to_client);
      exit(0);
    }else{                   /* server mode, bind to port, get server socket */

      close(0);              /* reopen stdin on /dev/null */
      freopen("/dev/null","r",stdin);
      Auth_token=get_broker_Authorization();
      sscanf(Auth_token,"%u",&Bauth_token);

      fserver=bind_to_localport(&server_port, buf, 1023);
      if(gethostname(buf2,sizeof buf2)){
        fprintf(stderr,"Can't find hostname\n");
        exit(1);
      }
      from_client=-1;
      to_client=from_client;

      printf("@%s:%d",buf2,server_port);
      fflush(stdout);
      if( set_host_and_port(channel,buf) ) exit(1) ;

      close(1); close(2);    /* close and reopen STDOUT and STDERR */
      freopen(LOGFILE,"a",stdout);
      freopen(LOGFILE,"a",stderr);


      if( fork() >0 ) exit(0);   /* parent exits now if regular server */

/*    set NO BUFFERING for STDOUT and STDERR */
      setvbuf(stdout,NULL,_IONBF,0);
      setvbuf(stderr,NULL,_IONBF,0);

/*    the first child is the actual server */

      listen(fserver,5);
/* ==================================================================================== */
      reset_timeout_counter();
      if(PING_INTERVAL) {
        if( (ping_pid=fork()) ==0 ){
          int watchdog_client;
          int ping_ord=0;
          char pingbuf[128];

          close(fserver);
          sleep(PING_INTERVAL);
          while(watchdog_client=connect_to_localport(server_port)){

/*         send PING command with ping counter                                   */
              sprintf(pingbuf,"PING %d",ping_ord++);
              write(watchdog_client,pingbuf,strlen(pingbuf));
              close(watchdog_client);

              sleep(PING_INTERVAL);
          }
          exit(0);
        } /* endif fork */
      } /* endif PING_INTERVAL */
/* ==================================================================================== */
      /* loop on connection requests */
      while( from_client = accept_from_sock(fserver) ) {
        to_client = from_client;

        while( (child_pid=waitpid(-1,&child_status,WNOHANG)) > 0 )
          fprintf(stderr,"Child pid %d terminated\n",child_pid) ;

        buflen = read(from_client, buf, (sizeof buf)-1);    /* get command */
        buf[buflen]='\0';

        if(strncmp(buf,"PING",4)==0) {                     /* PING ? */
            fprintf(stderr,"%s (%d)\n",buf,get_timeout_counter()); /* echo to logfile */
            close(from_client);
            if(get_timeout_counter() <= 0) {
              fprintf(stderr,"TIMEOUT detected, EXITING \n");
              if(ping_pid>0) kill(ping_pid,9);
              exit(1);
            }
          decrement_timeout_counter();
          continue;   /* go service next connection request */
        }
/*      validate login command, check client uid against server uid,
        verify that authorization token is the right one,
        register client pid as connected if connection accepted      */

        client_uid = -1 ;
        client_pid = -1 ;
        client_auth = -1 ;
        sscanf(buf,"LOGIN %d %d %u",&client_uid,&client_pid,&client_auth);

/*      reject connection if not my uid or bad authorization token or not a LOGIN command */
        if(client_uid!=myuid || Bauth_token!=client_auth){
          fprintf(stderr,"expected client = %d, got %d, expected auth= %d, got %d\n",
                  myuid,client_uid,Bauth_token,client_auth);
          send_ack_nack(to_client,1);  /* Negative ACK, LOGIN rejected */
          close(to_client);
          continue;
        } else {
          fprintf(stderr,"%s\n",buf);
          send_ack_nack(to_client,0);   /* LOGIN accepted, send ACK */
        }

/*      LOGIN sequence is valid, continue processing */

        while( (buflen = read(from_client, buf, sizeof buf))>0 ) {  /* get next command */
          buf[buflen]='\0';
/*        ECHO command */
          if(strncmp(buf,"ECHO",4)==0) {
              fprintf(stderr,"%s\n",buf);          /* echo command to logfile */
              send_ack_nack(to_client,0);    /* send ACK to indicate that command is accepted */
              reset_timeout_counter();     /* reset TIMEOUT counter */
              continue;
/*        END command */
          } else if(strncmp(buf,"END",3)==0) {
              fprintf(stderr,"%s\n",buf);          /* echo command to logfile */
              send_ack_nack(to_client,0);    /* send ACK to indicate that command is accepted */
              close(to_client);
              reset_timeout_counter();     /* reset TIMEOUT counter */
              fprintf(stderr,"END command received, server exiting normally\n");
              if(ping_pid>0) kill(ping_pid,9);
              exit(0);
              break;                    /* connection terminated, next client please */
/*        EXEC command */
          } else if( strncmp(buf,"EXEC",4)==0 ) {
              fprintf(stderr,"%s\n",buf);          /* echo command to logfile */
              send_ack_nack(to_client,     /* send ACK/NACK according to client code */
                (*user_client)(client_uid,client_auth,from_client,to_client,buf)  );
              reset_timeout_counter();     /* reset TIMEOUT counter */
              continue;
/*        STUFF command */
          } else if( strncmp(buf,"STUFF",5)==0 ) {
              fprintf(stderr,"%s\n",buf);          /* echo command to logfile */
              send_ack_nack(to_client,0);    /* send ACK to indicate that command is accepted */
              send_ack_nack(to_client,     /* send ACK/NACK according to client code */
                black_hole(client_uid,client_auth,from_client,to_client,buf)  );
              reset_timeout_counter();     /* reset TIMEOUT counter */
              continue;
/*        FORK command */
          } else if( strncmp(buf,"FORK",4)==0 ) {
              fprintf(stderr,"%s\n",buf);          /* echo command to logfile */
              send_ack_nack(to_client,0);    /* send ACK to indicate that command is accepted */
              if(fork()==0){  /* child */
                send_ack_nack(to_client,     /* send ACK/NACK according to client code */
                  (*user_client)(client_uid,client_auth,from_client,to_client,buf) );
                close(to_client);
                exit(0);
              }else{  /* parent */
                close(to_client);
              }
              reset_timeout_counter();     /* reset TIMEOUT counter */
              break;
/*        FLOOD command */
          } else if( strncmp(buf,"FLOOD",5)==0 ) {
              fprintf(stderr,"%s\n",buf);          /* echo command to logfile */
              send_ack_nack(to_client,0);    /* send ACK to indicate that command is accepted */
              if(fork()==0){  /* child */
                send_ack_nack(to_client,     /* send ACK/NACK according to client code */
                  black_hole(client_uid,client_auth,from_client,to_client,buf) );
                close(to_client);
                exit(0);
              }else{  /* parent */
                close(to_client);
              }
              reset_timeout_counter();     /* reset TIMEOUT counter */
              break;
/*        UNRECOGNIZED command, reject and close connection  */
          } else { /* assumed to be a bum call */
              fprintf(stderr,"Bad Command=%s\n",buf);
              send_ack_nack(to_client,1);  /* send NACK to indicate that command is rejected */
              continue;
          }
        } /* end while get next command */

        if(buflen == 0) close(to_client);

      }   /* end of loop on connection requests */
      fprintf(stderr,"Accept failed , exiting \n");
      if(ping_pid>0) kill(ping_pid,9);
      exit(1);
    }
}
/* test function */
/* issue a FLOOD command and send n junk bytes to server */
/* used for test purposes only */
void send_junk_to_server(int fserver,int n)
{
       char buf[4096*1024];
       int nsent;
       int nbuf=0;
       int nn=n>0 ? n : -n;

       sprintf(buf,"%s %d",n>0 ? "FLOOD" : "STUFF" ,nn);
       fprintf(stdout,buf,"%s %d",n>0 ? "FLOOD" : "STUFF" ,nn);
       if(send_command_to_server(fserver,buf)) {
         fprintf(stderr,"%s rejected\n",n>0 ? "FLOOD" : "STUFF");
         close(fserver);
         exit(1);
       }
       fprintf(stderr,"Sending %d bytes \n",nn);
       put_int32_to_channel(fserver,nn);
       while(nn>0){
         nsent=write(fserver,buf,sizeof(buf)>nn?nn:sizeof(buf));
         nbuf++;
         if(nsent<=0) {
           fprintf(stderr,"send_junk_to_server: ERROR, write returned %d\n",nsent);
           break;
         }
         nn=nn-nsent;
       }
       fprintf(stderr,"send_junk_to_server: buffers sent = %d\n",nbuf);
       nsent=get_ack_nack(fserver);
       fprintf(stderr,"send_junk_to_server: ACK = %d\n",nsent);
}
