#!/bin/ksh
# Purpose: Script to launch gemntr (pre-processor) for GEMDM model
# Author: Badrinath Nagarajan & Ruben Santos
# Date:   Dec 2010-Jan 2011
# Disclaimer: Provided "as is" and no warranty whatsoever
#
# Environment Variables for GEMNTR and GEMDM
. ./envvars.sh

# Setup up execution directory
EXECDIR=`pwd`

# Cleanup first any old crap
/bin/rm -rf ./process 

# Runtime process directory
mkdir process 2> /dev/null
cd process

# Link namelist, climatology, geophysical and analysis data files
#----------------------------------------------------------------------------
#-- namelists
ln -s $EXECDIR/gem_settings.nml                               model_settings
ln -s $EXECDIR/outcfg.out                                     output_settings

#-- climatology, geophysical fields
ln -s $EXECDIR/CASES/clim_gef_400_mars96                      climato

#-- Coarse resolution (1X1 degree) geophysical field
#ln -s $EXECDIR/CASES/geophy_400                              geophy
#-- Hires (6 km global) geophysical field
ln -s $EXECDIR/CASES/gengeoout_global_6km.fst_0000001-0000001 geophy

# Analysis data
ln -s $EXECDIR/CASES/20010920.120000                          analysis

#----------------------------------------------------------------------------

# Launch GEMNTR  (runs only serially and not a big deal as it is pretty fast)
#--------------
cd $EXECDIR
$EXECDIR/gemntr.Abs

# Prepare directories needed by GEMDM (mpi run)
# (e.g., 00-00, 00-01 etc depending on number of processors chosen for GEMDM )
# -----------------------------------------------------------------------------
cd $EXECDIR/process
ls bm* |awk 'BEGIN{FS="-"}{if (NF==3) print "mkdir  "substr($2,1,2)"-"substr($3,1,2)}'  > doit.sh
chmod u+x doit.sh
./doit.sh
rm -f ./doit.sh

# Wrap up
echo "CREATION OF DIRECTORIES FOR GEMDM MPI DONE ..."
cd $EXECDIR
