/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_calclatlon(wordint gdid)
{
   ftnfloat xlat00, xlon00, dlat, dlon;
   wordint i,j,k,ni, nj, npts, hemisphere;
   ftnfloat *lonp, *latp, *xp, *yp;
   ftnfloat *x, *y;

   if (!(Grille[gdid].flags & LAT))
      {
      ni = Grille[gdid].ni;
      nj = Grille[gdid].nj;
      npts = ni*nj;
      
      Grille[gdid].lat = (ftnfloat *) malloc(ni*nj*sizeof(ftnfloat));
      Grille[gdid].lon = (ftnfloat *) malloc(ni*nj*sizeof(ftnfloat));
      
      switch(Grille[gdid].grtyp)
         {
         case 'A':
         case 'B':
           f77name(grll)(Grille[gdid].lat,Grille[gdid].lon,&ni,&nj,
			 &Grille[gdid].xg[SWLAT],&Grille[gdid].xg[SWLON], &Grille[gdid].xg[DLAT], &Grille[gdid].xg[DLON]);
           break;
           
         case 'E':
           dlon = 360. /(ni-1);
           dlat = 180./(nj);
           xlon00 = 0.0;
           xlat00 = -90. + 0.5*dlat;

           f77name(grll)(Grille[gdid].lat,Grille[gdid].lon,&ni,&nj,&xlat00,&xlon00,&dlat,&dlon);     
	   
           f77name(cigaxg)(&Grille[gdid].grtyp, &Grille[gdid].xg[XLAT1], &Grille[gdid].xg[XLON1], 
			   &Grille[gdid].xg[XLAT2], &Grille[gdid].xg[XLON2], 
                           &Grille[gdid].ig[IG1],  &Grille[gdid].ig[IG2], &Grille[gdid].ig[IG3], &Grille[gdid].ig[IG4]);
           latp = (ftnfloat *) malloc(ni*nj*sizeof(ftnfloat));
           lonp = (ftnfloat *) malloc(ni*nj*sizeof(ftnfloat));
           f77name(ez_gfllfxy)(lonp,latp,Grille[gdid].lon,Grille[gdid].lat,&npts, 
			       &Grille[gdid].xg[XLAT1], &Grille[gdid].xg[XLON1], &Grille[gdid].xg[XLAT2], 
			       &Grille[gdid].xg[XLON2]);
           memcpy(Grille[gdid].lat,latp,npts*sizeof(ftnfloat));
           memcpy(Grille[gdid].lon,lonp,npts*sizeof(ftnfloat));
           free(latp);
           free(lonp);
           break;
           
         case 'L':
           f77name(grll)(Grille[gdid].lat,Grille[gdid].lon,&ni,&nj,
			 &Grille[gdid].xg[SWLAT],&Grille[gdid].xg[SWLON],
			 &Grille[gdid].xg[DLAT], &Grille[gdid].xg[DLON]);
           break;
           
         case 'N':
         case 'S':
           if (Grille[gdid].grtyp == 'N')
	     {
	     hemisphere = 1;
	     }
           else
              {
              hemisphere = 2;
              }
           f77name(grps)(Grille[gdid].lat,Grille[gdid].lon,&ni,&nj,
			 &Grille[gdid].xg[PI],&Grille[gdid].xg[PJ],
			 &Grille[gdid].xg[D60], &Grille[gdid].xg[DGRW], &hemisphere);
           break;
           
	 case 'T':
	   npts = ni * nj;
	   latp = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
	   lonp = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
	   xp = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
	   yp = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
	   for (j=0; j < nj; j++)
	     {
	     for (i=0; i < ni; i++)
	       {
	       k = j*ni + i;
	       yp[k] = 1.0 * (j+1);
	       xp[k] = 1.0 * (i+1);
	       }
	     }

	   f77name(ez_vtllfxy)(latp,lonp, xp,yp, 
			    &Grille[gdid].xg[CLAT], &Grille[gdid].xg[CLON],
			    &Grille[gdid].xg[TD60],&Grille[gdid].xg[TDGRW],
			    &ni,&nj,&npts);
	   
	   memcpy(Grille[gdid].lon, lonp, ni*nj*sizeof(ftnfloat));
	   memcpy(Grille[gdid].lat, latp, ni*nj*sizeof(ftnfloat));
	   free(lonp);
	   free(latp);
	   free(xp);
	   free(yp);
	   break;
	   
         case 'Y':
           switch (Grille[gdid].grref)
	     {
	     case 'N':
	     case 'S':
	       fprintf(stderr, "<ez_calclatlon> Operation not supported - Y grid on PS Grid\n");
	       exit(13);
	       break;
	       
	     case 'L':
	       memcpy(Grille[gdid].lon, Grille[gdid].ax, Grille[gdid].ni*Grille[gdid].nj*sizeof(ftnfloat));
	       memcpy(Grille[gdid].lat, Grille[gdid].ay, Grille[gdid].ni*Grille[gdid].nj*sizeof(ftnfloat));
	       for (i=0; i < Grille[gdid].ni*Grille[gdid].nj; i++)
		 {
		 if (Grille[gdid].lon[i] < 0.0)
		   {
		   Grille[gdid].lon[i] = Grille[gdid].ax[i] + 360.0;
		   }
		 }
	       break;
	       
	     case 'E':
	       fprintf(stderr, "<ez_calclatlon> Operation not supported - Y grid on E Grid\n");
	       exit(13);
                break;
		
	     }
           break;
           
         case '#':
         case 'Z':
	 case 'G':
	   for (j=0; j < nj; j++)
	     {
	     for (i=0; i < ni; i++)
	       {
	       Grille[gdid].lat[C_TO_FTN(i,j,ni)] = Grille[gdid].ay[j];
	       Grille[gdid].lon[C_TO_FTN(i,j,ni)] = Grille[gdid].ax[i];
	       }
	     }
	     
	   if (Grille[gdid].grtyp == 'G' && Grille[gdid].ig[IG1] == NORD)
	     {
	     for (j=0; j < nj; j++)
	       {
	       for (i=0; i < ni; i++)
		 {
		 Grille[gdid].lat[C_TO_FTN(i,j,ni)] = Grille[gdid].ay[j+nj];
		 }
	       }
	     }
	   
           switch (Grille[gdid].grref)
	     {
	     case 'N':
	     case 'S':
	       latp = (ftnfloat *) malloc(ni*nj*sizeof(ftnfloat));
	       lonp = (ftnfloat *) malloc(ni*nj*sizeof(ftnfloat));
	       f77name(ez_vllfxy)(latp,lonp,
			       Grille[gdid].lon,Grille[gdid].lat,&ni,&nj,
			       &Grille[gdid].xgref[D60],&Grille[gdid].xgref[DGRW],
			       &Grille[gdid].xgref[PI], &Grille[gdid].xgref[PJ], &Grille[gdid].hemisphere);
	       
	       for (i=0; i < ni*nj; i++)
		 {
		 if (lonp[i] < 0.0) lonp[i] += 360.0;
		 }
	       
	       memcpy(Grille[gdid].lon, lonp, ni*nj*sizeof(ftnfloat));
	       memcpy(Grille[gdid].lat, latp, ni*nj*sizeof(ftnfloat));
	       free(lonp);
	       free(latp);
	       break;
	       
	     case 'L':
	       for (j=0; j < nj; j++)
		 {
		 for (i=0; i < ni; i++)
		   {
		   Grille[gdid].lat[C_TO_FTN(i,j,ni)] += 1.0;
		   Grille[gdid].lon[C_TO_FTN(i,j,ni)] += 1.0;
		   }
		 }
	       c_llfgr(Grille[gdid].lat, Grille[gdid].lon, Grille[gdid].lon, Grille[gdid].lat, ni*nj,
		       Grille[gdid].xgref[SWLAT],Grille[gdid].xgref[SWLON], Grille[gdid].xgref[DLAT], Grille[gdid].xgref[DLON]);
	       break;
	       
	     case 'E':
	       latp = (ftnfloat *) malloc(ni*nj*sizeof(ftnfloat));
	       lonp = (ftnfloat *) malloc(ni*nj*sizeof(ftnfloat));
	       f77name(ez_gfllfxy)(lonp,latp,Grille[gdid].lon,Grille[gdid].lat,&npts,
				   &Grille[gdid].xgref[XLAT1],&Grille[gdid].xgref[XLON1], &Grille[gdid].xgref[XLAT2], &Grille[gdid].xgref[XLON2]);
	       memcpy(Grille[gdid].lon, lonp, ni*nj*sizeof(ftnfloat));
	       memcpy(Grille[gdid].lat, latp, ni*nj*sizeof(ftnfloat));
	       free(lonp);
	       free(latp);
	       break;
	       
	     }
           break;
           
         case '!':
	   x = (ftnfloat *) malloc(ni*nj*sizeof(ftnfloat));
	   y = (ftnfloat *) malloc(ni*nj*sizeof(ftnfloat));
	   for (j=0; j < nj; j++)
	     {
	     for (i=0; i < ni; i++)
	       {
	       x[C_TO_FTN(i,j,ni)] = (ftnfloat) (i+1.0);
	       y[C_TO_FTN(i,j,ni)] = (ftnfloat) (j+1.0);
	       }
	     }
	   f77name(ez_llflamb)(Grille[gdid].lat,Grille[gdid].lon,x,y,&npts,
			       &Grille[gdid].grtyp, &Grille[gdid].ig[IG1],&Grille[gdid].ig[IG2],
			       &Grille[gdid].ig[IG3],&Grille[gdid].ig[IG4]);
	   for (i=0; i < npts; i++)
	     {
	     if (Grille[gdid].lon[i] < 0.0)
	       {
	       Grille[gdid].lon[i] += 360.0;
	       }
	     }
	   break;
         }

      switch(Grille[gdid].grtyp)
	{
	case 'G':
	case 'B':
	case 'A':
	  if (Grille[gdid].ig[IG2] == 1)
	    {
	    f77name(permut)(Grille[gdid].lat, &Grille[gdid].ni, &Grille[gdid].nj);
	    }
	  break;
	  
	default:
	  break;
	}
      
      Grille[gdid].flags |= LAT;
      }
   
   if (groptions.verbose == 2)
     {
     fprintf(stderr, "gdid: %ld\n", gdid);
     for (j=0; j < nj; j++)
       {
       for (i=0; i < ni; i++)
	 {
	 fprintf(stderr, "%ld %ld %f %f\n", i,j,Grille[gdid].lat[i], Grille[gdid].lon[i]);
	 }
       }
     }
   return 0;
   
   
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_calcnpolarwind(ftnfloat *polar_uu_in, ftnfloat *polar_vv_in, ftnfloat *uuin, ftnfloat *vvin, 
			  wordint ni, wordint nj, wordint gdin)
{
  wordint k1, k2;
  ftnfloat *polar_wd, *polar_spd,*polar_lat,*polar_lon,*polar_lat_gem,
    *polar_lon_gem,*polar_x,*polar_y,*polar_uu,*polar_vv;
  char  grtyp[2],grref[2],grtypn[2],grtypa[2];
  wordint ig1in,ig2in,ig3in,ig4in,ig1in_ref,ig2in_ref,ig3in_ref,ig4in_ref;
  ftnfloat xlat1, xlat2, xlon1, xlon2;
  wordint ig1n, ig2n, ig3n, ig4n;
  ftnfloat pi, pj, d60, dgrw;
  wordint i,j,ier,gdps,gda;
  ftnfloat uupole, vvpole;
  ftnfloat quatrevingtdix, zero;

  polar_uu  = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_vv  = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_wd  = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_spd = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_lat = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_lon = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_x   = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_y   = (ftnfloat *) malloc(ni*sizeof(ftnfloat));

  for (i=0; i < ni; i++)
    {
    polar_x[i] = 1.0 * (i+1);
    polar_y[i] = 1.0 * nj;
    }
  
  c_gdllfxy(gdin, polar_lat, polar_lon, polar_x, polar_y, ni);
  ier = c_ezgxprm(gdin, &ni, &nj, grtyp, &ig1in, &ig2in, &ig3in, &ig4in,
		  grref, &ig1in_ref, &ig2in_ref, &ig3in_ref, &ig4in_ref);

  if (grtyp[0] == 'Z' && grref[0] == 'E')
    {
    polar_lat_gem   = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
    polar_lon_gem   = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
    
    for (i=0; i < ni; i++)
      {
      polar_lat_gem[i] = polar_lat[i];
      polar_lon_gem[i] = polar_lon[i];
      }
    
    f77name(cigaxg)(grref, &xlat1, &xlon1, &xlat2, &xlon2, &ig1in_ref, &ig2in_ref, &ig3in_ref, &ig4in_ref);
    f77name(ez_gfxyfll)(polar_lon_gem, polar_lat_gem, polar_lon, polar_lat, &ni, &xlat1, &xlon1, &xlat2, &xlon2);
    }

  grtypa[0] = 'A';
  gda = c_ezqkdef(24,12, grtypa, 0,0,0,0,0);
  c_gdwdfuv(gda, polar_spd, polar_wd,  &uuin[(nj-1)*ni], &vvin[(nj-1)*ni], polar_lat, polar_lon, ni);
  
  pi   = 0.0;
  pj   = 0.0;
  d60  = 1000.0;
  dgrw = 0.0;
  grtypn[0] = 'N';
  f77name(cxgaig)(grtypn, &ig1n, &ig2n, &ig3n, &ig4n, &pi, &pj, &d60, &dgrw);
  gdps = c_ezqkdef(ni, 1, grtypn, ig1n, ig2n, ig3n, ig4n, 0);
  c_gduvfwd(gdps, polar_uu, polar_vv, polar_spd,  polar_wd, polar_lat, polar_lon, ni);

  f77name(ez_calcpoleval)(&uupole, polar_uu, &ni, Grille[gdin].ax, &Grille[gdin].grtyp, &Grille[gdin].grref);
  f77name(ez_calcpoleval)(&vvpole, polar_vv, &ni, Grille[gdin].ax, &Grille[gdin].grtyp, &Grille[gdin].grref);

  quatrevingtdix = 90.0;
  zero = 0.0;
  c_gdwdfuv(gdps, polar_spd, polar_wd,  &uupole, &vvpole, &quatrevingtdix, &zero, 1);
 

  polar_lat[0] = 90.0;
  for (i=1; i < ni; i++)
    {
    polar_wd[i]  = polar_wd[0] + polar_lon[i];
    polar_spd[i] = polar_spd[0];
    polar_lat[i] = 90.0;
    }
  polar_wd[0] = polar_wd[0] + polar_lon[0];
  
  c_gduvfwd(gda, polar_uu, polar_vv, polar_spd,  polar_wd, polar_lat, polar_lon, ni);
  
  for (j=0; j < 3; j++)
    {
    for (i=0; i < ni; i++)
      {
      k1 = j * ni + i;
      k2 = (nj-3+j)  * ni + i;
      polar_uu_in[k1] = uuin[k2];
      polar_vv_in[k1] = vvin[k2];
      }
    }
  
  for (i=0; i < ni; i++)
    {
    k1 = 3 * ni+i;
    polar_uu_in[k1] = polar_uu[i];
    polar_vv_in[k1] = polar_vv[i];
    }

  free(polar_y);
  free(polar_x);
  free(polar_lat);
  free(polar_lon);
  free(polar_spd);
  free(polar_wd);
  free(polar_vv);
  free(polar_uu);

  ier = c_gdrls(gdps);
  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
void ez_calcntncof(wordint gdid)
{
  wordint nni, nnj;
  
  if (Grille[gdid].flags & NEWTON)
    return;

  nni = Grille[gdid].ni;
  nnj = Grille[gdid].j2 - Grille[gdid].j1 + 1;

  Grille[gdid].ncx = (ftnfloat *) malloc(nni*6*sizeof(ftnfloat));
  Grille[gdid].ncy = (ftnfloat *) malloc(nnj*6*sizeof(ftnfloat));
  f77name(ez_nwtncof)(Grille[gdid].ncx,Grille[gdid].ncy,
		      Grille[gdid].ax,Grille[gdid].ay,
		      &Grille[gdid].ni, &Grille[gdid].nj,
		      &Grille[gdid].i1, &Grille[gdid].i2, 
		      &Grille[gdid].j1, &Grille[gdid].j2,
		      &Grille[gdid].extension);
  
  Grille[gdid].flags |= NEWTON;
  
  
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_calcspolarwind(ftnfloat *polar_uu_in, ftnfloat *polar_vv_in, ftnfloat *uuin, ftnfloat *vvin, 
			  wordint ni, wordint nj, wordint gdin)
{
  wordint k1, k2;
  ftnfloat *polar_wd, *polar_spd,*polar_lat,*polar_lon,*polar_lat_gem,
    *polar_lon_gem,*polar_x,*polar_y,*polar_uu,*polar_vv;
  char  grtyp[2],grref[2],grtyps[2],grtypa[2];
  wordint ig1in,ig2in,ig3in,ig4in,ig1in_ref,ig2in_ref,ig3in_ref,ig4in_ref;
  ftnfloat xlat1, xlat2, xlon1, xlon2;
  wordint ig1n, ig2n, ig3n, ig4n;
  ftnfloat pi, pj, d60, dgrw;
  wordint i,j,ier,gdps,gda;
  ftnfloat uupole, vvpole;
  ftnfloat quatrevingtdix, zero;

  polar_uu  = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_vv  = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_wd  = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_spd = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_lat = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_lon = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_x   = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
  polar_y   = (ftnfloat *) malloc(ni*sizeof(ftnfloat));

  for (i=0; i < ni; i++)
    {
    polar_x[i] = 1.0 * (i+1);
    polar_y[i] = 1.0;
    }
  
  c_gdllfxy(gdin, polar_lat, polar_lon, polar_x, polar_y, ni);
  ier = c_ezgxprm(gdin, &ni, &nj, grtyp, &ig1in, &ig2in, &ig3in, &ig4in,
		  grref, &ig1in_ref, &ig2in_ref, &ig3in_ref, &ig4in_ref);

  if (grtyp[0] == 'Z' && grref[0] == 'E')
    {
    polar_lat_gem   = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
    polar_lon_gem   = (ftnfloat *) malloc(ni*sizeof(ftnfloat));
    
    for (i=0; i < ni; i++)
      {
      polar_lat_gem[i] = polar_lat[i];
      polar_lon_gem[i] = polar_lon[i];
      }
    
    f77name(cigaxg)(grref, &xlat1, &xlon1, &xlat2, &xlon2, &ig1in_ref, &ig2in_ref, &ig3in_ref, &ig4in_ref);
    f77name(ez_gfxyfll)(polar_lon_gem, polar_lat_gem, polar_lon, polar_lat, &ni, &xlat1, &xlon1, &xlat2, &xlon2);
    }

  grtypa[0] = 'A';
  gda = c_ezqkdef(24,12, grtypa, 0,0,0,0,0);
  c_gdwdfuv(gda, polar_spd, polar_wd,  uuin, vvin, polar_lat, polar_lon, ni);
  
  pi   = 0.0;
  pj   = 0.0;
  d60  = 1000.0;
  dgrw = 0.0;
  grtyps[0] = 'S';
  f77name(cxgaig)(grtyps, &ig1n, &ig2n, &ig3n, &ig4n, &pi, &pj, &d60, &dgrw);
  gdps = c_ezqkdef(ni, 1, grtyps, ig1n, ig2n, ig3n, ig4n, 0);
  c_gduvfwd(gdps, polar_uu, polar_vv, polar_spd,  polar_wd, polar_lat, polar_lon, ni);

  f77name(ez_calcpoleval)(&uupole, polar_uu, &ni, Grille[gdin].ax, 
			  &Grille[gdin].grtyp, &Grille[gdin].grref);

  f77name(ez_calcpoleval)(&vvpole, polar_vv, &ni, Grille[gdin].ax, 
			  &Grille[gdin].grtyp, &Grille[gdin].grref);

  quatrevingtdix = -90.0;
  zero = 0.0;
  c_gdwdfuv(gdps, polar_spd, polar_wd,  &uupole, &vvpole, &quatrevingtdix, &zero, 1);
  
  polar_lat[0] = -90.0;
  for (i=1; i < ni; i++)
    {
    polar_wd[i]  = polar_wd[0] - polar_lon[i];
    polar_spd[i] = polar_spd[0];
    polar_lat[i] = -90.0;
    }
  polar_wd[0] = polar_wd[0] + polar_lon[0];
  
  c_gduvfwd(gda, polar_uu, polar_vv, polar_spd,  polar_wd, polar_lat, polar_lon, ni);
  
  for (j=0; j < 3; j++)
    {
    for (i=0; i < ni; i++)
      {
      k1 = j * ni + i;
      k2 = (j+1)  * ni + i;
      polar_uu_in[k2] = uuin[k1];
      polar_vv_in[k2] = vvin[k1];
      }
    }
  
  for (i=0; i < ni; i++)
    {
    polar_uu_in[i] = polar_uu[i];
    polar_vv_in[i] = polar_vv[i];
    }

  free(polar_y);
  free(polar_x);
  free(polar_lat);
  free(polar_lon);
  free(polar_spd);
  free(polar_wd);
  free(polar_vv);
  free(polar_uu);

  if (grtyp[0] == 'Z' && grref[0] == 'E')
    {
    free(polar_lat_gem);
    free(polar_lon_gem);
    }
  ier = c_gdrls(gdps);
  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_calcxy(wordint iset)
{
   wordint coordonnee, ni_in, nj_in, ni_out, nj_out, ninj_out, gdin, gdout;

   if (gridset[iset].flags & XXX)
      {
      return 0;
      }

   /* Dans un premier temps on calcule la position x-y de tous les points sur la grille */

   gdin  = gridset[iset].gdin;
   gdout = gridset[iset].gdout;
   
   ni_in =  Grille[gdin].ni;
   nj_in =  Grille[gdin].nj;
   
   ni_out = Grille[gdout].ni;
   nj_out = Grille[gdout].nj;
   ninj_out = ni_out * nj_out;
   
   gridset[iset].x = (ftnfloat *) malloc(ninj_out*sizeof(ftnfloat));
   gridset[iset].y = (ftnfloat *) malloc(ninj_out*sizeof(ftnfloat));
   
   switch(Grille[gdin].grtyp)
      {
      case 'A':
      case 'B':
      case 'E':
      case 'L':
      case 'N':
      case 'S':
      case 'T':
      case '!':
        f77name(ez_ll2rgd)(gridset[iset].x, gridset[iset].y,
                           Grille[gdout].lat, Grille[gdout].lon, &ninj_out,
                           &ni_in, &nj_in, &Grille[gdin].grtyp,
                           &Grille[gdin].ig[IG1], &Grille[gdin].ig[IG2], 
			   &Grille[gdin].ig[IG3], &Grille[gdin].ig[IG4],
                           &groptions.symmetrie, Grille[gdin].ay);
        break;
        
        
      case '#':
      case 'Z':
      case 'G':
	coordonnee = RELATIF;
        f77name(ez_ll2igd)(gridset[iset].x, gridset[iset].y,
			   Grille[gdout].lat,Grille[gdout].lon,&ninj_out,
			   &ni_in,&nj_in,&Grille[gdin].grtyp, &Grille[gdin].grref,
			   &Grille[gdin].igref[IG1], &Grille[gdin].igref[IG2], 
			   &Grille[gdin].igref[IG3], &Grille[gdin].igref[IG4],
			   Grille[gdin].ax, Grille[gdin].ay, 
			   &coordonnee);
        
        break;
        
        
      default:
        break;
      }

   gridset[iset].flags |= XXX;
   
   return 0;
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_corrval(ftnfloat *zout, ftnfloat *zin, _gridset *gdset)
{
  
  wordint i;
  ftnfloat valmax, valmin,fudgeval;
  wordint gdin;
  _Grille grEntree, grSortie;
  wordint degIntCourant;
  wordint npts,nj;
  ftnfloat vpolnor, vpolsud;
  ftnfloat *temp;
  
  extern ftnfloat f77name(amax)();
  extern ftnfloat f77name(amin)();
  
  grEntree = Grille[gdset->gdin];
  grSortie = Grille[gdset->gdout];

  if (gdset->zones[DEHORS].npts > 0)
    {
    gdin = gdset->gdin;
    
    if (groptions.degre_extrap == ABORT)
      {
      fprintf(stderr, "<ez_corrval> There are points on the source grid that lie outside the source grid\n");
      fprintf(stderr, "<ez_corrval> aborting at your request!\n\n\n");
      exit(13);
      }
    
    nj = grEntree.j2 - grEntree.j1 +1;
    f77name(ez_aminmax)(&valmin,&valmax,zin,&grEntree.ni, &nj);
    if (groptions.degre_extrap >= MAXIMUM)
      {
      if (groptions.vecteur == VECTEUR)
	{
	fudgeval = 0.0;
	}
      else
	{
	switch (groptions.degre_extrap)
	  {
	  case MAXIMUM:
	    fudgeval = valmax + 0.05 * (valmax - valmin);
	    if (groptions.verbose > 0)
	      {
	      fprintf(stderr, "<ez_corrval>: maximum: %f \n", fudgeval);
	      }
	    break;
	    
	  case MINIMUM:
	    fudgeval = valmin - 0.05 * (valmax - valmin);
	    if (groptions.verbose > 0)
	      {
	      fprintf(stderr, "<ez_corrval>: minimum: %f \n", fudgeval);
	      }
	    break;
	    
	  case VALEUR:
	    fudgeval = groptions.valeur_extrap;
	    if (groptions.verbose > 0)
	      {
	      fprintf(stderr, "<ez_corrval>: valeur: %f \n", fudgeval);
	      }
	    break;
	  }
	}
      
      for (i=0; i < gdset->zones[DEHORS].npts; i++)
	{
	zout[gdset->zones[DEHORS].idx[i]] = fudgeval;
	}
      }
    else
      {
      degIntCourant = groptions.degre_interp;
      groptions.degre_interp = groptions.degre_extrap;
      temp = (ftnfloat *) malloc(gdset->zones[DEHORS].npts*sizeof(ftnfloat));
      
      c_gdinterp(temp, zin, gdset->gdin, gdset->zones[DEHORS].x, 
		 gdset->zones[DEHORS].y, gdset->zones[DEHORS].npts);

     for (i=0; i < gdset->zones[DEHORS].npts; i++)
	{
	zout[gdset->zones[DEHORS].idx[i]] = temp[i];
	}
      free(temp);
      groptions.degre_interp = degIntCourant;
      }
    }
  
  if (groptions.vecteur == VECTEUR)
    {
    return 0;
    }
  
  
  if (gdset->zones[AU_NORD].npts > 0)
    {
    ez_corrval_aunord(zout, zin, gdset);
    }
  
  if (gdset->zones[AU_SUD].npts > 0)
    {
    ez_corrval_ausud(zout, zin, gdset);
    }
  
  
  if (gdset->zones[POLE_NORD].npts > 0 || gdset->zones[POLE_SUD].npts > 0)
    {
    if (grEntree.grtyp != 'E')
      {
      npts = grEntree.ni * grEntree.j2;
      f77name(ez_calcpoleval)(&vpolnor, &zin[(nj-1)*grEntree.ni], &grEntree.ni, grEntree.ax, &grEntree.grtyp, &grEntree.grref);
      for (i=0; i < gdset->zones[POLE_NORD].npts; i++)
	{
	zout[gdset->zones[POLE_NORD].idx[i]] = vpolnor;
	}
      
      f77name(ez_calcpoleval)(&vpolsud, zin, &grEntree.ni, grEntree.ax, &grEntree.grtyp, &grEntree.grref);
      for (i=0; i < gdset->zones[POLE_SUD].npts; i++)
	{
	zout[gdset->zones[POLE_SUD].idx[i]] = vpolsud;
	}
      }
    }
  if ((grEntree.grtyp == 'Z' || grEntree.grtyp == '#') && grEntree.grref == 'E' && grSortie.grtyp == 'B')
    {
    f77name(ez_corrbgd)(zout, &grSortie.ni, &grSortie.nj, &grSortie.ig[IG1]);
    }

  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_corrval_aunord(ftnfloat *zout, ftnfloat *zin, _gridset *gdset)
{
  wordint i;
  wordint gdin;
  wordint npts;
  ftnfloat *temp, *vals, *temp_y;
  ftnfloat ay[4], poleval;
  wordint ni, nj, i1, i2, j1, j2;
  wordint un = 1;
  wordint quatre = 4;
  
  
  npts = gdset->zones[AU_NORD].npts;
  if (npts > 0)
    {
    gdin = gdset->gdin;
    ni = Grille[gdin].ni;
    nj = Grille[gdin].j2 - Grille[gdin].j1 + 1;
    
    i1 = 1;
    i2 = ni;

    j1 = Grille[gdin].j2-2;
    j2 = Grille[gdin].j2+1;

    temp = (ftnfloat *) malloc(4 * ni * sizeof(ftnfloat));
    vals = (ftnfloat *) malloc(npts * sizeof(ftnfloat));
    f77name(ez_calcpoleval)(&poleval, &zin[(nj-1)*ni], &ni, Grille[gdin].ax, &Grille[gdin].grtyp, &Grille[gdin].grref);
    f77name(ez_fillnpole)(temp, zin, &ni, &Grille[gdin].j1, &Grille[gdin].j2, &poleval);

    switch (groptions.degre_interp)
      {
      case CUBIQUE:
	switch (Grille[gdin].grtyp)
	  {
	  case 'Z':
	  case 'E':
	  case 'G':
	    ay[0] = Grille[gdin].ay[Grille[gdin].j2-3];
	    ay[1] = Grille[gdin].ay[Grille[gdin].j2-2];
	    ay[2] = Grille[gdin].ay[Grille[gdin].j2-1];
	    ay[3] = 90.0;
	    f77name(ez_irgdint_3_wnnc)(vals,gdset->zones[AU_NORD].x,
				       gdset->zones[AU_NORD].y,&npts,
				       Grille[gdin].ax, ay, temp,
				       &ni, &j1, &j2, &Grille[gdin].extension);
	    break;

	  default:
	    f77name(ez_rgdint_3_wnnc)(vals,gdset->zones[AU_NORD].x,
				      gdset->zones[AU_NORD].y,&npts,
				      temp,&ni, &j1, &j2, &Grille[gdin].extension);
	    break;
	  }
	break;
	
      case LINEAIRE:	
	temp_y = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
	for (i=0; i < npts; i++)
	  {
	  temp_y[i] = gdset->zones[AU_NORD].y[i] - (1.0 * (nj-3));
	  }
	f77name(ez_rgdint_1_nw)(vals,gdset->zones[AU_NORD].x,temp_y,&npts,temp,&ni, &un, &quatre);
	free(temp_y);
	break;
	
      case VOISIN:
	temp_y = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
	for (i=0; i < npts; i++)
	  {
	  temp_y[i] = gdset->zones[AU_NORD].y[i] - (1.0 * (nj-3));
	  }
	f77name(ez_rgdint_0)(vals,gdset->zones[AU_NORD].x,temp_y,&npts,temp,&ni, &un, &quatre);
	free(temp_y);
	break;
      }

      
    for (i=0; i < gdset->zones[AU_NORD].npts; i++)
      {
      zout[gdset->zones[AU_NORD].idx[i]] = vals[i];
      }
    
    free(vals);
    free(temp);
    }
  return 0;
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_corrval_ausud(ftnfloat *zout, ftnfloat *zin, _gridset *gdset)
{
  wordint i;
  wordint gdin;
  wordint npts;
  ftnfloat vpolesud;
  ftnfloat *temp, *temp_y, *vals;
  ftnfloat ay[4];
  wordint ni, nj, i1, i2, j1, j2;
  wordint un = 1;
  wordint quatre = 4;
  
  npts = gdset->zones[AU_SUD].npts;
  if (npts > 0)
    {
    gdin = gdset->gdin;
    ni = Grille[gdin].ni;
    
    i1 = 1;
    i2 = ni;
    j1 = 0;
    j2 = 3;
    
    temp = (ftnfloat *) malloc(4 * ni * sizeof(ftnfloat));
    vals = (ftnfloat *) malloc(npts * sizeof(ftnfloat));
    f77name(ez_calcpoleval)(&vpolesud, zin, &ni, Grille[gdin].ax, &Grille[gdin].extension, 
			    &Grille[gdin].grtyp, &Grille[gdin].grref);
    f77name(ez_fillspole)(temp, zin, &ni, &Grille[gdin].j1, &Grille[gdin].j2, &vpolesud);

    switch (groptions.degre_interp)
      {
      case CUBIQUE:
	switch (Grille[gdin].grtyp)
	  {
	  case 'Z':
	  case 'E':
	  case 'G':
	    ay[0] = -90.0;
	    ay[1] = Grille[gdin].ay[0];
	    ay[2] = Grille[gdin].ay[1];
	    ay[3] = Grille[gdin].ay[2];
	    f77name(ez_irgdint_3_wnnc)(vals,gdset->zones[AU_SUD].x,
				       gdset->zones[AU_SUD].y,&npts,
				       Grille[gdin].ax, ay, temp,
				       &ni, &j1, &j2, &Grille[gdin].extension);
	    break;
	    
	  default:
	    f77name(ez_rgdint_3_wnnc)(vals,gdset->zones[AU_SUD].x,
				      gdset->zones[AU_SUD].y,&npts,
				      temp,&ni, &j1, &j2, &Grille[gdin].extension);
	    break;
	  }
	break;
	
      case LINEAIRE:	
	temp_y = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
	for (i=0; i < npts; i++)
	  {
	  temp_y[i] = gdset->zones[AU_SUD].y[i] + 1.0;
	  }
	f77name(ez_rgdint_1_nw)(vals,gdset->zones[AU_SUD].x,temp_y,&npts,temp,&ni, &un, &quatre);
	free(temp_y);
	break;
	
      case VOISIN:
	temp_y = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
	for (i=0; i < npts; i++)
	  {
	  temp_y[i] = gdset->zones[AU_SUD].y[i] + 1.0;
	  }
	f77name(ez_rgdint_0)(vals,gdset->zones[AU_SUD].x,temp_y,&npts,temp,&ni, &un, &quatre);
	free(temp_y);
	break;
      }

    for (i=0; i < gdset->zones[AU_SUD].npts; i++)
      {
      zout[gdset->zones[AU_SUD].idx[i]] = vals[i];
      }
    
    free(vals);
    free(temp);
    }
  return 0;
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_corrvec(ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin, _gridset *gdset)
{
  wordint ier;

  if (gdset->zones[AU_NORD].npts > 0)
    {
    ier = ez_corrvec_aunord(uuout,vvout,uuin,vvin, gdset);
    }
  
  if (gdset->zones[AU_SUD].npts > 0)
    {
    ier = ez_corrvec_ausud(uuout,vvout,uuin,vvin, gdset);
    }
  
  if (gdset->zones[POLE_NORD].npts > 0)
    {
    ier = ez_corrvec_aunord(uuout,vvout,uuin,vvin, gdset);
    }
  
  if (gdset->zones[POLE_SUD].npts > 0)
    {
    ier = ez_corrvec_ausud(uuout,vvout,uuin,vvin, gdset);
    }
      
   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_corrvec_aunord(ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin, _gridset *gdset)
{
  ftnfloat uupole, vvpole;
  ftnfloat *polar_uu_in, *polar_vv_in, *polar_uu_out, *polar_vv_out, *corr_uus, *corr_vvs, *temp_y,ay[4];
  wordint gdin, ni, nj, i1, i2, j1, j2, degree,npts,i;
  wordint quatre = 4;	
  wordint un = 1;
  
  gdin = gdset->gdin;
  ni = Grille[gdin].ni;
  nj = Grille[gdin].j2;
  
  i1 = 1;
  i2 = ni;
  j1 = nj-2;
  j2 = nj+1;
  degree = 3;
  
  npts = gdset->zones[AU_NORD].npts;

  polar_uu_in = (ftnfloat *) malloc(4 * ni * sizeof(ftnfloat));
  polar_vv_in = (ftnfloat *) malloc(4 * ni * sizeof(ftnfloat));
  corr_uus = (ftnfloat *) malloc(npts * sizeof(ftnfloat));
  corr_vvs = (ftnfloat *) malloc(npts * sizeof(ftnfloat));
  
  ez_calcnpolarwind(polar_uu_in, polar_vv_in, uuin, vvin, ni, nj, gdin);
  
  switch (groptions.degre_interp)
    {
    case CUBIQUE:
      switch (Grille[gdin].grtyp)
	{
	case 'Z':
	case 'E':
	case 'G':
	  ay[0] = Grille[gdin].ay[nj-3];
	  ay[1] = Grille[gdin].ay[nj-2];
	  ay[2] = Grille[gdin].ay[nj-1];
	  ay[3] = 90.0;
	  f77name(ez_irgdint_3_wnnc)(corr_uus,gdset->zones[AU_NORD].x, gdset->zones[AU_NORD].y,&npts,
				     Grille[gdin].ax, ay, polar_uu_in,
				     &ni, &j1, &j2, &Grille[gdin].extension);
	  f77name(ez_irgdint_3_wnnc)(corr_vvs,gdset->zones[AU_NORD].x, gdset->zones[AU_NORD].y,&npts,
				     Grille[gdin].ax, ay, polar_vv_in,
				     &ni, &j1, &j2, &Grille[gdin].extension);
	  break;

	default:
	  f77name(ez_rgdint_3_wnnc)(corr_uus,gdset->zones[AU_NORD].x, gdset->zones[AU_NORD].y,&npts,
				    polar_uu_in,&ni, &j1, &j2, &Grille[gdin].extension);
	  f77name(ez_rgdint_3_wnnc)(corr_vvs,gdset->zones[AU_NORD].x, gdset->zones[AU_NORD].y,&npts,
				    polar_vv_in,&ni, &j1, &j2, &Grille[gdin].extension);
	  break;
	}
      
    case LINEAIRE:	
      temp_y = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      for (i=0; i < npts; i++)
	{
	temp_y[i] = gdset->zones[AU_NORD].y[i] - (1.0 * (nj-3));
	}
      f77name(ez_rgdint_1_nw)(corr_uus,gdset->zones[AU_NORD].x,temp_y,&npts,polar_uu_in,&ni, &un, &quatre);
      f77name(ez_rgdint_1_nw)(corr_vvs,gdset->zones[AU_NORD].x,temp_y,&npts,polar_vv_in,&ni, &un, &quatre);
      free(temp_y);
      break;
      
    case VOISIN:
      temp_y = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      for (i=0; i < npts; i++)
	{
	temp_y[i] = gdset->zones[AU_NORD].y[i] - (1.0 * (nj-3));
	}
      f77name(ez_rgdint_0)(corr_uus,gdset->zones[AU_NORD].x,temp_y,&npts,polar_uu_in,&ni, &un, &quatre);
      f77name(ez_rgdint_0)(corr_vvs,gdset->zones[AU_NORD].x,temp_y,&npts,polar_vv_in,&ni, &un, &quatre);
      free(temp_y);
      break;
    }
  
  
  for (i=0; i < gdset->zones[AU_NORD].npts; i++)
    {
    uuout[gdset->zones[AU_NORD].idx[i]] = corr_uus[i];
    vvout[gdset->zones[AU_NORD].idx[i]] = corr_vvs[i];
    }
  
  free(polar_uu_in);
  free(polar_vv_in);
  free(corr_uus);
  free(corr_vvs);
  
  return 0;  
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_corrvec_ausud(ftnfloat *uuout, ftnfloat *vvout, 
			 ftnfloat *uuin, ftnfloat *vvin, 
			 _gridset *gdset)
{
  ftnfloat *polar_uu_in, *polar_vv_in, *corr_uus, *corr_vvs, *temp_y, ay[4];
  wordint gdin, ni, nj, i1, i2, j1, j2, degree,npts,i;
  wordint quatre = 4;
  wordint un = 1;
  
  npts = gdset->zones[AU_SUD].npts;
  gdin = gdset->gdin;
  ni = Grille[gdin].ni;
  nj = Grille[gdin].j2;
  
  i1 = 1;
  i2 = ni;
  j1 = 0;
  j2 = 3;
  degree = 3;
  
  polar_uu_in = (ftnfloat *) malloc(4 * ni * sizeof(ftnfloat));
  polar_vv_in = (ftnfloat *) malloc(4 * ni * sizeof(ftnfloat));
  corr_uus = (ftnfloat *) malloc(npts * sizeof(ftnfloat));
  corr_vvs = (ftnfloat *) malloc(npts * sizeof(ftnfloat));
  
  ez_calcspolarwind(polar_uu_in, polar_vv_in, uuin, vvin, ni, nj, gdin);
  
  switch (groptions.degre_interp)
    {
    case CUBIQUE:
      switch (Grille[gdin].grtyp)
	{
	case 'Z':
	case 'E':
	case 'G':
	  ay[0] = -90.;
	  ay[1] = Grille[gdin].ay[0];
	  ay[2] = Grille[gdin].ay[1];
	  ay[3] = Grille[gdin].ay[2];
	  f77name(ez_irgdint_3_wnnc)(corr_uus,gdset->zones[AU_SUD].x,
				     gdset->zones[AU_SUD].y,&npts,
				     Grille[gdin].ax, ay, polar_uu_in,
				     &ni, &j1, &j2, &Grille[gdin].extension);
	  f77name(ez_irgdint_3_wnnc)(corr_vvs,gdset->zones[AU_SUD].x,
				     gdset->zones[AU_SUD].y,&npts,
				     Grille[gdin].ax, ay, polar_vv_in,
				     &ni, &j1, &j2, &Grille[gdin].extension);
	  break;
	  
	default:
	  f77name(ez_rgdint_3_wnnc)(corr_uus,gdset->zones[AU_SUD].x,
				    gdset->zones[AU_SUD].y,&npts,
				    polar_uu_in,&ni, &j1, &j2, &Grille[gdin].extension);
	  f77name(ez_rgdint_3_wnnc)(corr_vvs,gdset->zones[AU_SUD].x,
				    gdset->zones[AU_SUD].y,&npts,
				    polar_vv_in,&ni, &j1, &j2, &Grille[gdin].extension);
	  break;
	}
      break;
      
    case LINEAIRE:
      temp_y = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      for (i=0; i < npts; i++)
	{
	temp_y[i] = gdset->zones[AU_SUD].y[i] + 1.0;
	}
      f77name(ez_rgdint_1_nw)(corr_uus,gdset->zones[AU_SUD].x,temp_y,&npts,polar_uu_in,&ni, &un, &quatre);
      f77name(ez_rgdint_1_nw)(corr_vvs,gdset->zones[AU_SUD].x,temp_y,&npts,polar_vv_in,&ni, &un, &quatre);
      free(temp_y);
      break;
      
    case VOISIN:
      temp_y = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      for (i=0; i < npts; i++)
	{
	temp_y[i] = gdset->zones[AU_SUD].y[i] +  1.0;
	}
      f77name(ez_rgdint_0)(corr_uus,gdset->zones[AU_SUD].x,temp_y,&npts,polar_uu_in,&ni, &un, &quatre);
      f77name(ez_rgdint_0)(corr_vvs,gdset->zones[AU_SUD].x,temp_y,&npts,polar_vv_in,&ni, &un, &quatre);
      free(temp_y);
      break;
    }
  
  
  for (i=0; i < gdset->zones[AU_SUD].npts; i++)
    {
    uuout[gdset->zones[AU_SUD].idx[i]] = corr_uus[i];
    vvout[gdset->zones[AU_SUD].idx[i]] = corr_vvs[i];
    }
  
  free(polar_uu_in);
  free(polar_vv_in);
  free(corr_uus);
  free(corr_vvs);
  
  return 0;  
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

void c_ezdefaxes(wordint gdid, ftnfloat *ax, ftnfloat *ay)
{
  wordint i,j;
  ftnfloat *temp, dlon;
  wordint zero, deuxnj;
  
  _Grille *gr;
  
  gr = &Grille[gdid];
  switch (gr->grtyp)
    {
    case '#':
    case 'Z':
      f77name(cigaxg)(&gr->grref,&gr->xgref[XLAT1], &gr->xgref[XLON1], &gr->xgref[XLAT2], &gr->xgref[XLON2],
		      &gr->igref[IG1], &gr->igref[IG2], &gr->igref[IG3], &gr->igref[IG4]);
      
      Grille[gdid].ax = (ftnfloat *) malloc(gr->ni*sizeof(ftnfloat));
      Grille[gdid].ay = (ftnfloat *) malloc(gr->nj*sizeof(ftnfloat));
      
      memcpy(Grille[gdid].ax,ax,gr->ni*sizeof(ftnfloat));
      memcpy(Grille[gdid].ay,ay,gr->nj*sizeof(ftnfloat));
      ez_calcxpncof(gdid);
      ez_calcntncof(gdid);
      break;
      
    case 'Y':
      Grille[gdid].ax = (ftnfloat *) malloc(gr->ni*gr->nj*sizeof(ftnfloat));
      Grille[gdid].ay = (ftnfloat *) malloc(gr->ni*gr->nj*sizeof(ftnfloat));
      memcpy(Grille[gdid].ax,ax,gr->ni*gr->nj*sizeof(ftnfloat));
      memcpy(Grille[gdid].ay,ay,gr->ni*gr->nj*sizeof(ftnfloat));
      
      ez_calcxpncof(gdid);
/*      ez_calcntncof(gdid); */
      break;
      
    case 'G':
      gr->grref = 'L';
      gr->xgref[SWLAT] = 0.0;
      gr->xgref[SWLON] = 0.0;
      gr->xgref[DLAT] = 1.0;
      gr->xgref[DLON] = 1.0;
      f77name(cxgaig)(&gr->grref,&gr->igref[IG1], &gr->igref[IG2], &gr->igref[IG3], &gr->igref[IG4],
		      &gr->xgref[SWLAT], &gr->xgref[SWLON], &gr->xgref[DLAT], &gr->xgref[DLON]);
      
      Grille[gdid].ax = (ftnfloat *) malloc(gr->ni*sizeof(ftnfloat));
      dlon = 360. / (ftnfloat) gr->ni;
      for (i=0; i < gr->ni; i++)
	{
	Grille[gdid].ax[i] = (ftnfloat)i * dlon;
	}
      
      zero = 0;
      ez_calcxpncof(gdid);

      switch (Grille[gdid].ig[IG1])
	{
	case GLOBAL:
	  Grille[gdid].ay = (ftnfloat *) malloc(gr->nj*sizeof(ftnfloat));
	  temp    = (ftnfloat *) malloc(gr->nj*sizeof(ftnfloat));
	  f77name(ez_glat)(Grille[gdid].ay,temp,&gr->nj,&zero);
	  free(temp);
	  break;
	  
	case NORD:
	case SUD:
	  deuxnj = 2 * gr->nj;
	  Grille[gdid].ay = (ftnfloat *) malloc(deuxnj*sizeof(ftnfloat));
	  temp    = (ftnfloat *) malloc(deuxnj*sizeof(ftnfloat));
	  f77name(ez_glat)(Grille[gdid].ay,temp,&deuxnj,&zero);
	  free(temp);
	  break;
	}
      
      
      ez_calcntncof(gdid);
      Grille[gdid].flags |= AX;
      break;
      
    default:
      ez_calcxpncof(gdid);
      break;
    }
  
  
  Grille[gdid].flags |= AX;
  
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

void c_ezdefxg(wordint gdid)
{
  
  _Grille *gr;
  
  gr = &Grille[gdid];
  
  switch (gr->grtyp)
    {
    case 'A':
    case 'G':
      gr->xg[DLON]  = 360. /gr->ni;
      gr->xg[SWLON] = 0.0; 
      switch (gr->ig[IG1])
	{
	case 0:
	  gr->xg[DLAT] = 180./gr->nj;
	  gr->xg[SWLAT] = -90. + 0.5*gr->xg[DLAT];
	  break;
	  
	case 1:
	  gr->xg[DLAT] = 90./gr->nj;
	  gr->xg[SWLAT] = 0.5*gr->xg[DLAT];
	  gr->needs_expansion = OUI;
	  break;
	  
	case 2:
	  gr->xg[DLAT] = 90./gr->nj;
	  gr->xg[SWLAT] = -90. + 0.5*gr->xg[DLAT];
	  gr->needs_expansion = OUI;
	  break;
	  
	default:
	  fprintf(stderr, "<ez_gdef_fmem> 'A' grid has to be Global/North/South\n");
	  break;
	}

      switch(gr->ig[IG2])
	{
	case 1:
	  gr->axe_y_inverse = OUI;
	  break;

	default:
	  break;
	}

      break;
      
    case 'B':
      gr->xg[DLON] = 360. /(gr->ni-1);
      gr->xg[SWLON] = 0.0;
      switch (gr->ig[IG1])
	{
	case 0:
	  gr->xg[DLAT] = 180./(gr->nj-1);
	  gr->xg[SWLAT] = -90.;
	  break;
	  
	case 1:
	  gr->xg[DLAT] = 90./(gr->nj-1);
	  gr->xg[SWLAT] = 0.;
	  gr->needs_expansion = OUI;
	  break;
	  
	case 2:
	  gr->xg[DLAT] = 90./(gr->nj-1);
	  gr->xg[SWLAT] = -90.;
	  gr->needs_expansion = OUI;
	  break;
	  
	default:
	  fprintf(stderr, "<ezgdef_fmem> 'B' grid has to be Global/North/South\n");
	  break;
	}
      
      switch(gr->ig[IG2])
	{
	case 1:
	  gr->axe_y_inverse = OUI;
	  break;
	  
	default:
	  break;
	}
      break;
      
    case 'E':
      f77name(cigaxg)(&gr->grtyp,&gr->xg[XLAT1],&gr->xg[XLON1],&gr->xg[XLAT2],&gr->xg[XLON2],
		      &gr->ig[IG1],&gr->ig[IG2],&gr->ig[IG3],&gr->ig[IG4]);
      /*      gr->xg[DLAT] = 180./gr->nj;
	      gr->xg[DLON] = 360./(gr->ni-1);
	      gr->xg[SWLON] = 0.0;
	      gr->xg[SWLAT] = -90. + 0.5*gr->xg[DLAT];
      */
      break;

    case 'H':
    case 'Y':
    case '#':
    case 'Z':
    case '!':
      break;

    case 'L':
      f77name(cigaxg)(&gr->grtyp,&gr->xg[SWLAT], &gr->xg[SWLON], &gr->xg[DLAT], &gr->xg[DLON],
		      &gr->ig[IG1], &gr->ig[IG2], &gr->ig[IG3], &gr->ig[IG4]);
      break;

    case 'N':
      f77name(cigaxg)(&gr->grtyp,&gr->xg[PI], &gr->xg[PJ], &gr->xg[D60], &gr->xg[DGRW],
		      &gr->ig[IG1], &gr->ig[IG2], &gr->ig[IG3], &gr->ig[IG4]);
      gr->hemisphere = 1;
      break;

    case 'S':
      f77name(cigaxg)(&gr->grtyp,&gr->xg[PI], &gr->xg[PJ], &gr->xg[D60], &gr->xg[DGRW],
		      &gr->ig[IG1], &gr->ig[IG2], &gr->ig[IG3], &gr->ig[IG4]);
      gr->hemisphere = 2;
      break;
      
    case 'T':
      f77name(cigaxg)(&gr->grtyp,&gr->xg[TD60], &gr->xg[TDGRW], &gr->xg[CLAT], &gr->xg[CLON],
		      &gr->ig[IG1], &gr->ig[IG2], &gr->ig[IG3], &gr->ig[IG4]);
      break;
      
    case 'X':
      fprintf(stderr,"<c_ezgdef> There is no support for grid type 'X'\n");
      return;

    default:
      fprintf(stderr,"%s %c","<c_ezgdef> Grid type not supported",&gr->grtyp);
      return;
    }
  
  return;

}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_defzone_dehors(wordint gdin, ftnfloat *x, ftnfloat *y, wordint npts, _zone *zone)
{
  ftnfloat *tmpx, *tmpy;
  wordint *tmpidx;
  wordint nhits;
  wordint i;
  
  wordint offsetleft, offsetright, ix, iy;
  
  tmpx =   (ftnfloat *) malloc(npts*sizeof(ftnfloat));
  tmpy =   (ftnfloat *) malloc(npts*sizeof(ftnfloat));
  tmpidx = (wordint   *) malloc(npts*sizeof(wordint));
  
/*
  if (groptions.degre_interp == CUBIQUE)
    {
    offsetright = 2;
    offsetleft = 1;
    }
  else
    {
    offsetright = 0;
    offsetleft = 0;
    }
*/
  
  offsetright = 0;
  offsetleft = 0;
  if (groptions.verbose > 0)
    {
    fprintf(stderr, "degre_extrap: %d offset left: %d offset right: %d\n", groptions.degre_extrap, offsetleft, offsetright);
    }
  nhits = 0;
  for (i=0; i < npts; i++)
    {
    ix = (wordint)(x[i]+0.5);
    iy = (wordint)(y[i]+0.5);
    if (ix < (1+offsetleft) || iy < (1+offsetleft) || ix > (Grille[gdin].ni-offsetright) || iy > (Grille[gdin].nj-offsetright))
      {
      tmpx[nhits]  = x[i];
      tmpy[nhits]  = y[i];
      tmpidx[nhits]=i;
      nhits++;
      }
    }
  
  zone->npts = nhits;
  zone->x =   (ftnfloat *) malloc(zone->npts*sizeof(ftnfloat));
  zone->y =   (ftnfloat *) malloc(zone->npts*sizeof(ftnfloat));
  zone->idx = (wordint *) malloc(zone->npts*sizeof(wordint));
  if (groptions.verbose > 0)
    {
    fprintf(stderr, "Nombre de points dehors: %d\n", zone->npts); 
    }
  for (i=0; i < zone->npts; i++)
    {
    zone->x[i] = tmpx[i];      
    zone->y[i] = tmpy[i];     
    zone->idx[i] = tmpidx[i];
    }
  
  free(tmpx);
  free(tmpy);
  free(tmpidx);

  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

wordint ez_defzone_est(wordint gdin, ftnfloat *x, ftnfloat *y, wordint npts, _zone *zone)
{
  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

wordint ez_defzone_nord(wordint gdin, ftnfloat *x, ftnfloat *y, wordint npts, _zone *zone)
{
  ftnfloat *tmpx, *tmpy;
  wordint nhits, i;
  wordint *tmpidx;
  wordint jmax;

  tmpx =   (ftnfloat *) malloc(npts*sizeof(ftnfloat));
  tmpy =   (ftnfloat *) malloc(npts*sizeof(ftnfloat));
  tmpidx = (wordint  *) malloc(npts*sizeof(wordint));
  
  nhits = 0;
  jmax = Grille[gdin].j2-2;
  for (i=0; i < npts; i++)
    {
    if ((int)y[i] > jmax)
      {
      tmpx[nhits] = x[i];
      tmpy[nhits] = y[i];
      tmpidx[nhits]=i;
      nhits++;
      }
    }
  
  zone->npts = nhits;
  if (nhits > 0)
    {
    zone->x = (ftnfloat *) malloc(nhits*sizeof(ftnfloat));
    zone->y = (ftnfloat *) malloc(nhits*sizeof(ftnfloat));
    zone->idx = (wordint *) malloc(nhits*sizeof(wordint));
    if (groptions.verbose > 0)
      {
      fprintf(stderr, "Nombre de points entre nj-2 et le pole %d\n", nhits); 
      }
    
    for (i=0; i < zone->npts; i++)
      {
      zone->x[i]   = tmpx[i];      
      zone->y[i]   = tmpy[i];     
      zone->idx[i] = tmpidx[i];
      }
    }
  
  free(tmpx);
  free(tmpy);
  free(tmpidx);
  
  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

wordint ez_defzone_ouest(wordint gdin, ftnfloat *x, ftnfloat *y, wordint npts, _zone *zone)
{
  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

wordint ez_defzone_polenord(wordint gdin, ftnfloat *x, ftnfloat *y, wordint npts, _zone *zone)
{
  ftnfloat *tmpx, *tmpy;
  ftnfloat latpolenord, lonpolenord, xpolenord, ypolenord;
  wordint nhits, i;
  wordint *tmpidx;
  
  /* On commence par trouver les points au pole nord */
  tmpx =   (ftnfloat *) malloc(npts*sizeof(ftnfloat));
  tmpy =   (ftnfloat *) malloc(npts*sizeof(ftnfloat));
  tmpidx = (wordint  *) malloc(npts*sizeof(wordint));
  
  nhits = 0;
  if (Grille[gdin].grtyp == 'Z' && Grille[gdin].grref == 'E')
    {
    xpolenord = 0.5 * Grille[gdin].ni;
    ypolenord = Grille[gdin].nj+0.5;
    }
  else
    {
    latpolenord = 90.0;
    lonpolenord = 0.0;
    c_gdxyfll(gdin, &xpolenord, &ypolenord,  &latpolenord, &lonpolenord, 1);
    }
  
  for (i=0; i < npts; i++)
    {
    if (fabs(y[i]-ypolenord) < 1.0e-3)
      {
      tmpx[nhits] = x[i];
      tmpy[nhits] = y[i];
      tmpidx[nhits]=i;
      nhits++;
      }
    }
  
  zone->npts = nhits;
  if (nhits > 0)
    {
    zone->x = (ftnfloat *) malloc(nhits*sizeof(ftnfloat));
    zone->y = (ftnfloat *) malloc(nhits*sizeof(ftnfloat));
    zone->idx = (wordint *) malloc(nhits*sizeof(wordint));
    if (groptions.verbose > 0)
      {
      fprintf(stderr, "Nombre de points au pole nord: %d\n", nhits); 
      }
    
    for (i=0; i < zone->npts; i++)
      {
      zone->x[i]   = tmpx[i];      
      zone->y[i]   = tmpy[i];     
      zone->idx[i] = tmpidx[i];
      }
    }
  
  free(tmpx);
  free(tmpy);
  free(tmpidx);
  return 0;
  
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

wordint ez_defzone_polesud(wordint gdin, ftnfloat *x, ftnfloat *y, wordint npts, _zone *zone)
{

  ftnfloat *tmpx, *tmpy;
  ftnfloat latpolesud, lonpolesud, xpolesud, ypolesud;
  wordint nhits, i;
  wordint *tmpidx;

  tmpx =   (ftnfloat *) malloc(npts*sizeof(ftnfloat));
  tmpy =   (ftnfloat *) malloc(npts*sizeof(ftnfloat));
  tmpidx = (wordint   *) malloc(npts*sizeof(wordint));
  
  nhits = 0;
  
  if (Grille[gdin].grtyp == 'Z' && Grille[gdin].grref == 'E')
    {
    xpolesud = 0.5 * Grille[gdin].ni;
    ypolesud = 0.5;
    }
  else
    {
    latpolesud = -90.0;
    lonpolesud = 0.0;
    c_gdxyfll(gdin, &xpolesud, &ypolesud,  &latpolesud, &lonpolesud, 1);
    }


  for (i=0; i < npts; i++)
    {
    if ((fabs(y[i]-ypolesud) < 1.0e-3))
      {
      tmpx[nhits] = x[i];
      tmpy[nhits] = y[i];
      tmpidx[nhits]=i;
      nhits++;
      }
    }
  
  zone->npts = nhits;
  
  if (nhits > 0)
    {
    zone->x = (ftnfloat *) malloc(nhits*sizeof(ftnfloat));
    zone->y = (ftnfloat *) malloc(nhits*sizeof(ftnfloat));
    zone->idx = (wordint *) malloc(nhits*sizeof(wordint));
    if (groptions.verbose > 0)
      {
      fprintf(stderr, "Nombre de points au pole sud: %d\n", nhits); 
      }
    
    for (i=0; i < zone->npts; i++)
      {
      zone->x[i] = tmpx[i];      
      zone->y[i] = tmpy[i];     
      zone->idx[i] = tmpidx[i];
      }
    }
  
  free(tmpx);
  free(tmpy);
  free(tmpidx);
  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

wordint ez_defzone_sud(wordint gdin, ftnfloat *x, ftnfloat *y, wordint npts, _zone *zone)
{
  ftnfloat *tmpx, *tmpy;
  wordint *tmpidx;
  wordint nhits, i;
  wordint jmin;

  tmpx =   (ftnfloat *) malloc(npts*sizeof(ftnfloat));
  tmpy =   (ftnfloat *) malloc(npts*sizeof(ftnfloat));
  tmpidx = (wordint  *) malloc(npts*sizeof(wordint));
  
  nhits = 0;
  jmin = Grille[gdin].j1+1;
  for (i=0; i < npts; i++)
    {
    if ((int)y[i] < jmin)
      {
      tmpx[nhits] = x[i];
      tmpy[nhits] = y[i];
      tmpidx[nhits]=i;
      nhits++;
      }
    }
  
  zone->npts = nhits;
  if (nhits > 0)
    {
    zone->x = (ftnfloat *) malloc(nhits*sizeof(ftnfloat));
    zone->y = (ftnfloat *) malloc(nhits*sizeof(ftnfloat));
    zone->idx = (wordint *) malloc(nhits*sizeof(wordint));
    if (groptions.verbose > 0)
      {
      fprintf(stderr, "Nombre de points entre le pole et nj=2 : %d\n", nhits); 
      }
    
    for (i=0; i < zone->npts; i++)
      {
      zone->x[i]   = tmpx[i];      
      zone->y[i]   = tmpy[i];     
      zone->idx[i] = tmpidx[i];
      }
    }
  
  free(tmpx);
  free(tmpy);
  free(tmpidx);
  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_defzones(_gridset *gdset)

{
  wordint i;
  wordint extrap;
  wordint gdin,gdout;
  
  wordint npts;
  
  if (iset >= 0)
    {
    if (gdset->flags & ZONES)
      {
      return 0;
      }
    }
  
  gdin = gdset->gdin;
  gdout = gdset->gdout;
  npts = Grille[gdout].ni * Grille[gdout].nj;
  extrap = NO_EXTRAP;
  switch (Grille[gdin].grtyp)
    {
    case 'N':
    case 'S':
    case 'L':
    case '!':
      extrap = EXTRAP;
      break;
      
    case '#':
    case 'Z':
    case 'Y':
      switch(Grille[gdin].grref)
	{
	case 'N':
	case 'S':
	case 'L':
	  extrap = EXTRAP;
	  break;
	  
	case 'E':
	  if (359.0 > (Grille[gdin].ax[Grille[gdin].ni-1] - Grille[gdin].ax[0]))
	    {
	    extrap = EXTRAP;
	    }
	  break;
	}
      break;
    }
  
  for (i=0; i < NZONES; i++)
    {
    gdset->zones[i].npts = 0;
    }
  
  switch (extrap)
    {
    case EXTRAP:
      ez_defzone_dehors(gdin, gdset->x, gdset->y, npts, &gdset->zones[DEHORS]);
      break;
      
    case NO_EXTRAP:
      ez_defzone_polenord(gdin, gdset->x, gdset->y, npts, &gdset->zones[POLE_NORD]);
      ez_defzone_polesud(gdin, gdset->x, gdset->y, npts, &gdset->zones[POLE_SUD]);
      ez_defzone_sud(gdin, gdset->x, gdset->y, npts, &gdset->zones[AU_SUD]);
      ez_defzone_nord(gdin, gdset->x, gdset->y, npts, &gdset->zones[AU_NORD]);
    }
  
  gdset->flags |= ZONES;
  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

void EliminerGrille(wordint gridid)
{
   wordint i;

   if (Grille[gridid].flags & LAT)
      {
      free(Grille[gridid].lat);
      free(Grille[gridid].lon);
      }
   
   if (Grille[gridid].flags & AX)
      {
      free(Grille[gridid].ax);
      free(Grille[gridid].ay);
      }

   if (Grille[gridid].ncx != NULL)
      {
      free(Grille[gridid].ncx);
      free(Grille[gridid].ncy);
      }

   memset(&Grille[gridid], 0, sizeof(_Grille));
   nGrilles--;
   if (gridid != nGrilles)
      {
      for (i=gridid; i < nGrilles; i++)
         {
         Grille[i] = Grille[i+1];
         }
      }
   memset(&Grille[nGrilles], 0, sizeof(_Grille));
   

   for (i=0; i < nsets; i++)
      {
      if (gridset[i].gdin == gridid || gridset[i].gdout == gridid)
         {
         if (gridset[i].flags & XXX)
            {
            c_ezfreegridset(i);
            memset(&gridset[i], 0, sizeof(_gridset));
            }
         }
      }
   
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


wordint c_ezfreegridset(wordint selectedset)
{
   if (gridset[selectedset].x)
      {
      free(gridset[selectedset].x);
      gridset[selectedset].x  = NULL;
      }

   if (gridset[selectedset].y)
      {
      free(gridset[selectedset].y);
      gridset[selectedset].y  = NULL;
      }

   if (gridset[selectedset].gemin.lat_rot)
      {
      free(gridset[selectedset].gemin.lat_rot);
      free(gridset[selectedset].gemin.lon_rot);
      free(gridset[selectedset].gemin.sinlat_rot);
      free(gridset[selectedset].gemin.coslat_rot);
      free(gridset[selectedset].gemin.sinlon_rot);
      free(gridset[selectedset].gemin.coslon_rot);
      free(gridset[selectedset].gemin.sinlat_true);
      free(gridset[selectedset].gemin.coslat_true);
      free(gridset[selectedset].gemin.sinlon_true);
      free(gridset[selectedset].gemin.coslon_true);
      memset(&gridset[selectedset].gemin, (int) NULL, sizeof(_geminfo));
      }

   if (gridset[selectedset].gemout.lat_rot)
      {
      free(gridset[selectedset].gemout.lat_rot);
      free(gridset[selectedset].gemout.lon_rot);
      free(gridset[selectedset].gemout.sinlat_rot);
      free(gridset[selectedset].gemout.coslat_rot);
      free(gridset[selectedset].gemout.sinlon_rot);
      free(gridset[selectedset].gemout.coslon_rot);
      free(gridset[selectedset].gemout.sinlat_true);
      free(gridset[selectedset].gemout.coslat_true);
      free(gridset[selectedset].gemout.sinlon_true);
      free(gridset[selectedset].gemout.coslon_true);
      memset(&gridset[selectedset].gemout, (int) NULL, sizeof(_geminfo));
      }


  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

/**
 ******************************************************************************
 ** Routine strclean
 ** Objet:  Enleve les espaces entourant un string
 ** Ex:     "   Skippy le kangourou   " -> "Skippy le kangourou"
 ******************************************************************************
 **/

wordint ftnstrclean(str,lenstr)
  char str[];
wordint lenstr;
{
   wordint i, j, iinit, jinit;
   
   iinit = 0;
   i = iinit;
   while (str[i] == ' ' && i < lenstr)
     i++;
   
   if (i != iinit)
     strcpy(str, str+i);
   
   jinit = lenstr-1;
   j = jinit;
   
   while (str[j] == ' ' && j >= 0)
     j--;
   
   if (j != jinit)
     str[j+1] = '\0';

   return 0;
}
/**
******************************************************************************
******************************************************************************
**/

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdgxpndaxes)(wordint *gdid, ftnfloat *ax, ftnfloat *ay)
{
   c_gdgxpndaxes(*gdid, ax, ay);
   return 0;
}

wordint c_gdgxpndaxes(wordint gdid, ftnfloat *ax, ftnfloat *ay)
{
  
  wordint nix, njy;
  wordint istart, jstart;
  
  if (!Grille[gdid].flags & AX)
    {
    fprintf(stderr, "(gdgxpndaxes) Erreur! A l'aide! Descripteurs manquants!\n");
    exit(999);
    }

  switch(Grille[gdid].grtyp)
    {
    case 'Y':
      nix = Grille[gdid].ni * Grille[gdid].nj;
      memcpy(ax, Grille[gdid].ax, nix*sizeof(ftnfloat));
      memcpy(ay, Grille[gdid].ay, nix*sizeof(ftnfloat));
      break;
      
    default:
      nix = Grille[gdid].ni;
      njy = Grille[gdid].nj;
      if (Grille[gdid].i2 == (nix+1)) istart = 1;
      if (Grille[gdid].i2 == (nix+2)) istart = 2;
      if (Grille[gdid].i2 == (nix)) istart = 0;

      if (Grille[gdid].j2 == (njy+1)) jstart = 1;
      if (Grille[gdid].j2 == (njy+2)) jstart = 2;
      if (Grille[gdid].j2 == (njy))   jstart = 0;
      memcpy(&ax[istart],Grille[gdid].ax, nix*sizeof(ftnfloat));
      memcpy(&ay[jstart],Grille[gdid].ay, njy*sizeof(ftnfloat));
      
      if (Grille[gdid].i2 == (Grille[gdid].ni+1))
	{
	ax[0] = Grille[gdid].ax[nix-2] - 360.0; 
	ax[nix] = ax[2];
	}
      
      if (Grille[gdid].i2 == (Grille[gdid].ni+2))
	{
	ax[0] = Grille[gdid].ax[nix-1] - 360.0; 
	ax[nix] = Grille[gdid].ax[1]+360.0;
	ax[nix+1] = Grille[gdid].ax[2]+360.0;
	}

      if (Grille[gdid].j2 == (Grille[gdid].nj+1))
	{
	}

      if (Grille[gdid].j2 == (Grille[gdid].nj+2))
	{
	}

    }
  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gduvfwd_turbo)(wordint *gdid, ftnfloat *uugdout, ftnfloat *vvgdout, 
                         ftnfloat *uullin, ftnfloat *vvllin, ftnfloat *latin, ftnfloat *lonin, wordint *npts)
{
   wordint icode;
   
   icode = c_gduvfwd(*gdid, uugdout, vvgdout, uullin, vvllin, latin, lonin, *npts);
   return icode;
}

wordint c_gduvfwd_turbo(wordint gdid,  ftnfloat *uugdout, ftnfloat *vvgdout, ftnfloat *uullin, ftnfloat *vvllin,
              ftnfloat *latin, ftnfloat *lonin, wordint npts)
{
   ftnfloat *xlatingf, *xloningf, *xlatingf2, *xloningf2, *uvcart, *xyz;
   wordint ni, nj;

   ni = npts;
   nj = 1;

   memcpy(uugdout, uullin, npts*sizeof(ftnfloat));
   memcpy(vvgdout, vvllin, npts*sizeof(ftnfloat));
   
   switch (Grille[gdid].grtyp)
      {
      case 'E':
        if (gridset[iset].gemout.lat_true == NULL)
           {
           fprintf(stderr, "Erreur fatale routine c_gduvfwd\n");
           fprintf(stderr, "SVP Aviser la Section informatique de RPN\n");
           exit(13);
           }
	/*        uvcart = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
        xyz    = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
        c_gfwfllw(uugdout,vvgdout, latin, lonin,
                  gridset[iset].gemout.lat, gridset[iset].gemout.lon, uvcart, xyz,
                  gridset[iset].gemout.sinlat, gridset[iset].gemout.sinlon, gridset[iset].gemout.coslat, gridset[iset].gemout.coslon,
                  gridset[iset].gemout.sinlatp, gridset[iset].gemout.sinlonp, gridset[iset].gemout.coslatp, gridset[iset].gemout.coslonp,
                  gridset[iset].gemout.r, gridset[iset].gemout.ri,
                  &ni,&nj,
                  &Grille[gdid].grtyp,
                  &Grille[gdid].ig[IG1],&Grille[gdid].ig[IG2],
                  &Grille[gdid].ig[IG3],&Grille[gdid].ig[IG4]);
        free(uvcart);
        free(xyz);
        */
        
        break;
        
        
      case '#':
      case 'Y':
      case 'Z':
        switch(Grille[gdid].grref)
           {
           case 'E':
             if (gridset[iset].gemout.lat_true == NULL)
                {
                fprintf(stderr, "Erreur fatale routine c_gduvfwd\n");
                fprintf(stderr, "Contactez Y. Chartier au plus sacrant\n");
                exit(13);
                }
	     /*             uvcart = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
             xyz    = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
             c_gfwfllw(uugdout,vvgdout, latin, lonin,
                       gridset[iset].gemout.lat, gridset[iset].gemout.lon, uvcart, xyz,
                       gridset[iset].gemout.sinlat, gridset[iset].gemout.sinlon, gridset[iset].gemout.coslat, gridset[iset].gemout.coslon,
                       gridset[iset].gemout.sinlatp, gridset[iset].gemout.sinlonp, gridset[iset].gemout.coslatp, gridset[iset].gemout.coslonp,
                       gridset[iset].gemout.r, gridset[iset].gemout.ri,
                       &ni,&nj,
                       &Grille[gdid].grtyp,
                       &Grille[gdid].ig[IG1],&Grille[gdid].ig[IG2],
                       &Grille[gdid].ig[IG3],&Grille[gdid].ig[IG4]);
             free(uvcart);
             free(xyz);
	     */
             
             break;

           default:
             f77name(ez_gdwfllw)(uugdout,vvgdout,lonin,
                              &ni,&nj,
                              &Grille[gdid].grref,
                              &Grille[gdid].igref[IG1],&Grille[gdid].igref[IG2],
                              &Grille[gdid].igref[IG3],&Grille[gdid].igref[IG4], 1);
             break;
           }
        break;
        
      default:
        f77name(ez_gdwfllw)(uugdout,vvgdout,lonin,&ni,&nj,
                         &Grille[gdid].grtyp,
                         &Grille[gdid].ig[IG1],&Grille[gdid].ig[IG2],
                         &Grille[gdid].ig[IG3],&Grille[gdid].ig[IG4], 1);
          break;
      }

   return 0;
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdwdfuv_turbo)(wordint *gdid, ftnfloat *uullout, ftnfloat *vvllout, ftnfloat *uuin, ftnfloat *vvin,
                     ftnfloat *latin, ftnfloat *lonin, wordint *npts)
{
   wordint icode;
   
   icode = c_gdwdfuv(*gdid, uullout, vvllout, uuin, vvin,latin, lonin, *npts);
   return icode;
}

wordint c_gdwdfuv_turbo(wordint gdid, ftnfloat *uullout, ftnfloat *vvllout, ftnfloat *uuin, ftnfloat *vvin, 
              ftnfloat *latin, ftnfloat *lonin, wordint npts)
{
   ftnfloat *xlatingf, *xloningf, *uvcart, *xyz;
   wordint ni, nj;

   ni = npts;
   nj = 1;

   memcpy(uullout, uuin, npts*sizeof(ftnfloat));
   memcpy(vvllout, vvin, npts*sizeof(ftnfloat));

   switch (Grille[gdid].grtyp)
      {
      case 'E':
        if (gridset[iset].gemin.lat_true == NULL)
           {
           fprintf(stderr, "Erreur fatale routine c_gdwdfuv\n");
           fprintf(stderr, "Contactez la SI de RPN le plus tt possible\n");
           exit(13);
           }
	/*
        uvcart = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
        xyz    = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
        c_llwfgfw(uullout,vvllout, latin, lonin,
                  gridset[iset].gemin.latp, gridset[iset].gemin.lonp, uvcart, xyz,
                  gridset[iset].gemin.sinlat, gridset[iset].gemin.sinlon, gridset[iset].gemin.coslat, gridset[iset].gemin.coslon,
                  gridset[iset].gemin.sinlatp, gridset[iset].gemin.sinlonp, gridset[iset].gemin.coslatp, gridset[iset].gemin.coslonp,
                  gridset[iset].gemin.r, gridset[iset].gemin.ri,
                  &ni,&nj,
                  &Grille[gdid].grtyp,
                  &Grille[gdid].ig[IG1],&Grille[gdid].ig[IG2],
                  &Grille[gdid].ig[IG3],&Grille[gdid].ig[IG4]);
        free(uvcart);
        free(xyz);
	*/
        break;
        
      case '#':
      case 'Y':
      case 'Z':
        switch(Grille[gdid].grref)
           {
           case 'E':
             if (gridset[iset].gemin.lat_true == NULL)
                {
                fprintf(stderr, "Erreur fatale routine c_gdwdfuv\n");
                fprintf(stderr, "Contactez Y. Chartier au plus sacrant\n");
                exit(13);
                }
	     /*
             uvcart = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
             xyz    = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
             c_llwfgfw(uullout,vvllout, latin, lonin,
                       gridset[iset].gemin.latp, gridset[iset].gemin.lonp, uvcart, xyz,
                       gridset[iset].gemin.sinlat, gridset[iset].gemin.sinlon, gridset[iset].gemin.coslat, gridset[iset].gemin.coslon,
                       gridset[iset].gemin.sinlatp, gridset[iset].gemin.sinlonp, gridset[iset].gemin.coslatp, gridset[iset].gemin.coslonp,
                       gridset[iset].gemin.r, gridset[iset].gemin.ri,
                       &ni,&nj,
                       &Grille[gdid].grref,
                       &Grille[gdid].igref[IG1],&Grille[gdid].igref[IG2],
                       &Grille[gdid].igref[IG3],&Grille[gdid].igref[IG4]);
             free(uvcart);
             free(xyz);
	     */
             break;

           default:
             f77name(ez_llwfgdw)(uullout,vvllout,lonin,
                              &ni,&nj,
                              &Grille[gdid].grref,
                              &Grille[gdid].igref[IG1],&Grille[gdid].igref[IG2],
                              &Grille[gdid].igref[IG3],&Grille[gdid].igref[IG4]);
             break;
           }
        break;
        
      default:
        f77name(ez_llwfgdw)(uullout,vvllout,lonin,
                         &ni,&nj,
                         &Grille[gdid].grtyp,
                         &Grille[gdid].ig[IG1],&Grille[gdid].ig[IG2],
                         &Grille[gdid].ig[IG3],&Grille[gdid].ig[IG4]);
          break;
      }
   
   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


wordint c_ezgenerate_gem_cache()
{

  /* Cette routine accelere la rotation des vents lorsque une des grilles d'une
     paire "gdin-gdout" est de type GEM
   */

   wordint gdidin, gdidout,npts;
   ftnfloat xlat1, xlon1, xlat2, xlon2;

   if (gridset[iset].use_sincos_cache == NON)
     {
     return -1;
     }

   gdidin = gridset[iset].gdin;
   gdidout= gridset[iset].gdout;
   npts = Grille[gdidout].ni*Grille[gdidout].nj;
   
   ez_calclatlon(gdidin);
   ez_calclatlon(gdidout);

   /* latp, lon_true : latlon sur la terre (coordonnees naturelles)
      lat, lon   : latlon TOURNEES
   */

   if ((((Grille[gdidout].grtyp == 'Z' || Grille[gdidout].grtyp == '#') && Grille[gdidout].grref == 'E') || (Grille[gdidout].grtyp == 'E'))
       && !(gridset[iset].gemout.flags & SINLATLON_OK))
      {
      /*      fprintf(stderr,"%d %d %d\n", gridset[iset].gemout.flags, SINLATLON_OK, !(gridset[iset].gemout.flags & SINLATLON_OK));*/
      gridset[iset].gemout.lat_true = Grille[gdidout].lat;
      gridset[iset].gemout.lon_true = Grille[gdidout].lon;
      gridset[iset].gemout.lat_rot = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemout.lon_rot = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemout.sinlat_rot  = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemout.coslat_rot  = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemout.sinlat_true = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemout.coslat_true = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemout.sinlon_rot  = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemout.coslon_rot  = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemout.sinlon_true = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemout.coslon_true = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      
      if (Grille[gdidout].grtyp == 'Z' || Grille[gdidout].grtyp == '#') 
         {
	 c_ezgfxyfll(gridset[iset].gemout.lon_true,gridset[iset].gemout.lat_true,
		     gridset[iset].gemout.lon_rot,gridset[iset].gemout.lat_rot,
		     gridset[iset].gemout.r, gridset[iset].gemout.ri,&npts,
		     &Grille[gdidout].xgref[XLAT1], &Grille[gdidout].xgref[XLON1],
		     &Grille[gdidout].xgref[XLAT2], &Grille[gdidout].xgref[XLON2]);
         }
      else
         {
	 c_ezgfxyfll(gridset[iset].gemout.lon_true,gridset[iset].gemout.lat_true,
		     gridset[iset].gemout.lon_rot,gridset[iset].gemout.lat_rot,
		     gridset[iset].gemout.r, gridset[iset].gemout.ri,&npts,
		     &Grille[gdidout].xg[XLAT1], &Grille[gdidout].xg[XLON1],
		     &Grille[gdidout].xg[XLAT2], &Grille[gdidout].xg[XLON2]);
         }

      f77name(ezsincoslatlon)(gridset[iset].gemout.lat_rot, gridset[iset].gemout.lon_rot, 
                              gridset[iset].gemout.sinlat_rot, gridset[iset].gemout.sinlon_rot,
                              gridset[iset].gemout.coslat_rot,gridset[iset].gemout.coslon_rot, &npts);
      f77name(ezsincoslatlon)(gridset[iset].gemout.lat_true, gridset[iset].gemout.lon_true,
                              gridset[iset].gemout.sinlat_true, gridset[iset].gemout.sinlon_true,
                              gridset[iset].gemout.coslat_true,gridset[iset].gemout.coslon_true, &npts);
      gridset[iset].gemout.flags |= LATLON_PRIME_OK;
      gridset[iset].gemout.flags |= SINLATLON_OK;
      /*      fprintf(stderr,"%d %d %d\n", gridset[iset].gemout.flags, SINLATLON_OK, !(gridset[iset].gemout.flags & SINLATLON_OK)); */
      }
   
   if ((((Grille[gdidin].grtyp == 'Z' || Grille[gdidin].grtyp == '#') && Grille[gdidin].grref == 'E') || Grille[gdidin].grtyp == 'E')
       && !(gridset[iset].gemin.flags & SINLATLON_OK))
      {
      gridset[iset].gemin.lat_true = Grille[gdidout].lat;
      gridset[iset].gemin.lon_true = Grille[gdidout].lon;
      gridset[iset].gemin.lat_rot = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemin.lon_rot = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemin.sinlat_rot  = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemin.coslat_rot  = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemin.sinlat_true = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemin.coslat_true = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemin.sinlon_rot  = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemin.coslon_rot  = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemin.sinlon_true = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      gridset[iset].gemin.coslon_true = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
      

      if (Grille[gdidin].grtyp == 'Z' || Grille[gdidin].grtyp == '#') 
         {
	 c_ezgfxyfll(gridset[iset].gemin.lon_true,gridset[iset].gemin.lat_true,
		     gridset[iset].gemin.lon_rot,gridset[iset].gemin.lat_rot,
		     gridset[iset].gemin.r, gridset[iset].gemin.ri,&npts,
		     &Grille[gdidin].xgref[XLAT1], &Grille[gdidin].xgref[XLON1],
		     &Grille[gdidin].xgref[XLAT2], &Grille[gdidin].xgref[XLON2]);
         }
      else
         {
	 c_ezgfxyfll(gridset[iset].gemin.lon_true,gridset[iset].gemin.lat_true,
		     gridset[iset].gemin.lon_rot,gridset[iset].gemin.lat_rot,
		     gridset[iset].gemin.r, gridset[iset].gemin.ri,&npts,
		     &Grille[gdidin].xg[XLAT1], &Grille[gdidin].xg[XLON1],
		     &Grille[gdidin].xg[XLAT2], &Grille[gdidin].xg[XLON2]);
         }

      f77name(ezsincoslatlon)(gridset[iset].gemin.lat_rot, gridset[iset].gemin.lon_rot, 
                       gridset[iset].gemin.sinlat_rot, gridset[iset].gemin.sinlon_rot,
                       gridset[iset].gemin.coslat_rot,gridset[iset].gemin.coslon_rot, &npts);
      f77name(ezsincoslatlon)(gridset[iset].gemin.lat_true, gridset[iset].gemin.lon_rot,
                       gridset[iset].gemin.sinlat_true, gridset[iset].gemin.sinlon_true,
                       gridset[iset].gemin.coslat_true,gridset[iset].gemin.coslon_true, &npts);
      gridset[iset].gemin.flags |= LATLON_PRIME_OK;
      gridset[iset].gemin.flags |= SINLATLON_OK;
      }

  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


wordint f77name(ezgenpole)(ftnfloat *vpolnor, ftnfloat *vpolsud, ftnfloat *fld,
                           wordint *ni, wordint *nj, wordint *vecteur, 
                           char *grtyp, wordint *hem)
{
   return c_ezgenpole(vpolnor, vpolsud, fld, *ni, *nj, *vecteur, grtyp, *hem);

}

wordint c_ezgenpole(ftnfloat *vpolnor, ftnfloat *vpolsud, ftnfloat *fld,
                           wordint ni, wordint nj, wordint vecteur, 
                           char *grtyp, wordint hem)
{
   wordint lni, lnj, lvecteur, lhem;
   ftnfloat *x, *y, *z, *lat, *lon, *gausslat;

   lni = ni; 
   lnj = nj; 
   lvecteur = vecteur; 
   lhem = hem;

   x =    (ftnfloat *) malloc(2*lni*sizeof(ftnfloat));
   y =    (ftnfloat *) malloc(2*lni*sizeof(ftnfloat));
   z =    (ftnfloat *) malloc(2*lni*sizeof(ftnfloat));
   lat =  (ftnfloat *) malloc(2*lni*sizeof(ftnfloat));
   lon =  (ftnfloat *) malloc(2*lni*sizeof(ftnfloat));
   gausslat = (ftnfloat *) malloc(2*lnj*sizeof(ftnfloat));
   
   f77name(ez_genpole)(vpolnor, vpolsud, fld, &lni, &lnj, &lvecteur, grtyp, &lhem,
                       x,y,z,lat,lon,gausslat, &groptions.degre_interp);

   free(x);
   free(y);
   free(z);
   free(lat);
   free(lon);
   free(gausslat);

   return 0;

}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


void f77name(ezgfllfxy)(ftnfloat *lonp, ftnfloat *latp,
		    ftnfloat *lon, ftnfloat *lat, 
		    ftnfloat *r, ftnfloat *ri, wordint *npts, 
		    ftnfloat *xlat1, ftnfloat *xlon1, ftnfloat *xlat2, ftnfloat *xlon2)
{
  c_ezgfllfxy(lonp, latp, lon, lat, r, ri, npts, xlat1, xlon1, xlat2, xlon2);
}



void  c_ezgfllfxy(ftnfloat *lonp, ftnfloat *latp,
                 ftnfloat *lon, ftnfloat *lat, 
                 ftnfloat *r, ftnfloat *ri, wordint *npts, 
                 ftnfloat *xlat1, ftnfloat *xlon1, ftnfloat *xlat2, ftnfloat *xlon2)
{
   ftnfloat *cart, *carot;
   wordint trois = 3;

   cart = (ftnfloat *) malloc(3* *npts*sizeof(ftnfloat));
   carot = (ftnfloat *) malloc(3* *npts*sizeof(ftnfloat));

   f77name(ez_crot)( r, ri, xlon1, xlat1, xlon2, xlat2);
   f77name(ez_lac) (cart,lon,lat, npts);
   f77name(mxm)(ri,&trois,cart,&trois,carot,npts);
   f77name(ez_cal)(lonp,latp,carot,npts);

   free(cart);
   free(carot);

}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

void f77name(ez_gfwfllw)(ftnfloat *uullout, ftnfloat *vvllout, ftnfloat *latin, ftnfloat *lonin,
			 ftnfloat *xlatingf, ftnfloat *xloningf, 
			 wordint *ni, wordint *nj,
			 char *grtyp, wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4)
{
  c_ezgfwfllw(uullout, vvllout, latin, lonin, xlatingf, xloningf, 
	      ni, nj, grtyp, ig1, ig2, ig3, ig4);
    }

void c_ezgfwfllw(ftnfloat *uullout, ftnfloat *vvllout, ftnfloat *latin, ftnfloat *lonin,
                  ftnfloat *xlatingf, ftnfloat *xloningf, 
                  wordint *ni, wordint *nj,
                  char *grtyp, wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4)
{

  /*
    gfwfllw -> GeF Winds From LatLon Winds
    Ce sous-programme effectue la rotation des vents d'un systeme de coordonne
    non tourne a un systeme de coordonnee tourne.
    latin, lonin sont les latlons vraies
    xlatingf, xloningf sont les latlons sur la grille tournee
  */

  wordint zero = 0;
  wordint npts = *ni * *nj;
  wordint trois = 3;
  ftnfloat r[9], ri[9], xlon1, xlat1, xlon2, xlat2;
  ftnfloat *uvcart, *xyz;
  char grtypl[2];

  uvcart = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
  xyz    = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
  
  
  f77name(cigaxg)(grtyp, &xlat1, &xlon1, &xlat2, &xlon2, ig1, ig2, ig3, ig4, 1);
  f77name(ez_crot)(r, ri, &xlon1, &xlat1, &xlon2, &xlat2);
  
  grtypl[0] = 'L';
  f77name(ez_gdwfllw)(uullout,vvllout,lonin,ni,nj,grtypl, &zero, &zero, &zero, &zero, 1);
  
  f77name(ez_uvacart)(xyz, uullout, vvllout, lonin, latin, ni, nj);
  f77name(mxm)(r, &trois, xyz, &trois, uvcart, &npts);
  f77name(ez_cartauv)(uullout, vvllout, uvcart, xloningf, xlatingf, ni, nj);

  free(uvcart);
  free(xyz);
}


/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

void  c_gfwfllw_turbo(ftnfloat *uullout, ftnfloat *vvllout, ftnfloat *latin, ftnfloat *lonin,
                  ftnfloat *xlatingf, ftnfloat *xloningf, ftnfloat *uvcart, ftnfloat *xyz,
                  ftnfloat *sinlat, ftnfloat *sinlon, ftnfloat *coslat, ftnfloat *coslon,
                  ftnfloat *sinlatp, ftnfloat *sinlonp, ftnfloat *coslatp, ftnfloat *coslonp,
                  ftnfloat *r, ftnfloat *ri,
                  wordint *ni, wordint *nj,
                  char *grtyp, wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4)
{
   wordint zero = 0;
   wordint npts = *ni * *nj;
   wordint trois = 3;
   char grtypl[2];

   grtypl[0] = 'L';
   f77name(ez_gdwfllw)(uullout,vvllout,lonin,ni,nj,grtypl, &zero, &zero, &zero, &zero, 1);

   f77name(ez_uvacart_turbo)(xyz, uullout, vvllout, sinlatp, sinlonp, coslatp, coslonp, ni, nj);
   f77name(mxm)(r, &trois, xyz, &trois, uvcart, &npts);
   f77name(ez_cartauv_turbo)(uullout, vvllout, uvcart, sinlat, sinlon, coslat, coslon, ni, nj);
}


/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


void f77name(ezgfxyfll)(ftnfloat *lonp, ftnfloat *latp,
		    ftnfloat *lon, ftnfloat *lat, 
		    ftnfloat *r, ftnfloat *ri, wordint *npts, 
		    ftnfloat *xlat1, ftnfloat *xlon1, ftnfloat *xlat2, ftnfloat *xlon2)
{
  c_ezgfxyfll(lonp, latp, lon, lat, r, ri, npts, xlat1, xlon1, xlat2, xlon2);
}

void c_ezgfxyfll(ftnfloat *lonp, ftnfloat *latp,
		    ftnfloat *lon, ftnfloat *lat, 
		    ftnfloat *r, ftnfloat *ri, wordint *npts, 
		    ftnfloat *xlat1, ftnfloat *xlon1, ftnfloat *xlat2, ftnfloat *xlon2)
{
  ftnfloat *cart, *carot;
  wordint trois = 3;
  
  cart = (ftnfloat *) malloc(3* *npts*sizeof(ftnfloat));
  carot = (ftnfloat *) malloc(3* *npts*sizeof(ftnfloat));
  
  f77name(ez_crot)( r, ri, xlon1, xlat1, xlon2, xlat2);
  f77name(ez_lac) (cart,lonp,latp, npts);
  f77name(mxm)(r,&trois,cart,&trois,carot,npts);
  f77name(ez_cal)(lon,lat,carot,npts);
  
  free(cart);
  free(carot);
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

wordint c_ezidentifygrid(wordint ni, wordint nj, char *grtyp, char *grref,
	     wordint ig1, wordint ig2, wordint ig3, wordint ig4, ftnfloat *ax, ftnfloat *ay)
{
  wordint i;
  wordint gdid;
  wordint res1, res2;
  char typeGrille;
  _Grille *gr;
  
  gdid = -1;
  typeGrille = grtyp[0];
  
  for (i=0; i < nGrilles; i++)
    {
    gr = &Grille[i];
    switch (typeGrille)
      {
      case '#':
      case 'Z':
	if (typeGrille == gr->grtyp && grref[0] == gr->grref &&
	    ni == gr->ni && nj == gr->nj &&
	    ig1 == gr->igref[IG1]  && ig2 == gr->igref[IG2] && 
	    ig3 == gr->igref[IG3]  && ig4 == gr->igref[IG4])
	  {
	  if (gr->ax != NULL)
	    {
	    res1 = memcmp(ay, gr->ay, nj*sizeof(ftnfloat));
	    res2 = memcmp(ax, gr->ax, ni*sizeof(ftnfloat));
	    if (res1 == 0 && res2 == 0)
	      {
	      gdid = i;
	      }
	    }
	  }
	break;
	
      case 'Y':
	if (typeGrille == gr->grtyp && grref[0] == gr->grref &&
	    ni == gr->ni && nj == gr->nj &&
	    ig1 == gr->igref[IG1]  && ig2 == gr->igref[IG2] && 
	    ig3 == gr->igref[IG3]  && ig4 == gr->igref[IG4])
	  {
	  if (gr->lat != NULL)
	    {
	    res1 = memcmp(ay, gr->lat, ni*nj*sizeof(ftnfloat));
	    res2 = memcmp(ax, gr->lon, ni*nj*sizeof(ftnfloat));
	    if (res1 == 0 && res2 == 0)
	      {
	      gdid = i;
	      }
	    }
	  }
	break;
	
      default:
	if (typeGrille == gr->grtyp && ni == gr->ni && nj == gr->nj &&
	    ig1 == gr->ig[IG1]  && ig2 == gr->ig[IG2]  &&
	    ig3 == gr->ig[IG3]  && ig4 == gr->ig[IG4])
	  {
	  gdid = i;
	  }
	break;
      }
    }
  
  if (gdid == -1)
    {
    gdid = i;
    nGrilles++;
    }
  else
    {
    return gdid;
    }

  gr = &Grille[gdid];
  if (nGrilles > NMAXGRIDS)
    {
    fprintf(stderr, "<ezgdef_fmem> : Too many defined grids. \n");
    fprintf(stderr, "<ezgdef_fmem> : No way but to abort. \n");
    exit(13);
    }
  else
    {
    gr->flags = 0;
    }
  
  gr->grtyp = typeGrille;
  if (grref != NULL)
    {
    gr->grref = grref[0];
    gr->igref[IG1] = ig1;
    gr->igref[IG2] = ig2;
    gr->igref[IG3] = ig3;
    gr->igref[IG4] = ig4;
    }
  gr->ni  = ni;
  gr->nj  = nj;
  gr->ig[IG1] = ig1;
  gr->ig[IG2] = ig2;
  gr->ig[IG3] = ig3;
  gr->ig[IG4] = ig4;
  gr->axe_y_inverse = 0;
  gr->needs_expansion = NON;
  
  return gdid;
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <rpnmacros.h>

static ftnfloat *eziglat = NULL;
static ftnfloat *eziglon = NULL;

f77name(ez_igscint)(ftnfloat *zo, wordint *li, wordint *lj, ftnfloat *xlat, ftnfloat *xlon, 
                    ftnfloat *zi, wordint *ni, wordint *nj, 
                    char* grtyp, char *grref, wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4, 
                    ftnfloat *sym, ftnfloat *ax, ftnfloat *ay, wordint lengrtyp, wordint lengrref)
{
  wordint gdin, gdout, npts, i, ier;
  char ogrtyp[2], ogrref[2];
  wordint oig1, oig2, oig3, oig4;
  ftnfloat xg1, xg2, xg3, xg4;
  ftnfloat *tmplon;
  
  ftnstrclean(grtyp,lengrtyp);
  ftnstrclean(grref,lengrref);
  
  npts = *li * *lj;
  tmplon = (ftnfloat *) malloc(npts * sizeof(ftnfloat));
  for (i=0; i < npts; i++)
    {
    tmplon[i] = xlon[i] < 0.0 ? xlon[i] + 360.0 : xlon[i];
    }
  
  gdin  = c_ezgdef_fmem(*ni, *nj, grtyp, grref,  *ig1,  *ig2,  *ig3,  *ig4, ax,   ay);
  ier = c_gdllsval(gdin, zo, zi, xlat, tmplon, npts);
  free(tmplon);
  return 0;
}

f77name(ez_rgscint)(ftnfloat *zo, wordint *li, wordint *lj, ftnfloat *xlat, ftnfloat *xlon, 
                    ftnfloat *zi, wordint *ni, wordint *nj, 
                    char* grtyp, wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4, ftnfloat *sym,
                    wordint lengrtyp)
{
  wordint gdin, gdout;
  char ogrtyp[2], ogrref[2], igrtyp[2],igrref[2];
  wordint oig1, oig2, oig3, oig4, npts, i, ier;
  ftnfloat xg1, xg2, xg3, xg4;
  ftnfloat *tmplon;
  
  npts = *li * *lj;
  tmplon = (ftnfloat *) malloc(npts * sizeof(ftnfloat));
  for (i=0; i < npts; i++)
    {
    tmplon[i] = xlon[i] < 0.0 ? xlon[i] + 360.0 : xlon[i];
    }
  
  gdin  = c_ezgdef_fmem(*ni, *nj, grtyp, NULL,  *ig1,  *ig2,  *ig3,  *ig4, NULL, NULL);
  ier = c_gdllsval(gdin, zo, zi, xlat, tmplon, npts);
  free(tmplon);
  return 0;
}

f77name(ez_iguvint)(ftnfloat *spdo, ftnfloat *psio, wordint *li, wordint *lj, ftnfloat *xlat, ftnfloat *xlon, 
                    ftnfloat *ui, ftnfloat *vi, wordint *ni, wordint *nj, 
                    char* grtyp, char *grref, wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4, 
                    ftnfloat *sws, ftnfloat *ax, ftnfloat *ay, wordint lengrtyp, wordint lengrref)
{
  ftnfloat *tmplon;
  wordint gdin, gdout, i, ier;
  char ogrtyp[2], ogrref[2];
  ftnfloat xg1, xg2, xg3, xg4;
  wordint npts, oig1, oig2, oig3, oig4;
  
  npts = *li * *lj;
  tmplon = (ftnfloat *) malloc(npts * sizeof(ftnfloat));
  for (i=0; i < npts; i++)
    {
    tmplon[i] = xlon[i] < 0.0 ? xlon[i] + 360.0 : xlon[i];
    }
  
  ftnstrclean(grtyp,lengrtyp);
  ftnstrclean(grref,lengrref);
  gdin  = c_ezgdef_fmem(*ni,*nj, grtyp, grref,  *ig1,  *ig2,  *ig3,  *ig4, ax, ay);
  ier = c_gdllwdval(gdin, spdo, psio, ui, vi, xlat, tmplon, npts);
  free(tmplon);
}

f77name(ez_rguvint)(ftnfloat *spdo, ftnfloat *psio, wordint *li, wordint *lj, ftnfloat *xlat, ftnfloat *xlon, 
                    ftnfloat *ui, ftnfloat *vi, wordint *ni, wordint *nj, 
                    char* grtyp, wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4, ftnfloat *sws, wordint lengrtyp)
{
  ftnfloat *tmplon;
  wordint gdin, gdout, i, ier;
  char ogrtyp[2], ogrref[2];
  ftnfloat xg1, xg2, xg3, xg4;
  wordint npts, oig1, oig2, oig3, oig4;
  
  npts = *li * *lj;
  tmplon = (ftnfloat *) malloc(npts * sizeof(ftnfloat));
  for (i=0; i < npts; i++)
    {
    tmplon[i] = xlon[i] < 0.0 ? xlon[i] + 360.0 : xlon[i];
    }
  
  ftnstrclean(grtyp,lengrtyp);
  gdin  = c_ezgdef(*ni,*nj, grtyp, NULL,  *ig1,  *ig2,  *ig3,  *ig4, NULL, NULL);
  ier = c_gdllwdval(gdin, spdo, psio, ui, vi, xlat, tmplon, npts);
  free(tmplon);
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_interp(ftnfloat *zout, ftnfloat *zin, wordint iset)
{
   wordint ni_in, nj_in, ni_out, nj_out, ninj_out ,gdin, gdout;
   
   if (gridset[iset].flags & XXX)
      {
      gdin  = gridset[iset].gdin;
      gdout = gridset[iset].gdout;
      
      ni_in =  Grille[gdin].ni;
      nj_in =  Grille[gdin].nj;
      
      ni_out = Grille[gdout].ni;
      nj_out = Grille[gdout].nj;
      ninj_out = ni_out * nj_out;
      
      c_gdinterp(zout, zin, gdin, gridset[iset].x,gridset[iset].y, ninj_out);
      }
   return 0;   
}

#include "ezscint.h"
#include "ez_funcdef.h"

void RemplirDeBlancs(char str[],wordint longueur);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint LireEnrPositionnels(_Grille *gr, wordint iunit, wordint ip1, wordint ip2, wordint ip3, wordint ip4)
{
   wordint moins1 = -1;
   wordint cle;
   wordint niy, njy, nky, nix, njx, nkx;
   wordint ier, ier1, ier2;
   wordint clex, bidon,ig1ref,ig2ref,ig3ref,ig4ref;
   char grref[2];
   ftnfloat *ax,*ay;

   char nomvarx[5], typvarx[3], etikx[13];
   char nomvary[5], typvary[3], etiky[13];
   wordint i,j;

   wordint dateo, deet, npas, nbits;
   wordint intip1, intip2, intip3,intip4,intiunit,offsetx,offsety;

   intip1 = ip1;
   intip2 = ip2;
   intip3 = ip3;
   intip4 = ip4;
   
   if (gr->grtyp == '#') 
     {
     intip3 = -1;
     }

   strcpy(nomvary, "^^");
   strcpy(nomvarx, ">>");
   strcpy(etikx, "            ");
   strcpy(etiky, "            ");
   strcpy(typvarx, "  ");
   strcpy(typvary, "  ");
   
   intiunit = iunit;


#ifdef FICHSTD98   
   ier1 = f77name(fstinf)(&intiunit, &niy, &njy, &nky, &moins1, etiky, 
			  &intip1, &intip2, &intip3, typvary, nomvary,12,2,4);
   ier2 = f77name(fstinf)(&intiunit, &nix, &njx, &nkx, &moins1, etikx, 
			  &intip1, &intip2, &intip3, typvarx, nomvarx,12,2,4);
#else
   ier1 = f77name(fstinf)(&intiunit, &niy, &njy, &nky, &moins1, etiky, 
                          &intip1, &intip2, &intip3, typvary, nomvary,8,1,2);
   ier2 = f77name(fstinf)(&intiunit, &nix, &njx, &nkx, &moins1, etikx, 
                          &intip1, &intip2, &intip3, typvarx, nomvarx,8,1,2);
#endif
   
   if (ier1 < 0 || ier2 < 0)
     {
     fprintf(stderr,"<LireEnrPositionnels>: Positional records ^^ and >> not found. Exiting...\n\n");
     exit(333);
     }
   else
     {
#ifdef FICHSTD98
     ier = f77name(fstprm)(&ier1, &dateo, &deet, &npas, &niy, &njy, &nky, &nbits,
			   &bidon, &intip1, &intip2, &intip3, typvary, nomvary, etiky,
			   grref, &ig1ref, &ig2ref, &ig3ref, &ig4ref, &bidon, &bidon, &bidon,
			   &bidon, &bidon, &bidon, &bidon,2,4,12,2);
#else
     ier = f77name(fstprm)(&ier1, &dateo, &deet, &npas, &niy, &njy, &nky, &nbits,
			   &bidon, &intip1, &intip2, &intip3, typvary, nomvary, etiky,
			   grref, &ig1ref, &ig2ref, &ig3ref, &ig4ref, &bidon, &bidon, &bidon,
			   &bidon, &bidon, &bidon, &bidon,1,2,8,1);
#endif
     

     if (niy == nix && njy == njx)
       {
       gr->grtyp = 'Y';
       }
     else
       {
       if (gr->ni < nix)
	 {
	 gr->grtyp = '#';
	 }
       else
	 {
	 gr->grtyp = 'Z';
	 }
       }
     
     
     if (grref[0] != 'N' && grref[0] != 'S' &&  grref[0] != 'L' && grref[0] != 'E')
       {
       fprintf(stderr,"<LireEnrPositionnels>: Unknown reference grid. Exiting...\n");
       exit(333);
       }
     else
       {
       ay = (ftnfloat *) malloc(niy*njy*sizeof(ftnfloat));
       ier = f77name(fstluk)(ay, &ier1, &niy, &njy, &nky);
       
       ax = (ftnfloat *) malloc(nix*njx*sizeof(ftnfloat));
       clex = f77name(fstluk)(ax, &ier2, &nix, &njx, &nkx);
       
       switch (gr->grtyp)
	 {
	 case 'Y':
	 case 'Z':
	   gr->ax = (ftnfloat *) malloc(nix*njx*sizeof(ftnfloat));
	   gr->ay = (ftnfloat *) malloc(niy*njy*sizeof(ftnfloat));
	   memcpy(gr->ax,ax,nix*njx*sizeof(ftnfloat));
	   memcpy(gr->ay,ay,niy*njy*sizeof(ftnfloat));
	   break;

	 case '#':
	   gr->ax = (ftnfloat *) malloc(gr->ni*sizeof(ftnfloat));
	   gr->ay = (ftnfloat *) malloc(gr->nj*sizeof(ftnfloat));
	   offsetx = ip3 - 1;
	   offsety = ip4 - 1;
	   for (j=0; j < gr->nj; j++)
	     {
	     gr->ay[j] = ay[j+offsety];
	     }
	   
	   for (i=0; i < gr->ni; i++)
	     {
	     gr->ax[i] = ax[i+offsetx];
	     }
	   break;
	 }

       if (grref[0] == 'L')
	 {
	 for (i=0; i < nix*njx; i++)
	   {
	   if (gr->ax[i] < 0.0)
	     {
	     gr->ax[i] = gr->ax[i] += 360.0;
	     }
	   }
	 }
       
#ifdef FICHSTD98
       ier = f77name(fstprm)(&ier2, &dateo, &deet, &npas, &nix, &njx, &nkx, &nbits,
			     &bidon, &intip1, &intip2, &intip3, typvarx, nomvarx, etikx,
			     grref, &ig1ref, &ig2ref, &ig3ref, &ig4ref, &bidon, &bidon, &bidon,
			     &bidon, &bidon, &bidon, &bidon,2,4,12,2);
#else
       ier = f77name(fstprm)(&ier2, &dateo, &deet, &npas, &nix, &njx, &nkx, &nbits,
			     &bidon, &intip1, &intip2, &intip3, typvarx, nomvarx, etikx,
			     grref, &ig1ref, &ig2ref, &ig3ref, &ig4ref, &bidon, &bidon, &bidon,
			     &bidon, &bidon, &bidon, &bidon,1,2,8,1);
#endif
       
       
       
       
       switch (gr->grtyp)
	 {
	 case 'Y':
	 case 'Z':
	   gr->ip1     = ig1ref;
	   gr->ip2     = ig2ref;
	   gr->ip3     = ig3ref;
	   break;
	   
	 case '#':
	   gr->ip1     = ig1ref;
	   gr->ip2     = ig2ref;
	   gr->ip3     = -1;
	   break;
	 }
        
       gr->ig[IG1]    =  ip1;
       gr->ig[IG2]    =  ip2;
       gr->ig[IG3]    =  ip3;
       gr->ig[IG4]    =  ip4;

       gr->xg[IG1]    =  0.0;
       gr->xg[IG2]    =  0.0;
       gr->xg[IG3]    =  0.0;
       gr->xg[IG4]    =  0.0;

       gr->grref   = grref[0];
       
       gr->igref[IG1]  = ig1ref;      
       gr->igref[IG2]  = ig2ref;      
       gr->igref[IG3]  = ig3ref;      
       gr->igref[IG4]  = ig4ref;

       f77name(cigaxg)(&(gr->grref),&gr->xgref[XLAT1], &gr->xgref[XLON1], &gr->xgref[XLAT2], &gr->xgref[XLON2],
		       &gr->igref[IG1], &gr->igref[IG2], &gr->igref[IG3], &gr->igref[IG4]);
       
       gr->deet    = deet;
       gr->npas    = npas;
       gr->nbits   = nbits;
       gr->date    = dateo;
       
       strcpy(gr->nomvarx, nomvarx);
       strcpy(gr->typvarx, typvarx);
       strcpy(gr->etiketx, etikx);
       strcpy(gr->nomvary, nomvary);
       strcpy(gr->typvary, typvary);
       strcpy(gr->etikety, etiky);
       
       RemplirDeBlancs(gr->nomvarx, 5);
       RemplirDeBlancs(gr->typvarx, 3);
       RemplirDeBlancs(gr->etiketx, 13);
       RemplirDeBlancs(gr->nomvary, 5);
       RemplirDeBlancs(gr->typvary, 3);
       RemplirDeBlancs(gr->etikety, 13);
       
       gr->flags  |= AX;
       }
     free(ax);
     free(ay);
     }
   return 0;
}

void RemplirDeBlancs(char str[],wordint longueur)
{
  wordint i;
  
  for (i=strlen(str);i < longueur; i++)
    {
    str[i] = ' ';
    }
  str[longueur - 1] = '\0';
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
void c_llfgr(ftnfloat *lat, ftnfloat *lon, ftnfloat *x, ftnfloat *y, wordint npts,
	     ftnfloat latOrigine, ftnfloat lonOrigine, ftnfloat deltaLat, ftnfloat deltaLon)
{
  wordint i;
  
  for (i=0; i < npts; i++)
    {
      lon[i] = lonOrigine + deltaLon * (x[i]-1.0);
      lon[i]=  fmod(fmod(lon[i],360.0)+360.0,360.0);
      lat[i] = latOrigine + deltaLat * (y[i]-1.0);
    }
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


void f77name(ez_llwfgfw)(ftnfloat *uullout, ftnfloat *vvllout, ftnfloat *latin, ftnfloat *lonin,
                  ftnfloat *xlatingf, ftnfloat *xloningf, wordint *ni,wordint *nj,
                  char *grtyp,wordint *ig1,wordint *ig2,wordint *ig3,wordint *ig4)
{
  c_ezllwfgfw(uullout, vvllout, latin, lonin, xlatingf, xloningf, ni,nj,
	      grtyp,ig1,ig2,ig3,ig4);
}


void c_ezllwfgfw(ftnfloat *uullout, ftnfloat *vvllout, ftnfloat *latin, ftnfloat *lonin,
                  ftnfloat *xlatingf, ftnfloat *xloningf, wordint *ni,wordint *nj,
                  char *grtyp,wordint *ig1,wordint *ig2,wordint *ig3,wordint *ig4)
{
  /*
    llwfgfw -> LatLon Winds From GeF Winds
    Ce sous-programme effectue la rotation des vents d'un systeme de coordonne
    tourne a un systeme de coordonnee non tourne.
    latin, lonin sont les latlons vraies
    xlatingf, xloningf sont les latlons sur la grille tournee
  */

  wordint zero = 0;
  wordint npts = *ni * *nj;
  wordint trois = 3;
  ftnfloat r[9], ri[9], xlon1, xlat1, xlon2, xlat2;
  ftnfloat *uvcart, *xyz;
  char grtypl[2];

 
  uvcart = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
  xyz    = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
  
  
  f77name(cigaxg)(grtyp, &xlat1, &xlon1, &xlat2, &xlon2, ig1, ig2, ig3, ig4, 1);
  f77name(ez_crot)(r, ri, &xlon1, &xlat1, &xlon2, &xlat2);
  
  f77name(ez_uvacart)(xyz, uullout, vvllout, xloningf, xlatingf, ni, nj); 
  f77name(mxm)(ri, &trois, xyz, &trois, uvcart, &npts);
  f77name(ez_cartauv)(uullout, vvllout, uvcart, lonin, latin, ni, nj); 
  grtypl[0] = 'L';
  f77name(ez_llwfgdw)(uullout,vvllout,xloningf,ni,nj,grtypl, &zero, &zero, &zero, &zero, 1);

  free(uvcart);
  free(xyz); 
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


void c_llwfgfw_turbo(ftnfloat *uullout, ftnfloat *vvllout, ftnfloat *latin, ftnfloat *lonin,
		     ftnfloat *xlatingf, ftnfloat *xloningf, ftnfloat *uvcart, ftnfloat *xyz,
		     ftnfloat *sinlat, ftnfloat *sinlon, ftnfloat *coslat, ftnfloat *coslon,
		     ftnfloat *sinlatp, ftnfloat *sinlonp, ftnfloat *coslatp, ftnfloat *coslonp,
		     ftnfloat *r, ftnfloat *ri,
		     wordint *ni,wordint *nj,
		     char *grtyp,wordint *ig1,wordint *ig2,wordint *ig3,wordint *ig4)
{
  wordint zero = 0;
  wordint npts = *ni * *nj;
  wordint trois = 3;
  char grtypl[2];
  
  f77name(ez_uvacart_turbo)(xyz, uullout, vvllout, sinlat, sinlon, coslat, coslon, ni, nj); 
  f77name(mxm)(ri, &trois, xyz, &trois, uvcart, &npts);
  
  f77name(ez_cartauv_turbo)(uullout, vvllout, uvcart, sinlatp, sinlonp, coslatp, coslonp, ni, nj); 
  grtypl[0] = 'L';
  f77name(ez_llwfgdw)(uullout,vvllout,xloningf,ni,nj,grtypl, &zero, &zero, &zero, &zero, 1);
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <rpnmacros.h>

f77name(ez_rgll2gd)(ftnfloat *z1, ftnfloat *z2, ftnfloat *xlon, wordint *ni, wordint *nj, 
                    char *grtyp, wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4,wordint lengrtyp)
{
  ftnfloat *tmplat;
  wordint gdid, npts,n;
  char lgrtyp[2];
  
  npts = *ni * *nj;
  tmplat = (float *)malloc(npts*sizeof(ftnfloat));
  for (n=0; n < npts; n++)
    {
    tmplat[n] = 0.0;
    }

  ftnstrclean(grtyp,lengrtyp);
  strcpy(lgrtyp, grtyp);

  gdid  = c_ezqkdef(*ni,*nj, lgrtyp, *ig1,  *ig2,  *ig3,  *ig4, 0);

  c_gduvfwd(gdid, z1, z2, z1, z2, tmplat, xlon, npts);
  
  
  free(tmplat);
  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


wordint c_ezsincoslatlon(ftnfloat *lat, ftnfloat *lon,
                         ftnfloat *sinlat, ftnfloat *sinlon, ftnfloat *coslat, ftnfloat *coslon, wordint npts)
{
   wordint i,gdidout;
   ftnfloat dar = M_PI/180.0;
   
   for (i=0; i < npts; i++)
      {
      sinlat[i]  = sin(dar*lat[i]);
      coslat[i]  = cos(dar*lat[i]);
      sinlon[i]  = sin(dar*lon[i]);
      coslon[i]  = cos(dar*lon[i]);
      }

   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


wordint f77name(ezgetgdout)()
{
   return c_ezgetgdout();
}
wordint c_ezgetgdin()
{
   if (iset >= 0)
     return gridset[iset].gdin;
   else
     return iset;
}

wordint c_ezgetgdout()
{
   if (iset >= 0)
     return gridset[iset].gdout;
   else
     return iset;
}

wordint c_ezsetgdout(gdout)
wordint gdout;
{
   gridset[iset].gdout = gdout;
   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint ez_calcxpncof(wordint gdid)
{
  ez_xpncof(&Grille[gdid].i1,&Grille[gdid].i2,&Grille[gdid].j1,&Grille[gdid].j2,&Grille[gdid].extension,
	    Grille[gdid].ni,Grille[gdid].nj,Grille[gdid].grtyp,Grille[gdid].grref,
	    Grille[gdid].ig[IG1], Grille[gdid].ig[IG2], Grille[gdid].ig[IG3], Grille[gdid].ig[IG4],
	    groptions.symmetrie, Grille[gdid].ax, Grille[gdid].ay);
  
   return 0;
   
}
      
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
void ez_xpncof(wordint *i1, wordint *i2, wordint *j1, wordint *j2, wordint *extension,
            wordint ni,wordint nj,char grtyp, char grref,
            wordint ig1,wordint ig2,wordint ig3,wordint ig4,wordint sym, ftnfloat *ax, ftnfloat *ay)
{
  
  *i1 = 1;
  *i2 = ni;
  switch (grtyp)
      {
      case '!':
      case 'L':
      case 'N':
      case 'S':
      case 'T':
        *j1 = 1;
        *j2 = nj;
	*extension = 0;
        break;
	
      case 'A':
      case 'G':
	*extension = 2;
        switch (ig1)
	  {
	  case GLOBAL:
	    *j1 = 1;
	    *j2 = nj;
	    break;
	    
	  case NORD:
	    *j1 = -nj+1;
	    *j2 =  nj;
	    break;

	  case SUD:
	    *j1 = 1;
	    *j2 =  2 * nj;
	    break;
	  }
        break;
	
      case 'B':
	*extension = 1;
        switch (ig1)
	  {
	  case GLOBAL:
	    *j1 = 1;
	    *j2 = nj;
	    break;
	    
	  case NORD:
	    *j1 = -nj+2;
	    *j2 = nj;
	    break;


	  case SUD:
	    *j1 = 1;
	    *j2 = 2 * nj - 1;
	    break;
	  }
        break;
	
      case 'E':
        *j1 = 1;
        *j2 = nj;
        break;
	
      case '#':
      case 'Z':
        switch (grref)
	  {
	  case 'E':
	    *j1 = 1;
	    *j2 = nj;
	    if ((ax[ni-1]-ax[0]) < 359.0)
	      {
	      *extension = 0;
	      }
	    else
	      {
	      *extension = 1;
	      }
	    break;
	    
	  default:
	    *j1 = 1;
	    *j2 = nj;
	    *extension = 0;
	    break;
	  }
	break;
	
      default:
	*j1 = 1;
	*j2 = nj;
	*extension = 0;
	break;
      }
  
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

void ez_xpnsrcgd(wordint gdid, ftnfloat *zout, ftnfloat *zin)
{
   _Grille gr;
   
   gr = Grille[gdid];
   
   switch (gr.grtyp)
     {
     case 'A':
     case 'G':
       f77name(ez_xpngdag2)(zout,zin,&gr.ni,&gr.nj,&gr.j1,&gr.j2,&gr.ig[IG1],&groptions.symmetrie);
       break;

     case 'B':
       f77name(ez_xpngdb2)(zout,zin,&gr.ni,&gr.nj,&gr.j1,&gr.j2,&gr.ig[IG1],&groptions.symmetrie);
       break;

     default:
       break;
     }
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezdefset)(wordint *gdout, wordint *gdin)
{
   wordint icode;
   
   icode = c_ezdefset(*gdout, *gdin);
   return icode;
}

wordint c_ezdefset(wordint gdout, wordint gdin)
{
   /* d'abord trouver si l'ensemble est deja defini */
   
   wordint i;
   static wordint found = -1;
   static wordint ncalls = 0;
  
 fprintf(stderr,"%s, %d %d","gdin nGrilles=", gdin,nGrilles); 
   if (gdin >= nGrilles)
      {
      fprintf(stderr,"<ezgset>: source grid number non existent\n");
      exit(333);
      }
   
   if (gdout >= nGrilles)
      {
      fprintf(stderr,"<ezgset>: target grid number non existent\n");
      exit(333);
      }
   
/*   fprintf(stderr,"%d %d %d\n", gdin, gdout, nsets); */
   for (i=0; i < nsets; i++)
      {
/*      fprintf(stderr,"%d %d %d %d %d\n", gdin, gdout, nsets, gridset[i].gdin, gridset[i].gdout); */
      if (gdin == gridset[i].gdin && gdout == gridset[i].gdout)
         {
         found = i;
         iset = found;
         gridset[found].gdin = gdin;
         gridset[found].gdout = gdout;
         return 1;
         }
      }
   
   /* si on se rend jusqu'ici alors c'est que le set n'a pas ete trouve */
   
   
   if (i >= nsets)
      {
      found = i;
      nsets++;
      }

   
   if (nsets > NMAXSETS)
      {
      fprintf(stderr, "<ezdefset> : Too many defined gridsetes. \n");
      fprintf(stderr, "<ezdefset> : No way but to abort. \n");
      exit(13);
      
      /*
	nsets = NMAXSETS;
	found = ncalls % NMAXSETS;
	ncalls++;
	
	if (gridset[found].flags & XXX)
	{
	c_ezfreegridset(found);
	}*/
      }
   
   
   
   gridset[found].gdin = gdin;
   gridset[found].gdout = gdout;
   gridset[found].use_sincos_cache = OUI;
   iset = found;
   
   if (groptions.verbose > 0)
     {
     printf("iset                = %02d\n", iset);
     printf("gridset[%02d].gdin  = %d\n",   iset, gridset[iset].gdin);
     printf("gridset[%02d].gdout = %d\n",   iset, gridset[iset].gdout);
     }
   return 1;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezgdef)(wordint *ni, wordint *nj, char *grtyp, char *grref, 
			wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4, 
			ftnfloat *ax, ftnfloat *ay, wordint lengrtyp, wordint lengrref)
{
  wordint icode;
  char lgrtyp[2],lgrref[2];
  
  lgrtyp[0] = grtyp[0];
  lgrtyp[1] = '\0';
  
  lgrref[0] = grref[0];
  lgrref[1] = '\0';
  
  icode = c_ezgdef(*ni, *nj, lgrtyp, lgrref, *ig1, *ig2, *ig3, *ig4, ax, ay);
  return icode;
}

wordint c_ezgdef(wordint ni, wordint nj, char *grtyp, char *grref,
		 wordint ig1, wordint ig2, wordint ig3, wordint ig4, ftnfloat *ax, ftnfloat *ay)
{
  wordint found,source,iunit;
  char typeGrille;
  

  found = -1;
  typeGrille = grtyp[0];
  
  if (grtyp[0] == '#') 
    {
    fprintf(stderr, "The '#' grid type is not supported with ezgdef.\nPlease use ezgdef_ffile or ezgdef_fmem\n");
    exit(13);
    }

  switch(typeGrille)
    {
    case 'Y':
    case 'Z':
    case '#':
      if ((0 == strcmp(grref, "FILE")) || (0 == strcmp(grref, "file")))
	{
	source = FICHIER;
	}
      else
	{
	source = MEMOIRE;
	}       
      break;
      
    default:
      source = MEMOIRE;
      break;
    }
  
  switch (source)
    {
    case MEMOIRE:
      found = c_ezgdef_fmem(ni, nj, grtyp, grref, ig1, ig2, ig3, ig4, ax, ay);
      break;
      
    case FICHIER:
      found = c_ezgdef_ffile(ni, nj, grtyp, ig1, ig2, ig3, ig4, ig4);
      break;
    }
  
  return found;
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezgdef_ffile)(wordint *ni, wordint *nj, char *grtyp,
			      wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4, 
			      wordint *iunit, wordint lengrtyp)
{
  wordint icode;
  char lgrtyp[2],lgrref[2];
  
  lgrtyp[0] = grtyp[0];
  lgrtyp[1] = '\0';
  
  icode = c_ezgdef_ffile(*ni, *nj, lgrtyp, *ig1, *ig2, *ig3, *ig4, *iunit);
  return icode;
}

wordint c_ezgdef_ffile(wordint ni, wordint nj, char *grtyp,
		       wordint ig1, wordint ig2, wordint ig3, wordint ig4, wordint iunit)
{
  wordint i;
  wordint found;
  char typeGrille;
  char grref[2];
  ftnfloat *bidon = NULL;
  
  _Grille *gr;

  found = -1;
  typeGrille = (char)grtyp[0];
  
  if (typeGrille != '#' && typeGrille != 'Y' && typeGrille != 'Z')
    {
    strcpy(grref, " ");
    return c_ezgdef_fmem(ni, nj, grtyp, grref, ig1, ig2, ig3, ig4, bidon, bidon);
    }

  for (i=0; i < nGrilles; i++)
    {
    gr = (_Grille *) &(Grille[i]);
    if (typeGrille == gr->grtyp && \
	ni  == gr->ni   && \
	nj  == gr->nj   && \
	ig1 == gr->ig[IG1]  && \
	ig2 == gr->ig[IG2]  && \
	ig3 == gr->ig[IG3]  && \
	ig4 == gr->ig[IG4])
      {
      found = i;
      }
    }
  

  if (found == -1)
    {
    found = i;
    nGrilles++;
    }
  else
    {
    return found;
    }
  
  if (nGrilles > NMAXGRIDS)
    {
    fprintf(stderr, "<ezgdef_ffile> : Too many defined grids. \n");
    fprintf(stderr, "<ezgdef_ffile> : No way but to abort. \n");
    exit(13);
    }
  else
    {
    Grille[found].flags = 0;
    }
  
  Grille[found].grtyp = typeGrille;
  Grille[found].ni  = ni;
  Grille[found].nj  = nj;
  Grille[found].ig[IG1] = ig1;
  Grille[found].ig[IG2] = ig2;
  Grille[found].ig[IG3] = ig3;
  Grille[found].ig[IG4] = ig4;
  Grille[found].axe_y_inverse = 0;
  
  switch(grtyp[0])
    {
    case '#':
      LireEnrPositionnels(&(Grille[found]),iunit, ig1, ig2, ig3, ig4);
      break;
      
    default:
      LireEnrPositionnels(&(Grille[found]),iunit, ig1, ig2, ig3, 0);
      break;
    }
  
  ez_calcxpncof(found);
  Grille[found].i1 = 1;
  Grille[found].i2 = ni;
  Grille[found].j1 = 1;
  Grille[found].j2 = nj;
  ez_calcntncof(found);
  
  if (groptions.verbose > 0)
    {
    printf("Found = %02d\n", found);
    printf("Grille[%02d].grtyp = '%c'\n", found, Grille[found].grtyp);
    printf("Grille[%02d].ni    = %d\n",   found, Grille[found].ni);
    printf("Grille[%02d].nj    = %d\n",   found, Grille[found].nj);
    printf("Grille[%02d].ig1   = %d\n",   found, Grille[found].ig[IG1]);
    printf("Grille[%02d].ig2   = %d\n",   found, Grille[found].ig[IG2]);
    printf("Grille[%02d].ig3   = %d\n",   found, Grille[found].ig[IG3]);
    printf("Grille[%02d].ig4   = %d\n",   found, Grille[found].ig[IG4]);
    }
  
  return found;
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezgdef_fmem)(wordint *ni, wordint *nj, char *grtyp, char *grref, 
			      wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4, 
			      ftnfloat *ax, ftnfloat *ay, wordint lengrtyp, wordint lengrref)
{
  wordint icode;
  char lgrtyp[2],lgrref[2];
  
  lgrtyp[0] = grtyp[0];
  lgrtyp[1] = '\0';
  
  lgrref[0] = grref[0];
  lgrref[1] = '\0';
  
  icode = c_ezgdef_fmem(*ni, *nj, lgrtyp, lgrref, *ig1, *ig2, *ig3, *ig4, ax, ay);
  return icode;
}

wordint c_ezgdef_fmem(wordint ni, wordint nj, char *grtyp, char *grref,
		      wordint ig1, wordint ig2, wordint ig3, wordint ig4, ftnfloat *ax, ftnfloat *ay)
{
  wordint gdid;
  

  gdid = c_ezidentifygrid(ni, nj, grtyp, grref, ig1, ig2, ig3, ig4, ax, ay);
  c_ezdefxg(gdid);
  c_ezdefaxes(gdid, ax, ay);

 
  if (groptions.verbose > 0)
    {
    printf("Gdid = %02d\n", gdid);
    printf("Grille[%02d].grtyp = '%c'\n", gdid, Grille[gdid].grtyp);
    printf("Grille[%02d].ni    = %d\n",   gdid, Grille[gdid].ni);
    printf("Grille[%02d].nj    = %d\n",   gdid, Grille[gdid].nj);
    printf("Grille[%02d].ig[IG1]   = %d\n",   gdid, Grille[gdid].ig[IG1]);
    printf("Grille[%02d].ig[IG2]   = %d\n",   gdid, Grille[gdid].ig[IG2]);
    printf("Grille[%02d].ig[IG3]   = %d\n",   gdid, Grille[gdid].ig[IG3]);
    printf("Grille[%02d].ig[IG4]   = %d\n",   gdid, Grille[gdid].ig[IG4]);
    printf("Grille[%02d].grref = '%c'\n", gdid, Grille[gdid].grref);
    printf("Grille[%02d].igref[IG1]= %d\n",   gdid, Grille[gdid].igref[IG1]);
    printf("Grille[%02d].igref[IG2]= %d\n",   gdid, Grille[gdid].igref[IG2]);
    printf("Grille[%02d].igref[IG3]= %d\n",   gdid, Grille[gdid].igref[IG3]);
    printf("Grille[%02d].igref[IG4]= %d\n",   gdid, Grille[gdid].igref[IG4]);
    }
  
  return gdid;
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

wordint f77name(ezgetopt)(char *option, char *value, wordint lenoption, wordint lenvalue)
{
   wordint icode;
   char local_opt[32], local_val[32];

   ftnstrclean(option,lenoption);
   ftnstrclean(value,lenvalue);
   strncpy(local_opt, option, lenoption);
   local_opt[lenoption] = '\0';
   
   icode = c_ezgetopt(local_opt, value);

   return icode;
}

wordint c_ezgetopt(char *option, char *value)
{
  char local_opt[32], local_val[32];
  wordint i;
  
  strcpy(local_opt, option);
  
  for (i=0; i < strlen(local_opt); i++)
    local_opt[i] = (char) tolower((int)local_opt[i]);
  
  if (0 == strcmp(local_opt, "verbose"))
    {
    switch (groptions.verbose)
      {
      case 1:
	strcpy(value, "yes");
	break;
	
      case 2:
	strcpy(value, "yesyesyes");
	break;
	
      default:
	strcpy(value, "no");
	break;
      }
    }
  
  if (0 == strcmp(local_opt, "polar_correction"))
    {
    switch (groptions.polar_correction)
      {
      case 1:
	strcpy(value, "yes");
	break;
	
      default:
	strcpy(value, "no");
	break;
      }
    }
  
  if (0 == strcmp(local_opt, "interp_degree"))
    {
    switch (groptions.degre_interp)
      {
      case 0:
	strcpy(value, "nearest");
	break;
	
      case 1:
	strcpy(value, "linear");
	break;
	
      case 3:
	strcpy(value, "cubic");
	  break;
	  
      default:
	strcpy(value, "error");
	break;
      }
    }
  
  if (0 == strcmp(local_opt, "degre_interp"))
    {
    switch (groptions.degre_interp)
      {
      case 0:
	strcpy(value, "voisin");
	break;
	
      case 1:
	strcpy(value, "lineaire");
	break;
	
      case 3:
	strcpy(value, "cubique");
	  break;
	  
      default:
	strcpy(value, "erreur");
	break;
      }
    }
  
  if (0 == strcmp(local_opt, "extrap_degree"))
    {
    switch (groptions.degre_extrap)
      {
      case 0:
	strcpy(value, "nearest");
	break;
	
      case 1:
	strcpy(value, "linear");
	break;
	
      case 3:
	strcpy(value, "cubic");
	break;
	
      case MAXIMUM:
	strcpy(value, "maximum");
	break;
	
      case MINIMUM:
	strcpy(value, "minimum");
	break;
	
      case VALEUR:
	strcpy(value, "value");
	break;
	
      case ABORT:
	strcpy(value, "abort");
	break;
	
      default:
	strcpy(value, "error");
	break;
      }
    }
   
  if (0 == strcmp(local_opt, "degre_extrap"))
    {
    switch (groptions.degre_extrap)
      {
      case 0:
	strcpy(value, "voisin");
	break;
	
      case 1:
	strcpy(value, "lineaire");
	break;
	
      case 3:
	strcpy(value, "cubique");
	break;
	
      case MAXIMUM:
	strcpy(value, "maximum");
	break;
	
      case MINIMUM:
	strcpy(value, "minimum");
	break;
	
      case VALEUR:
	strcpy(value, "valeur");
	break;
	
      case ABORT:
	strcpy(value, "abort");
	break;
	
      default:
	strcpy(value, "erreur");
	break;
      }
    }
   
  return 0;
}
      
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezgetval)(char *option, ftnfloat *fvalue, wordint lenoption)
{
   wordint icode;
   
   ftnstrclean(option,lenoption);
   icode = c_ezgetval(option, fvalue);
   
   return icode;
}

wordint c_ezgetval(char *option, ftnfloat *fvalue)
{
   char local_opt[32];
   wordint i;
   
   strcpy(local_opt, option);
   
   for (i=0; i < strlen(local_opt); i++)
     local_opt[i] = (char) tolower((int)local_opt[i]);
   
   if (0 == strcmp(local_opt, "extrap_value"))
      {
      *fvalue = groptions.valeur_extrap;
      }
   
   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezgfstp)(wordint *gdid, 
                         char *nomvarx, char *typvarx, char *etiketx,
                         char *nomvary, char *typvary, char *etikety,
                         wordint *ip1, wordint *ip2, wordint *ip3, wordint *dateo, 
                         wordint *deet, wordint *npas, wordint *nbits,
                         wordint lennomvarx, wordint lentypvarx, wordint lenetiketx,
                         wordint lennomvary, wordint lentypvary, wordint lenetikety)
{
   wordint i;
   wordint icode;
   
   char lnomvarx[5],lnomvary[5],letiketx[13],letikety[13],ltypvarx[2],ltypvary[2];

   c_ezgfstp(*gdid, lnomvarx, ltypvarx, letiketx, lnomvary, ltypvary,letikety,
             ip1, ip2, ip3, dateo, deet, npas, nbits);

   for (i=0; i < lennomvarx; i++)
      {
      nomvarx[i] = ' ';
      nomvary[i] = ' ';
      }
   
   for (i=0; i < lentypvarx; i++)
      {
      typvarx[i] = ' ';
      typvary[i] = ' ';
      }
   
   for (i=0; i < lenetiketx; i++)
      {
      etiketx[i] = ' ';
      etikety[i] = ' ';
      }
   
#ifdef FICH_STD98
   strncpy(nomvarx, lnomvarx, 4);
   strncpy(nomvary, lnomvary, 4);
   strncpy(typvarx, ltypvarx, 2);
   strncpy(typvary, ltypvary, 2);
   strncpy(etiketx, letiketx, 12);
   strncpy(etikety, letikety, 12);
#else
   strncpy(nomvarx, lnomvarx, 2);
   strncpy(nomvary, lnomvary, 2);
   strncpy(typvarx, ltypvarx, 1);
   strncpy(typvary, ltypvary, 1);
   strncpy(etiketx, letiketx, 8);
   strncpy(etikety, letikety, 8);
#endif
   icode = 0;
   return icode;
}

wordint c_ezgfstp(wordint gdid, char *nomvarx, char *typvarx, char *etiketx, 
              char *nomvary, char *typvary, char *etikety,
              wordint *ip1, wordint *ip2, wordint *ip3, wordint *dateo, wordint *deet, wordint *npas, wordint *nbits)
{
   strcpy(nomvarx, Grille[gdid].nomvarx);
   strcpy(typvarx, Grille[gdid].typvarx);
   strcpy(etiketx, Grille[gdid].etiketx);

   strcpy(nomvary, Grille[gdid].nomvary);
   strcpy(typvary, Grille[gdid].typvary);
   strcpy(etikety, Grille[gdid].etikety);

   *ip1 = Grille[gdid].ip1;
   *ip2 = Grille[gdid].ip2;
   *ip3 = Grille[gdid].ip3;

   *dateo = Grille[gdid].date;
   *deet = Grille[gdid].deet;
   *npas = Grille[gdid].npas;
   *nbits = Grille[gdid].nbits;

   return 0;
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezgprm)(wordint *gdid, char *grtyp, wordint *ni, wordint *nj,
             wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4)
{
  return c_ezgprm(*gdid, grtyp, ni, nj, ig1, ig2, ig3, ig4);
}

wordint   c_ezgprm(wordint gdid, char *grtyp, wordint *ni, wordint *nj, wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4)
{
   *grtyp = Grille[gdid].grtyp;
   *ni    = Grille[gdid].ni;
   *nj    = Grille[gdid].nj;
   *ig1   = Grille[gdid].ig[IG1];
   *ig2   = Grille[gdid].ig[IG2];
   *ig3   = Grille[gdid].ig[IG3];
   *ig4   = Grille[gdid].ig[IG4];

   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezgxprm)(wordint *gdid, wordint *ni, wordint *nj, char *grtyp,
                     wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4, 
                     char *grref, wordint *ig1ref, wordint *ig2ref, 
                     wordint *ig3ref, wordint *ig4ref,
                     wordint lengrtyp, wordint lengrref)
{
   wordint icode;
   
   ftnstrclean(grtyp,lengrtyp);
   ftnstrclean(grref,lengrtyp);
   icode = c_ezgxprm(*gdid, ni, nj, grtyp, ig1, ig2, ig3, ig4, 
                     grref, ig1ref, ig2ref, ig3ref, ig4ref);
   return icode;
}

wordint c_ezgxprm(wordint gdid, wordint *ni, wordint *nj, 
              char *grtyp, wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4,
              char *grref, wordint *ig1ref, wordint *ig2ref, wordint *ig3ref, wordint *ig4ref)
{
   *ni     = Grille[gdid].ni;
   *nj     = Grille[gdid].nj;
   *grtyp  = Grille[gdid].grtyp;
   *grref  = Grille[gdid].grref;
   *ig1    = Grille[gdid].ig[IG1];
   *ig2    = Grille[gdid].ig[IG2];
   *ig3    = Grille[gdid].ig[IG3];
   *ig4    = Grille[gdid].ig[IG4];
   *ig1ref = Grille[gdid].igref[IG1];
   *ig2ref = Grille[gdid].igref[IG2];
   *ig3ref = Grille[gdid].igref[IG3];
   *ig4ref = Grille[gdid].igref[IG4];

   return 0;
}


/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezqkdef)(wordint *ni, wordint *nj, char *grtyp, 
                    wordint *ig1, wordint *ig2, wordint *ig3, wordint *ig4, wordint *iunit, wordint lengrtyp)
{
   wordint icode;
   char cgrtyp[2];

   cgrtyp[0] = grtyp[0];
   cgrtyp[1] = '\0';
   icode = c_ezqkdef(*ni, *nj, cgrtyp, *ig1, *ig2, *ig3, *ig4, *iunit);
   return icode;
}

wordint c_ezqkdef(wordint ni, wordint nj, char *grtyp,
             wordint ig1, wordint ig2, wordint ig3, wordint ig4, wordint iunit)
{
  wordint icode;

  icode = c_ezgdef_ffile(ni, nj, grtyp, ig1, ig2, ig3, ig4, iunit);
  
  return icode;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezsetopt)(char *option, char *value, wordint lenoption, wordint lenvalue)
{
   wordint i, icode;
   wordint longueur_option, longueur_value;

   char local_opt[32], local_val[32];
   longueur_option = f77name(longueur)(option, lenoption);
   longueur_value = f77name(longueur)(value, lenvalue);
   
   longueur_option = longueur_option < 32 ? longueur_option : 31;
   longueur_value  = longueur_value  < 32 ? longueur_value  : 31;

   for (i=0; i < longueur_option; i++)
     {
     local_opt[i] = option[i];
     }
   for (i=0; i < longueur_value; i++)
     {
     local_val[i] = value[i];
     }

   local_opt[longueur_option] = '\0';
   local_val[longueur_value] = '\0';
   icode = c_ezsetopt(local_opt, local_val);

   return icode;
}

wordint c_ezsetopt(char *option, char *value)
{
   char local_opt[32], local_val[32];
   wordint i, option_ok, value_ok;


   option_ok = 0;
   value_ok = 0;
   
   strcpy(local_opt, option);
   strcpy(local_val, value);
   
   for (i=0; i < strlen(local_opt); i++)
     local_opt[i] = (char) tolower((int)local_opt[i]);
   
   for (i=0; i < strlen(local_val); i++)
     local_val[i] = (char) tolower((int)local_val[i]);
   
   if (0 == strcmp(local_opt, "correction_polaire")) strcpy(local_opt, "polar_correction");
   if (0 == strcmp(local_opt, "degre_interp")) strcpy(local_opt, "interp_degree");
   if (0 == strcmp(local_opt, "degre_extrap")) strcpy(local_opt, "extrap_degree");

   if (0 == strcmp(local_val, "oui")) strcpy(local_val, "yes");
   if (0 == strcmp(local_val, "ouiouioui")) strcpy(local_val, "yesyesyes");
   if (0 == strcmp(local_val, "non")) strcpy(local_val, "no");
   if (0 == strcmp(local_val, "voisin")) strcpy(local_val, "nearest");
   if (0 == strcmp(local_val, "lineair")) strcpy(local_val, "linear");
   if (0 == strcmp(local_val, "lineaire")) strcpy(local_val, "linear");
   if (0 == strcmp(local_val, "cubique")) strcpy(local_val, "cubic");
   if (0 == strcmp(local_val, "neutre")) strcpy(local_val, "neutral");
   if (0 == strcmp(local_val, "valeur")) strcpy(local_val, "value");

   if (0 == strcmp(local_opt, "verbose"))
      {
      option_ok = 1;
      value_ok = 1;
      if (0 == strcmp(local_val, "yes"))
         {
         groptions.verbose = 1;
         }
      else if (0 == strcmp(local_val, "yesyesyes"))
         {
         groptions.verbose = 2;
         }
      else if (0 == strcmp(local_val, "no"))
	{
	groptions.verbose = 0;
	}
      else
	{
	value_ok = 0;
	}
	       
      }

   if (0 == strcmp(local_opt, "polar_correction"))
      {
      option_ok = 1;
      value_ok = 1;
      if (0 == strcmp(local_val, "yes"))
         {
         groptions.degre_interp = 1;
         }
      else if (0 == strcmp(local_val, "no"))
         {
         groptions.degre_interp = 0;
         }
      else
	{
	value_ok = 0;
	}
      }



   if (0 == strcmp(local_opt, "interp_degree"))
     {
     option_ok = 1;
     value_ok = 1;
     if (0 == strcmp(local_val, "nearest"))
       {
       groptions.degre_interp = 0;
       }
     
     else if (0 == strcmp(local_val, "linear"))
       {
       groptions.degre_interp = 1;
       }
     
     else if (0 == strcmp(local_val, "cubic"))
       {
       groptions.degre_interp = 3;
       }
     else
       {
       value_ok = 0;
       }
     
     }
   
   if (0 == strcmp(local_opt, "extrap_degree"))
     {
     option_ok = 1;
     value_ok = 1;
     if (0 == strcmp(local_val, "neutral"))
       {
       groptions.degre_extrap = groptions.degre_interp;
       }     
     else if (0 == strcmp(local_val, "nearest"))
       {
       groptions.degre_extrap = 0;
       }     
     else if (0 == strcmp(local_val, "linear"))
       {
         groptions.degre_extrap = 1;
         }      
      else if (0 == strcmp(local_val, "cubic"))
         {
         groptions.degre_extrap = 3;
         }
      else if (0 == strcmp(local_val, "maximum"))
         {
         groptions.degre_extrap = MAXIMUM;
         if (groptions.verbose == 1)
            {
            fprintf(stderr, "Extrapolation set to maximum value\n");
            }
         }
      else if (0 == strcmp(local_val, "minimum"))
         {
         groptions.degre_extrap = MINIMUM;
         if (groptions.verbose == 1)
            {
            fprintf(stderr, "Extrapolation set to minimum value\n");
            }
         }
      else if (0 == strcmp(local_val, "value"))
         {
         groptions.degre_extrap = VALEUR;
         if (groptions.verbose == 1)
            {
            fprintf(stderr, "Extrapolation set to value: %f\n", groptions.valeur_extrap);
            }
         }
      else if (0 == strcmp(local_val, "abort"))
         {
         groptions.degre_extrap = ABORT;
         if (groptions.verbose == 1)
            {
            fprintf(stderr, "Extrapolation set to ABORT\n");
            }
         }
      else
	{
	value_ok = 0;
	}

      }
   
   if ((value_ok + option_ok) != 2)
     {
     if (option_ok == 0)
       {
       fprintf(stderr, "ezsetopt : option not recognized : %s\n", option);
       }
     if (value_ok == 0)
       {
       fprintf(stderr, "ezsetopt : value not recognized : %s\n", value);
       }
     return -1;
     }
   else
     {
     return 0;
     }


}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezsetval)(char *option, ftnfloat *fvalue, wordint lenoption)
{
   wordint icode;
   
   option[lenoption] = '\0';
   ftnstrclean(option,lenoption);
   icode = c_ezsetval(option, *fvalue);
   
   return icode;
}

wordint c_ezsetval(char *option, ftnfloat fvalue)
{
   char local_opt[32];
   wordint i;
   
   strcpy(local_opt, option);
   
   for (i=0; i < strlen(local_opt); i++)
     local_opt[i] = (char) tolower((int)local_opt[i]);
   
   if (0 == strcmp(local_opt, "extrap_value"))
      {
      groptions.valeur_extrap = fvalue;
      }
   
   return 0;
}

wordint c_ezsetval2(char *option, ftnfloat *fvalue)
{
   char local_opt[32];
   wordint i;
   
   strcpy(local_opt, option);
   
   for (i=0; i < strlen(local_opt); i++)
     local_opt[i] = (char) tolower((int)local_opt[i]);
   
   if (0 == strcmp(local_opt, "extrap_value"))
      {
      groptions.valeur_extrap = *fvalue;
      }
   
   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezsint)(ftnfloat *zout, ftnfloat *zin)
{
   wordint icode;
   
   icode = c_ezsint(zout, zin);
   return icode;
}

wordint c_ezsint(ftnfloat *zout, ftnfloat *zin)
{
   wordint gdin, gdout;
   wordint ier;
   wordint npts;
   ftnfloat *lzin, *lxzin;

   lzin  = NULL;
   lxzin = NULL;

   if (iset == UNDEFINED)
      {
      fprintf(stderr,"<c_ezsint> Source and target grid undefined! Aborting...\n");
      exit(333);
      }

   
   gdin = gridset[iset].gdin;
   gdout = gridset[iset].gdout;

   if (gridset[iset].gdin == gridset[iset].gdout)
      {
      memcpy(zout, zin, Grille[gdin].ni*Grille[gdin].nj*sizeof(ftnfloat));
      return 1;
      }

   if (Grille[gdin].axe_y_inverse == 1)
      {
      lzin = (ftnfloat *) malloc(Grille[gdin].ni*Grille[gdin].nj*sizeof(ftnfloat));
      memcpy(lzin, zin, Grille[gdin].ni*Grille[gdin].nj*sizeof(ftnfloat));
      f77name(permut)(lzin, &Grille[gdin].ni, &Grille[gdin].nj);
      }
   else
     {
     lzin = zin;
     }

   if (Grille[gdin].needs_expansion == OUI)
     {
     lxzin = (ftnfloat *) malloc(2*Grille[gdin].ni*Grille[gdin].nj*sizeof(ftnfloat));
     ez_xpnsrcgd(gdin, lxzin, lzin);
     }
   else
     {
     lxzin = lzin;
     }

   ier = ez_calclatlon(gridset[iset].gdout);
   ier = ez_calcxy(iset);
   npts = Grille[gdout].ni*Grille[gdout].nj;

   ier = ez_interp(zout, lxzin, iset);

   if (groptions.polar_correction == OUI)
     {
     ier = ez_defzones(&gridset[iset]);
     ier = ez_corrval(zout, lxzin, &gridset[iset]);
     }
   
   if (lzin != zin && lzin != NULL)
      {
      free(lzin);
      }

   if (lxzin != lzin && lxzin != zin && lxzin != NULL)
      {
      free(lxzin);
      }

   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezuvint)(ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin)
{
   wordint icode;
   
   icode = c_ezuvint(uuout, vvout, uuin, vvin);
   return icode;
}

wordint c_ezuvint(ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin)
{
   wordint gdidin,gdidout;
   wordint npts;
   ftnfloat *uullout = NULL;
   ftnfloat *vvllout = NULL;
   
   gdidin = gridset[iset].gdin;
   gdidout= gridset[iset].gdout;
   npts = Grille[gdidout].ni*Grille[gdidout].nj;

   groptions.vecteur = VECTEUR;

   groptions.symmetrie = SYM;
   c_ezsint(uuout,uuin);
   groptions.symmetrie = ANTISYM;
   c_ezsint(vvout,vvin);
   groptions.symmetrie = SYM;

   if (groptions.polar_correction == OUI)
     {
     ez_corrvec(uuout, vvout, uuin, vvin, &gridset[iset]);
     }

   uullout = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
   vvllout = (ftnfloat *) malloc(npts*sizeof(ftnfloat));

   c_ezgenerate_gem_cache();
   
   c_gdwdfuv(gdidin, uullout, vvllout, uuout, vvout,
             Grille[gdidout].lat, Grille[gdidout].lon, npts);
   c_gduvfwd(gdidout, uuout, vvout, uullout, vvllout,
             Grille[gdidout].lat, Grille[gdidout].lon, npts);

   groptions.vecteur = SCALAIRE;
   free(uullout);
   free(vvllout);

   return 0;
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezwdint)(ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin)
{
   wordint icode;
   
   icode = c_ezwdint(uuout, vvout, uuin, vvin);
   return icode;
}

wordint c_ezwdint(ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin)
{
   wordint gdidin,gdidout;
   ftnfloat *uullout = NULL;
   ftnfloat *vvllout = NULL;
   wordint npts;
   
   gdidin = gridset[iset].gdin;
   gdidout= gridset[iset].gdout;
   npts = Grille[gdidout].ni*Grille[gdidout].nj;

   groptions.vecteur = VECTEUR;

   groptions.symmetrie = SYM;
   c_ezsint(uuout,uuin);

   groptions.symmetrie = ANTISYM;
   c_ezsint(vvout,vvin);

   groptions.symmetrie = SYM;

   if (groptions.polar_correction == OUI)
     {
     ez_corrvec(uuout, vvout, uuin, vvin, &gridset[iset]);
     }

   uullout = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
   vvllout = (ftnfloat *) malloc(npts*sizeof(ftnfloat));
   
   c_ezgenerate_gem_cache();

   c_gdwdfuv(gdidin, uullout, vvllout, uuout, vvout,
             Grille[gdidout].lat, Grille[gdidout].lon, npts);

   memcpy(uuout, uullout, npts*sizeof(ftnfloat));
   memcpy(vvout, vvllout, npts*sizeof(ftnfloat));

   groptions.vecteur = SCALAIRE;
   free(uullout);
   free(vvllout);
   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdgaxes)(wordint *gdid, ftnfloat *ax, ftnfloat *ay)
{
   c_gdgaxes(*gdid, ax, ay);
   return 0;
}

wordint c_gdgaxes(wordint gdid, ftnfloat *ax, ftnfloat *ay)
{

   wordint nix, njy;

   switch(Grille[gdid].grtyp)
      {
      case 'Y':
        nix = Grille[gdid].ni * Grille[gdid].nj;
        njy = nix;
        break;

      default:
        nix = Grille[gdid].ni;
        njy = Grille[gdid].nj;
        break;
      }
   
   if (Grille[gdid].flags & AX)
      {
      memcpy(ax, Grille[gdid].ax, nix*sizeof(ftnfloat));
      memcpy(ay, Grille[gdid].ay, njy*sizeof(ftnfloat));
      }
   else
      {
      fprintf(stderr, "(gdgaxes) Erreur! A l'aide! Descripteurs manquants!\n");
      exit(999);
      }
   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

wordint c_gdinterp(ftnfloat *zout, ftnfloat *zin, wordint gdin, ftnfloat *x, ftnfloat *y, wordint npts)
{
  wordint lnpts;

  lnpts = npts;

  switch(Grille[gdin].grtyp)
    {
    case '#':
    case 'Z':
    case 'G':
      switch (groptions.degre_interp)
	{
	case VOISIN:
	  f77name(ez_rgdint_0)(zout,x,y,
			       &lnpts, zin, &Grille[gdin].ni, 
			       &Grille[gdin].j1, &Grille[gdin].j2);
	  break;
	  
	case LINEAIRE:
	  switch(Grille[gdin].extension)
	    {
	    case 0:
	      f77name(ez_irgdint_1_nw)(zout,x, y,
				       &lnpts, Grille[gdin].ax, Grille[gdin].ay,
				       zin,&Grille[gdin].ni, &Grille[gdin].nj);
	      break;
	      
	    case 1:
	    case 2:
	      f77name(ez_irgdint_1_w)(zout,x, y,
				      &lnpts, Grille[gdin].ax, Grille[gdin].ay,
				      zin,&Grille[gdin].ni, &Grille[gdin].j1, &Grille[gdin].j2, &Grille[gdin].extension);
	      break;
	    }
	  break;
	  
	case CUBIQUE:
	  switch(Grille[gdin].extension)
	    {
	    case 0:
	      f77name(ez_irgdint_3_nw)(zout, x, y,
				       &lnpts, Grille[gdin].ax, Grille[gdin].ay,
				       Grille[gdin].ncx, Grille[gdin].ncy, zin,
				       &Grille[gdin].i1, &Grille[gdin].i2,
				       &Grille[gdin].j1, &Grille[gdin].j2);
	      break;
	      
	    case 1:
	    case 2:
	      f77name(ez_irgdint_3_w)(zout, x, y,
				      &lnpts, Grille[gdin].ax, Grille[gdin].ay,
				      Grille[gdin].ncx, Grille[gdin].ncy, zin,
				      &Grille[gdin].ni, &Grille[gdin].j1, &Grille[gdin].j2, 
				      &Grille[gdin].extension);
	      break;
	    }
	  break;
	}
      
      break;
      
    default:
      switch (groptions.degre_interp)
	{
	case VOISIN:
	  f77name(ez_rgdint_0)(zout,x,y,
			       &lnpts, zin, &Grille[gdin].ni, 
			       &Grille[gdin].j1, &Grille[gdin].j2);
	  
	  break;
	  
	case LINEAIRE:
	  switch(Grille[gdin].extension)
	    {
	    case 0:
	    case 1:
	      f77name(ez_rgdint_1_nw)(zout,x,y,
				      &lnpts, zin, &Grille[gdin].ni, 
				      &Grille[gdin].j1, &Grille[gdin].j2);
	      break;
	      
	    case 2:
	      f77name(ez_rgdint_1_w)(zout,x,y,
				     &lnpts, zin, &Grille[gdin].ni, 
				     &Grille[gdin].j1, &Grille[gdin].j2,
				     &Grille[gdin].extension);
	    }
	  break;
	  
	case CUBIQUE:
	  switch(Grille[gdin].extension)
	    {
	    case 0:
	      f77name(ez_rgdint_3_nw)(zout, x, y,
				      &lnpts, zin, &Grille[gdin].ni, 
				      &Grille[gdin].j1, &Grille[gdin].j2); 
	      break;
	      
	    case 1:
	    case 2:
	      f77name(ez_rgdint_3_w)(zout, x, y,
				     &lnpts, zin,  &Grille[gdin].ni, 
				     &Grille[gdin].j1, &Grille[gdin].j2,
				     &Grille[gdin].extension);
	      break;
	    }
	  break;
	}
      break;
    }
  
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(ezll)(wordint *gdid, ftnfloat *lat, ftnfloat *lon)
{
   wordint icode;
   
   icode = c_gdll(*gdid, lat, lon);
   return icode;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdll)(wordint *gdid, ftnfloat *lat, ftnfloat *lon)
{
   wordint icode;
   
   icode = c_gdll(*gdid, lat, lon);
   return icode;
}

wordint c_gdll(wordint gdid, ftnfloat *lat, ftnfloat *lon)
{
   ez_calclatlon(gdid);
   if (Grille[gdid].flags & LAT)
      {
      memcpy(lon, Grille[gdid].lon, Grille[gdid].ni*Grille[gdid].nj*sizeof(ftnfloat));
      if (Grille[gdid].axe_y_inverse == 0)
         {
         memcpy(lat, Grille[gdid].lat, Grille[gdid].ni*Grille[gdid].nj*sizeof(ftnfloat));
         }
      else
         {
         f77name(permut)(Grille[gdid].lat, &Grille[gdid].ni, &Grille[gdid].nj);
         memcpy(lat, Grille[gdid].lat, Grille[gdid].ni*Grille[gdid].nj*sizeof(ftnfloat));
         f77name(permut)(Grille[gdid].lat, &Grille[gdid].ni, &Grille[gdid].nj);
         }
      }
   else
      {
      fprintf(stderr, "Erreur! A l'aide! Descripteurs manquants!\n");
      exit(999);
      }
   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdllfxy)(wordint *gdid, ftnfloat *lat, ftnfloat *lon, ftnfloat *x, ftnfloat *y, wordint *n)
{
  return c_gdllfxy(*gdid, lat, lon, x, y, *n);
 
}

wordint c_gdllfxy(wordint gdid, ftnfloat *lat, ftnfloat *lon, ftnfloat *x, ftnfloat *y, wordint n)
{
  ftnfloat xlat1, xlon1, xlat2, xlon2;
  wordint i,j, npts, un;
  ftnfloat *tmpx, *tmpy;
  ftnfloat delxx, delyy;
  ftnfloat dlat, dlon, swlat, swlon;
  wordint indx, indy;
  
  _Grille gr;
  
  gr = Grille[gdid];
  npts = n;
  un = 1;
  
  switch(gr.grtyp)
    {
    case 'A': 
      for (i=0; i < n; i++)
	{
	lat[i] = (y[i]-1.0)*gr.xg[DLAT]+gr.xg[SWLAT];
	lon[i] = (x[i]-1.0)*gr.xg[DLON]+gr.xg[SWLON];
	lon[i] = (ftnfloat) (fmod((double) (lon[i] + 360.0), (double) 360.0));
	}
      break;
      
    case 'B':
      for (i=0; i < n; i++)
	{
	lat[i] = (y[i]-1.0)*gr.xg[DLAT]+gr.xg[SWLAT];
	lon[i] = (x[i]-1.0)*gr.xg[DLON]+gr.xg[SWLON];
	lon[i] = (ftnfloat) (fmod((double) (lon[i] + 360.0), (double) 360.0));
	}
      break;
      
    case 'E':
      tmpx = (ftnfloat *) malloc(n*sizeof(ftnfloat));
      tmpy = (ftnfloat *) malloc(n*sizeof(ftnfloat));
      for (i=0; i < n; i++)
	{
	dlat  = 180.0 / gr.nj;
	dlon  = 360.0 / (gr.ni - 1);
	swlat = -90.0 + 0.5 * dlat;
	swlon = 0.0;
	tmpx[i] = (x[i]-1.0)*dlon+swlon;
	tmpy[i] = (y[i]-1.0)*dlat+swlat;
	}
      
      f77name(ez_gfllfxy)(lon,lat,tmpx,tmpy,&n,&gr.xg[XLAT1],&gr.xg[XLON1],&gr.xg[XLAT2],&gr.xg[XLON2]);
      free(tmpx);
      free(tmpy);
      break;
      
      
    case 'L':
      for (i=0; i < n; i++)
	{
	lon[i] = (x[i]-1.0)*gr.xg[DLON]+gr.xg[SWLON];
	lon[i] = (ftnfloat) (fmod((double) (lon[i] + 360.0), (double) 360.0));
	lat[i] = (y[i]-1.0)*gr.xg[DLAT]+gr.xg[SWLAT];
	}
      break;
      
    case 'N':
    case 'S':
      f77name(ez_vllfxy)(lat,lon,x,y,&npts,&un,&gr.xg[D60],&gr.xg[DGRW],&gr.xg[PI],&gr.xg[PJ],&gr.hemisphere);
      for (i=0; i < n; i++)
	{
	lon[i] = (ftnfloat) (fmod((double) (lon[i] + 360.0), (double) 360.0));
	}
      break;
      
    case 'T':
      f77name(ez_vtllfxy)(lat,lon,x,y, &gr.xg[CLAT], &gr.xg[CLON], &gr.xg[TD60], &gr.xg[TDGRW], &gr.ni, &gr.nj, &npts);
      break;
      
    case '!':
      f77name(ez_llflamb)(lat,lon,x,y,&npts,&gr.grtyp,&gr.ig[IG1], &gr.ig[IG2], &gr.ig[IG3], &gr.ig[IG4]);
      break;


    case 'Y':
      fprintf(stderr, "********************************************************\n");
      fprintf(stderr, "<gdllfxy>: This operation is not supported for 'Y' grids\n");
      fprintf(stderr, "********************************************************\n");
      break;
      
    case '#':
    case 'Z':
    case 'G':
      tmpx = (ftnfloat *) malloc(n*sizeof(ftnfloat));
      tmpy = (ftnfloat *) malloc(n*sizeof(ftnfloat));
      for (i=0; i < n; i++)
	{
	indx = (int)x[i]-1;
	indy = (int)y[i]-1;
	
	indx = indx < 0 ? 0 : indx;
	indy = indy < 0 ? 0 : indy;
	indx = indx > gr.ni-2 ? gr.ni-2 : indx;
	indy = indy > gr.j2-2 ? gr.j2-2 : indy;
	delxx = gr.ax[indx+1]-gr.ax[indx];
	tmpx[i] = gr.ax[indx] + ((x[i]-1.0-indx)*delxx);
	
	delyy = gr.ay[indy+1]-gr.ay[indy];
	tmpy[i] = gr.ay[indy] + ((y[i]-1.0-indy)*delyy);
	}
      
      switch (gr.grref)
	{
	case 'E':
	  f77name(cigaxg)(&gr.grref,&xlat1,&xlon1,&xlat2,&xlon2,
			  &gr.igref[IG1],&gr.igref[IG2],&gr.igref[IG3],&gr.igref[IG4]);
	  f77name(ez_gfllfxy)(lon, lat, tmpx, tmpy, &npts, &gr.xgref[XLAT1], &gr.xgref[XLON1],
			      &gr.xgref[XLAT2], &gr.xgref[XLON2]);
	  break;
	  
	case 'S':
	case 'N':
	  f77name(ez_vllfxy)(lat,lon,tmpx,tmpy,&npts,&un,&gr.xgref[D60],
			     &gr.xgref[DGRW],&gr.xgref[PI],&gr.xgref[PJ],&gr.hemisphere);
	  for (i=0; i < n; i++)
	    {
	    lon[i] = (ftnfloat) (fmod((double) (lon[i] + 360.0), (double) 360.0));
	    }
	  break;
	  
	case 'L':
	  for (i=0; i < n; i++)
	    {
	    lat[i] = (tmpy[i])*gr.xgref[DLAT]+gr.xgref[SWLAT];
	    lon[i] = (tmpx[i])*gr.xgref[DLON]+gr.xgref[SWLON];
	    lon[i] = (ftnfloat) (fmod((double) (lon[i] + 360.0), (double) 360.0));
	    }
	  break;
	  
	default:
	  fprintf(stderr,"<gdllfxy> Errrrrrrrrrrreur!\n");
	  break;
	}
      free(tmpx);
      free(tmpy);
      break;
    }
  
  return 0;
  
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdllsval)(wordint *gdid, ftnfloat *zout, ftnfloat *zin, ftnfloat *lat, ftnfloat *lon, wordint *n)
{
   wordint icode;
   
   icode = c_gdllsval(*gdid, zout, zin, lat, lon, *n);
   return icode;
}

wordint c_gdllsval(wordint gdid, ftnfloat *zout, ftnfloat *zin, ftnfloat *lat, ftnfloat *lon, wordint n)
{
   ftnfloat *x, *y;
   wordint ier;
   
   x = (ftnfloat *)malloc(n * sizeof(float));
   y = (ftnfloat *)malloc(n * sizeof(float));
   
   ier = c_gdxyfll(gdid, x, y, lat, lon, n);
   ier = c_gdxysval(gdid, zout, zin, x, y, n);
   
   free(x);
   free(y);
   return 0;
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdllvval)(wordint *gdid, ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin, 
                      ftnfloat *lat, ftnfloat *lon, wordint *n)
{
   wordint icode;
   
   icode = c_gdllvval(*gdid, uuout,vvout, uuin, vvin, lat, lon, *n);
   return icode;
}

wordint c_gdllvval(wordint gdid, ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin, 
               ftnfloat *lat, ftnfloat *lon, wordint n)
{
   ftnfloat *x, *y;
   wordint ier;
   
   x = (ftnfloat *) malloc(n * sizeof(float));
   y = (ftnfloat *) malloc(n * sizeof(float));
   
   ier = c_gdxyfll(gdid, x, y, lat, lon, n);
   ier = c_gdxyvval(gdid, uuout, vvout, uuin, vvin, x, y, n);
   
   free(x);
   free(y);
   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdllwdval)(wordint *gdid, ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin, 
                      ftnfloat *lat, ftnfloat *lon, wordint *n)
{
   wordint icode;
   
   icode = c_gdllwdval(*gdid, uuout,vvout, uuin, vvin, lat, lon, *n);
   return icode;
}

wordint c_gdllwdval(wordint gdid, ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin, 
               ftnfloat *lat, ftnfloat *lon, wordint n)
{

   wordint ier;

   ier = c_gdllvval(gdid, uuout, vvout, uuin, vvin, lat, lon, n);
   ier = c_gdwdfuv(gdid, uuout, vvout, uuout, vvout, lat, lon, n);

   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdrls)(wordint *gdin)
{
   wordint icode;
   
   icode = c_gdrls(*gdin);
   return icode;
}

wordint c_gdrls(wordint gdin)
{
   EliminerGrille(gdin);

   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gduvfwd)(wordint *gdid, ftnfloat *uugdout, ftnfloat *vvgdout, 
                         ftnfloat *uullin, ftnfloat *vvllin, ftnfloat *latin, ftnfloat *lonin, wordint *npts)
{
   wordint icode;
   
   icode = c_gduvfwd(*gdid, uugdout, vvgdout, uullin, vvllin, latin, lonin, *npts);
   return icode;
}

wordint c_gduvfwd(wordint gdid,  ftnfloat *uugdout, ftnfloat *vvgdout, ftnfloat *uullin, ftnfloat *vvllin,
              ftnfloat *latin, ftnfloat *lonin, wordint npts)
{
   ftnfloat *xlatingf, *xloningf, *xlatingf2, *xloningf2, *uvcart, *xyz;
   wordint ni, nj, use_sincos_cache;
   ftnfloat *lat_true,*lon_true;

   ni = npts;
   nj = 1;

   memcpy(uugdout, uullin, npts*sizeof(ftnfloat));
   memcpy(vvgdout, vvllin, npts*sizeof(ftnfloat));
   
   if (iset == -1)
     {
     use_sincos_cache = NON;
     }
   else
     {
     if (gridset[iset].gemin.lat_true == NULL || gridset[iset].use_sincos_cache == NON)
       {
       use_sincos_cache = NON;
       }
     else
       {
       use_sincos_cache = OUI;
       }
     }

   use_sincos_cache = NON;
   switch (Grille[gdid].grtyp)
      {
      case 'E':
        if (use_sincos_cache == NON)
	  {
	  lat_true=(ftnfloat *)(malloc(npts*sizeof(ftnfloat)));
	  lon_true=(ftnfloat *)(malloc(npts*sizeof(ftnfloat)));
	  f77name(ez_gfxyfll)(lon_true,lat_true,lonin,latin,&ni,
			      &Grille[gdid].xg[XLAT1],&Grille[gdid].xg[XLON1],
			      &Grille[gdid].xg[XLAT2],&Grille[gdid].xg[XLON2]);
	  
	  c_ezgfwfllw(uugdout,vvgdout,latin,lonin,lat_true,lon_true,
		      &ni,&nj,&Grille[gdid].grtyp,
		      &Grille[gdid].ig[IG1],&Grille[gdid].ig[IG2],
		      &Grille[gdid].ig[IG3],&Grille[gdid].ig[IG4]);
	  free(lat_true);
	  free(lon_true);
	  return 0;
	  }
	else
	  {
	  /*	  c_ezgfwfllw(uugdout,vvgdout, latin, lonin,
		  gridset[iset].gemout.lat_rot, gridset[iset].gemout.lon_rot,
		  &ni,&nj,&Grille[gdid].grtyp,
		  &Grille[gdid].ig[IG1],&Grille[gdid].ig[IG2],
		  &Grille[gdid].ig[IG3],&Grille[gdid].ig[IG4]);
	  */
	  uvcart = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
	  xyz    = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
	  c_gfwfllw_turbo(uugdout,vvgdout, latin, lonin,
			  gridset[iset].gemout.lat_rot, gridset[iset].gemout.lon_rot, uvcart, xyz,
			  gridset[iset].gemout.sinlat_rot, gridset[iset].gemout.sinlon_rot, gridset[iset].gemout.coslat_rot, gridset[iset].gemout.coslon_rot,
			  gridset[iset].gemout.sinlat_true, gridset[iset].gemout.sinlon_true, gridset[iset].gemout.coslat_true, gridset[iset].gemout.coslon_true,
			  gridset[iset].gemout.r, gridset[iset].gemout.ri,
			  &ni,&nj, &Grille[gdid].grtyp, &Grille[gdid].ig[IG1],&Grille[gdid].ig[IG2], &Grille[gdid].ig[IG3],&Grille[gdid].ig[IG4]);
	  
	  free(uvcart);
	  free(xyz);
	  }
        break;
        
        
      case '#':
      case 'Y':
      case 'Z':
        switch(Grille[gdid].grref)
	  {
	  case 'E':
	    if (use_sincos_cache == NON)
	      {
	      lat_true=(ftnfloat *)(malloc(npts*sizeof(ftnfloat)));
	      lon_true=(ftnfloat *)(malloc(npts*sizeof(ftnfloat)));
	      f77name(ez_gfxyfll)(lonin,latin,lon_true,lat_true,&ni,
				  &Grille[gdid].xgref[XLAT1],&Grille[gdid].xgref[XLON1],
				  &Grille[gdid].xgref[XLAT2],&Grille[gdid].xgref[XLON2]);
	      
	      c_ezgfwfllw(uugdout,vvgdout,latin,lonin,lat_true,lon_true,
			  &ni,&nj,&Grille[gdid].grref,
			  &Grille[gdid].igref[IG1],&Grille[gdid].igref[IG2],
			  &Grille[gdid].igref[IG3],&Grille[gdid].igref[IG4]);
	      free(lat_true);
	      free(lon_true);
	      return 0;
	      }
	    else
	      {
	      /*	      c_ezgfwfllw(uugdout,vvgdout, latin, lonin,
			      gridset[iset].gemout.lat_rot, gridset[iset].gemout.lon_rot, 
			      &ni,&nj, &Grille[gdid].grref,
			      &Grille[gdid].igref[IG1],&Grille[gdid].igref[IG2],
			      &Grille[gdid].igref[IG3],&Grille[gdid].igref[IG4]);
	      */

	      uvcart = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
	      xyz    = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
	      c_gfwfllw_turbo(uugdout,vvgdout, latin, lonin,
			      gridset[iset].gemout.lat_rot, gridset[iset].gemout.lon_rot, uvcart, xyz,
			      gridset[iset].gemout.sinlat_rot, gridset[iset].gemout.sinlon_rot, gridset[iset].gemout.coslat_rot, gridset[iset].gemout.coslon_rot,
			      gridset[iset].gemout.sinlat_true, gridset[iset].gemout.sinlon_true, gridset[iset].gemout.coslat_true, gridset[iset].gemout.coslon_true,
			      gridset[iset].gemout.r, gridset[iset].gemout.ri, &ni,&nj,
			      &Grille[gdid].grref,&Grille[gdid].igref[IG1],&Grille[gdid].igref[IG2], &Grille[gdid].igref[IG3],&Grille[gdid].igref[IG4]);
	      free(uvcart);
	      free(xyz);
	      }
	    
	    break;
	    
	  default:
	    f77name(ez_gdwfllw)(uugdout,vvgdout,lonin,&ni,&nj,&Grille[gdid].grref,
				&Grille[gdid].igref[IG1],&Grille[gdid].igref[IG2],
				&Grille[gdid].igref[IG3],&Grille[gdid].igref[IG4], 1);
	    break;
	  }
        break;
        
      default:
        f77name(ez_gdwfllw)(uugdout,vvgdout,lonin,&ni,&nj,&Grille[gdid].grtyp,
			    &Grille[gdid].ig[IG1],&Grille[gdid].ig[IG2],
			    &Grille[gdid].ig[IG3],&Grille[gdid].ig[IG4], 1);
	break;
      }
   
   return 0;
}

/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdwdfuv)(wordint *gdid, ftnfloat *spd_out, ftnfloat *wd_out, ftnfloat *uuin, ftnfloat *vvin,
                     ftnfloat *latin, ftnfloat *lonin, wordint *npts)
{
   wordint icode;
   
   icode = c_gdwdfuv(*gdid, spd_out, wd_out, uuin, vvin,latin, lonin, *npts);
   return icode;
}

wordint c_gdwdfuv(wordint gdid, ftnfloat *spd_out, ftnfloat *wd_out, ftnfloat *uuin, ftnfloat *vvin, 
              ftnfloat *latin, ftnfloat *lonin, wordint npts)
{
   ftnfloat *xlatingf, *xloningf, *uvcart, *xyz;
   wordint ni, nj, use_sincos_cache;
   ftnfloat *lat_rot, *lon_rot;

   ni = npts;
   nj = 1;

   memcpy(spd_out, uuin, npts*sizeof(ftnfloat));
   memcpy(wd_out, vvin, npts*sizeof(ftnfloat));

   if (iset == -1)
     {
     use_sincos_cache = NON;
     }
   else
     {
     if (gridset[iset].gemin.lat_true == NULL || gridset[iset].use_sincos_cache == NON)
       {
       use_sincos_cache = NON;
       }
     else
       {
       use_sincos_cache = OUI;
       }
     }


   use_sincos_cache = NON;
   switch (Grille[gdid].grtyp)
     {
     case 'E':
       if (use_sincos_cache == NON)
	 {
	 lat_rot=(ftnfloat *)(malloc(npts*sizeof(ftnfloat)));
	 lon_rot=(ftnfloat *)(malloc(npts*sizeof(ftnfloat)));
	 f77name(ez_gfxyfll)(lonin, latin, lon_rot,lat_rot, &ni,
			     &Grille[gdid].xg[XLAT1],&Grille[gdid].xg[XLON1],
			     &Grille[gdid].xg[XLAT2],&Grille[gdid].xg[XLON2]);
	 
	 c_ezllwfgfw(spd_out,wd_out, latin,lonin, lat_rot,lon_rot,
		     &ni,&nj,&Grille[gdid].grtyp,
		     &Grille[gdid].ig[IG1],&Grille[gdid].ig[IG2],
		     &Grille[gdid].ig[IG3],&Grille[gdid].ig[IG4]);
	 free(lat_rot);
	 free(lon_rot);
	 return 0;
	 }
       else
	 {
	 uvcart = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
	 xyz    = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
	 /*	 c_ezllwfgfw(spd_out,wd_out, latin, lonin,
		 gridset[iset].gemin.lat_rot, gridset[iset].gemin.lon_rot, 
		 &ni,&nj,&Grille[gdid].grtyp,
		 &Grille[gdid].ig[IG1],&Grille[gdid].ig[IG2],
		 &Grille[gdid].ig[IG3],&Grille[gdid].ig[IG4]);
	 */
	 uvcart = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
	 xyz    = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
	 c_llwfgfw_turbo(spd_out,wd_out, latin, lonin,
			 gridset[iset].gemin.lat_rot, gridset[iset].gemin.lon_rot, uvcart, xyz,
			 gridset[iset].gemin.sinlat_true, gridset[iset].gemin.sinlon_true, gridset[iset].gemin.coslat_true, gridset[iset].gemin.coslon_true,
			 gridset[iset].gemin.sinlat_rot, gridset[iset].gemin.sinlon_rot, gridset[iset].gemin.coslat_rot, gridset[iset].gemin.coslon_rot,
			 gridset[iset].gemin.r, gridset[iset].gemin.ri, &ni,&nj,
			 &Grille[gdid].grref, &Grille[gdid].igref[IG1],&Grille[gdid].igref[IG2], &Grille[gdid].igref[IG3],&Grille[gdid].igref[IG4]);
	 free(uvcart);
	 free(xyz);
	 }
       break;
       
     case '#':
     case 'Y':
     case 'Z':
       switch(Grille[gdid].grref)
	 {
	 case 'E':
	   if (use_sincos_cache == NON)
	     {
	     lat_rot=(ftnfloat *)(malloc(npts*sizeof(ftnfloat)));
	     lon_rot=(ftnfloat *)(malloc(npts*sizeof(ftnfloat)));
	     f77name(ez_gfxyfll)(lonin,latin,lon_rot, lat_rot, &ni,
				 &Grille[gdid].xgref[XLAT1],&Grille[gdid].xgref[XLON1],
				 &Grille[gdid].xgref[XLAT2],&Grille[gdid].xgref[XLON2]);
	     
	     c_ezllwfgfw(spd_out,wd_out,latin,lonin,lat_rot, lon_rot, 
			 &ni,&nj,&Grille[gdid].grref,
			 &Grille[gdid].igref[IG1],&Grille[gdid].igref[IG2],
			 &Grille[gdid].igref[IG3],&Grille[gdid].igref[IG4]);
	     free(lat_rot);
	     free(lon_rot);
	     return 0;
	     }
	   else
	     {
	     uvcart = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
	     xyz    = (ftnfloat *) malloc(3*npts*sizeof(ftnfloat));
	     c_llwfgfw_turbo(spd_out,wd_out, latin, lonin,
			     gridset[iset].gemin.lat_rot, gridset[iset].gemin.lon_rot, uvcart, xyz,
			     gridset[iset].gemin.sinlat_rot, gridset[iset].gemin.sinlon_rot, gridset[iset].gemin.coslat_rot, gridset[iset].gemin.coslon_rot,
			     gridset[iset].gemin.sinlat_true, gridset[iset].gemin.sinlon_true, gridset[iset].gemin.coslat_true, gridset[iset].gemin.coslon_true,
			     gridset[iset].gemin.r, gridset[iset].gemin.ri, &ni,&nj,
			     &Grille[gdid].grref, &Grille[gdid].igref[IG1],&Grille[gdid].igref[IG2], &Grille[gdid].igref[IG3],&Grille[gdid].igref[IG4]);
	     free(uvcart);
	     free(xyz);
	     /*	     c_ezllwfgfw(spd_out,wd_out, latin, lonin,
		     gridset[iset].gemin.lat_rot, gridset[iset].gemin.lon_rot, 
		     &ni,&nj,&Grille[gdid].grref,
		     &Grille[gdid].igref[IG1],&Grille[gdid].igref[IG2],
		     &Grille[gdid].igref[IG3],&Grille[gdid].igref[IG4]);
	     */
	     }
	   break;
	   

	 default:
	   f77name(ez_llwfgdw)(spd_out,wd_out,lonin,
			       &ni,&nj,
			       &Grille[gdid].grref,
			       &Grille[gdid].igref[IG1],&Grille[gdid].igref[IG2],
			       &Grille[gdid].igref[IG3],&Grille[gdid].igref[IG4]);
	   break;
	 }
       break;
       
     default:
       f77name(ez_llwfgdw)(spd_out,wd_out,lonin,
			   &ni,&nj,
			   &Grille[gdid].grtyp,
			   &Grille[gdid].ig[IG1],&Grille[gdid].ig[IG2],
			   &Grille[gdid].ig[IG3],&Grille[gdid].ig[IG4]);
       break;
     }
   
   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdxpncf)(wordint *gdin, wordint *i1, wordint *i2, wordint *j1, wordint *j2)
{
   wordint icode;
   
   icode = c_gdxpncf(*gdin, i1, i2, j1, j2);
   return icode;
}

wordint c_gdxpncf(wordint gdin, wordint *i1, wordint *i2, wordint *j1, wordint *j2)
{
  if (gdin < 0 || gdin >= nGrilles) return -1;
  
  *i1 = Grille[gdin].i1;
  *i2 = Grille[gdin].i2;
  *j1 = Grille[gdin].j1;
  *j2 = Grille[gdin].j2;
  return 0;
}
    
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


wordint c_gdxysint(ftnfloat *zout, ftnfloat *zin, wordint gdin, ftnfloat *x, ftnfloat *y, wordint npts);
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdxysint)(ftnfloat *zout, ftnfloat *zin, wordint *gdin, ftnfloat *x, ftnfloat *y, wordint *npts)
{
   wordint icode;
   
   icode = c_gdxysint(zout, zin, *gdin, x, y, *npts);
   return icode;
}

wordint c_gdxysint(ftnfloat *zout, ftnfloat *zin, wordint gdin, ftnfloat *x, ftnfloat *y, wordint npts)
{
   wordint ier;
   ftnfloat *lzin, *lxzin;
  _zone zones[NZONES];
  wordint gdout;
  _gridset tmpset;

   lzin  = NULL;
   lxzin = NULL;

   if (Grille[gdin].axe_y_inverse == 1)
      {
      lzin = (ftnfloat *) malloc(Grille[gdin].ni*Grille[gdin].nj*sizeof(ftnfloat));
      memcpy(lzin, zin, Grille[gdin].ni*Grille[gdin].nj*sizeof(ftnfloat));
      f77name(permut)(lzin, &Grille[gdin].ni, &Grille[gdin].nj);
      }
   else
     {
     lzin = zin;
     }

   if (Grille[gdin].needs_expansion == OUI)
     {
     lxzin = (ftnfloat *) malloc(2*Grille[gdin].ni*Grille[gdin].nj*sizeof(ftnfloat));
     ez_xpnsrcgd(gdin, lxzin, lzin);
     }
   else
     {
     lxzin = lzin;
     }

   ier = c_gdinterp(zout, lxzin, gdin, x, y, npts);

   gdout = NMAXGRIDS-1;
   tmpset.gdin = gdin;
   tmpset.gdout = gdout;
   tmpset.x = x;
   tmpset.y = y;

   Grille[gdout].ni = npts;
   Grille[gdout].nj = 1;
   Grille[gdout].grtyp = 'L';


   if (groptions.polar_correction == OUI && groptions.vecteur != VECTEUR)
     {
     ier = ez_defzones(&tmpset);
     ier = ez_corrval(zout, lxzin, &tmpset);
     }
   
   
   if (lzin != zin && lzin != NULL)
      {
      free(lzin);
      }

   if (lxzin != lzin && lxzin != zin && lxzin != NULL)
      {
      free(lxzin);
      }
   
   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdxyfll)(wordint *gdid, ftnfloat *x, ftnfloat *y, ftnfloat *lat, ftnfloat *lon, wordint *n)
{
  return c_gdxyfll(*gdid, x, y, lat, lon, *n);
}

wordint c_gdxyfll(wordint gdid, ftnfloat *x, ftnfloat *y, ftnfloat *lat, ftnfloat *lon, wordint n)
{
  ftnfloat *tmplons;
  
  wordint ni_in, nj_in;
  wordint sym=groptions.symmetrie;
  
  
  _Grille gr;
  wordint npts;
  wordint coordonnee;
  
  gr =  Grille[gdid];
  npts = n;
  
  ni_in =  gr.ni;
  nj_in =  gr.nj;

  switch(gr.grtyp)
    {
    case 'A':
    case 'B':
    case 'E':
    case 'L':
    case 'N':
    case 'S':
    case 'T':
    case '!':
      tmplons = (ftnfloat *)malloc(npts * sizeof(ftnfloat));
      memcpy(tmplons,lon,sizeof(ftnfloat)*npts);
      
      f77name(ez_ll2rgd)(x, y,
			 lat, tmplons, &npts,
			 &ni_in, &nj_in, &gr.grtyp,
			 &gr.ig[IG1], &gr.ig[IG2], &gr.ig[IG3], &gr.ig[IG4],
			 &sym, gr.ay);
      free(tmplons);
      break;

    case '#':
    case 'Z':
    case 'G':
      coordonnee = RELATIF;
      nj_in =  gr.j2;
      f77name(ez_ll2igd)(x, y, lat, lon, &npts,
			 &ni_in,&nj_in,&gr.grtyp, &gr.grref,
			 &gr.igref[IG1], &gr.igref[IG2], 
			 &gr.igref[IG3], &gr.igref[IG4],
			 gr.ax, gr.ay,&coordonnee);
      break;
      
      
    default:
      break;
    }
  
  
  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

void ez_freezones(_gridset *gdset);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdxysval)(wordint *gdin, ftnfloat *zout, ftnfloat *zin, ftnfloat *x, ftnfloat *y, wordint *n)
{
   wordint icode;
   
   icode = c_gdxysval(*gdin, zout, zin, x, y, *n);
   return icode;
}

wordint c_gdxysval(wordint gdin, ftnfloat *zout, ftnfloat *zin, ftnfloat *x, ftnfloat *y, wordint n)
{
   wordint ninj_out;
   wordint ier;
   wordint local_gdout,local_iset;
   
   ninj_out = n;

   ier = c_gdxysint(zout, zin, gdin, x, y, n);

   return 0;
}


void ez_freezones(_gridset *gdset)
{
  wordint i;

  for (i=0; i < NZONES; i++)
    {
    if (gdset->zones[i].npts > 0)
      {
      free(gdset->zones[i].idx);
      free(gdset->zones[i].x);
      free(gdset->zones[i].y);
      gdset->zones[i].npts = 0;
      gdset->zones[i].idx  = NULL;
      gdset->zones[i].x  = NULL;
      gdset->zones[i].y = NULL;
      }
    }

}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdxyvval)(wordint *gdin, ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin, ftnfloat *x, ftnfloat *y, wordint *n)
{
   wordint icode;
   
   icode = c_gdxyvval(*gdin, uuout, vvout, uuin, vvin, x, y, *n);
   return icode;
}

wordint c_gdxyvval(wordint gdin, ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin, ftnfloat *x, ftnfloat *y, wordint n)
{
  wordint ninj_out;
  wordint local_gdout,local_iset, ier;
  wordint gdout;
  _gridset tmpset;
  
  groptions.vecteur = VECTEUR;
  
  memset(&tmpset, (int) NULL, sizeof(_gridset));
  gdout = NMAXGRIDS-1;
  tmpset.gdin = gdin;
  tmpset.gdout = gdout;
  tmpset.x = x;
  tmpset.y = y;
  
  Grille[gdout].ni = n;
  Grille[gdout].nj = 1;
  Grille[gdout].grtyp = 'L';
  
  groptions.symmetrie = SYM;
  c_gdxysint(uuout,uuin, gdin, x, y, n);
  groptions.symmetrie = ANTISYM;
  c_gdxysint(vvout,vvin, gdin, x, y, n);
  groptions.symmetrie = SYM;
  
 if (groptions.polar_correction == OUI)
      {
      tmpset.gdin = gdin;
      ier = ez_defzones(&tmpset);
      ez_corrvec(uuout, vvout, uuin, vvin, &tmpset);
      ez_freezones(&tmpset);
      }

 groptions.vecteur = SCALAIRE;
  
  return 0;
  
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdxywdval)(wordint *gdin, ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin, ftnfloat *x, ftnfloat *y, wordint *n)
{
   wordint icode;
   
   icode = c_gdxywdval(*gdin, uuout, vvout, uuin, vvin, x, y, *n);
   return icode;

}

wordint c_gdxywdval(wordint gdin, ftnfloat *uuout, ftnfloat *vvout, ftnfloat *uuin, ftnfloat *vvin, ftnfloat *x, ftnfloat *y, wordint n)
{
  wordint ier;

  ftnfloat *tmplat, *tmplon;
  
  tmplat = (ftnfloat *) malloc(n * sizeof(ftnfloat));
  tmplon = (ftnfloat *) malloc(n * sizeof(ftnfloat));
  
  ier = c_gdxyvval(gdin, uuout, vvout, uuin, vvin, x, y, n);
  ier = c_gdllfxy(gdin, tmplat, tmplon, x, y, n);
  ier = c_gdwdfuv(gdin, uuout, vvout, uuout, vvout, tmplat, tmplon, n);
  
  free(tmplat);
  free(tmplon);


  return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdxyzfll)(wordint *gdid, ftnfloat *x, ftnfloat *y, ftnfloat *lat, ftnfloat *lon, wordint *n)
{
   return c_gdxyzfll(*gdid, x, y, lat, lon, *n);
}

wordint c_gdxyzfll(wordint gdid, ftnfloat *x, ftnfloat *y, ftnfloat *lat, ftnfloat *lon, wordint n)
{
   wordint ni_in, nj_in;
   
   wordint coordonnee;

   _Grille grEntree;
   wordint npts;

   grEntree =  Grille[gdid];
   npts = n;
              
   ni_in =  grEntree.ni;
   nj_in =  grEntree.nj;

   
   switch(grEntree.grtyp)
      {
      case 'A':
      case 'B':
      case 'E':
      case 'G':
      case 'L':
      case 'N':
      case 'S':
      case 'T':
      case '!':
	c_gdxyfll(gdid, x, y, lat, lon, n);
        break;
        
      case 'Y':
	fprintf(stderr, "********************************************************\n");
	fprintf(stderr, "<gdllfxy>: This operation is not supported for 'Y' grids\n");
	fprintf(stderr, "********************************************************\n");
	break;
	
      case '#':
      case 'Z':
	coordonnee = ABSOLU;
        f77name(ez_ll2igd)(x, y, lat, lon, &npts,
			    &ni_in,&nj_in,&grEntree.grtyp, &grEntree.grref,
			    &grEntree.igref[IG1], &grEntree.igref[IG2], 
			    &grEntree.igref[IG3], &grEntree.igref[IG4],
			    grEntree.ax, grEntree.ay, &coordonnee);
        break;
        
        
      default:
        break;
      }


   return 0;
}
/* RMNLIB - Library of useful routines for C and FORTRAN programming
 * Copyright (C) 1975-2001  Division de Recherche en Prevision Numerique
 *                          Environnement Canada
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation,
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "ezscint.h"
#include "ez_funcdef.h"

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
wordint f77name(gdllfxyz)(wordint *gdid, ftnfloat *lat, ftnfloat *lon, ftnfloat *x, ftnfloat *y, wordint *n)
{
  return c_gdllfxyz(*gdid, lat, lon, x, y, *n);
}

wordint c_gdllfxyz(wordint gdid, ftnfloat *lat, ftnfloat *lon, ftnfloat *x, ftnfloat *y, wordint n)
{
  ftnfloat xlat00, xlon00, dlat, dlon;
  ftnfloat xlat1, xlon1, xlat2, xlon2;
  ftnfloat pi, pj, d60, dgrw;
  wordint i,j,ni, nj, npts, hem, un;
  
  _Grille grEntree;
  
  grEntree = Grille[gdid];
  npts = n;
  
  switch(grEntree.grtyp)
    {
    case 'A':
    case 'B':
    case 'G':
    case 'L':
    case 'N':
    case 'S':
    case 'T':
    case '!':
      c_gdllfxy(gdid, lat, lon, x, y, n);
      break;
      
    case 'Y':
      fprintf(stderr, "********************************************************\n");
      fprintf(stderr, "<gdllfxy>: This operation is not supported for 'Y' grids\n");
      fprintf(stderr, "********************************************************\n");
      break;
      
    case '#':
    case 'Z':
      switch (grEntree.grref)
	{
	case 'E':
	  f77name(ez_gfllfxy)(lon,lat,x,y,&npts,&grEntree.xgref[XLAT1],&grEntree.xgref[XLON1],
			      &grEntree.xgref[XLAT2],&grEntree.xgref[XLON2]);
	  break;
	  
	case 'S':
	case 'N':
	  if (grEntree.grref == 'N') 
	    hem = 1;
	  else
	    hem = 2;

	  un = 1;
	  f77name(ez_vllfxy)(lat,lon,x,y,&npts,&un,&grEntree.xgref[D60],
			  &grEntree.xgref[DGRW], &grEntree.xgref[PI], &grEntree.xgref[PJ],&grEntree.hemisphere);
	  break;
	  
	case 'L':
	  for (i=0; i < n; i++)
	    {
	      lat[i] = (x[i])*grEntree.xgref[DLAT]+ grEntree.xgref[SWLAT];
	      lon[i] = (y[i])*grEntree.xgref[DLON]+ grEntree.xgref[SWLON];
	      lon[i] = lon[i] < 0.0 ? lon[i] + 360.0 : lon[i];
	    }
	  break;
	  
	default:
	  fprintf(stderr,"<gdllfxy> Errrrrrrrrrrreur!\n");
	  break;
	}
      break;
    }
  
  return 0;
  
}
